@@ -487,9 +487,9 @@ void thread_sleep(int microseconds);
 	Parameters:
 		threadfunc - Entry point for the new thread.
 		user - Pointer to pass to the thread.
-
+		name - name describing the use of the thread
 */
-void *thread_init(void (*threadfunc)(void *), void *user);
+void *thread_init(void (*threadfunc)(void *), void *user, const char *name);
 
 /*
 	Function: thread_wait
@@ -517,6 +517,20 @@ void thread_yield();
 */
 void thread_detach(void *thread);
 
+/*
+	Function: thread_init_and_detach
+		Creates a new thread and if it succeeded detaches it.
+
+	Parameters:
+		threadfunc - Entry point for the new thread.
+		user - Pointer to pass to the thread.
+		name - name describing the use of the thread
+
+	Returns:
+		Returns the thread if no error occured, 0 on error.
+*/
+void *thread_init_and_detach(void (*threadfunc)(void *), void *user, const char *name);
+
 /* Group: Locks */
 typedef void* LOCK;
 
@@ -982,6 +996,23 @@ void str_append(char *dst, const char *src, int dst_size);
 */
 void str_copy(char *dst, const char *src, int dst_size);
 
+/*
+	Function: str_truncate
+		Truncates a string to a given length.
+
+	Parameters:
+		dst - Pointer to a buffer that shall receive the string.
+		dst_size - Size of the buffer dst.
+		str - String to be truncated.
+		truncation_len - Maximum length of the returned string (not
+		counting the zero termination).
+
+	Remarks:
+		- The strings are treated as zero-terminated strings.
+		- Guarantees that dst string will contain zero-termination.
+*/
+void str_truncate(char *dst, int dst_size, const char *src, int truncation_len);
+
 /*
 	Function: str_length
 		Returns the length of a zero terminated string.
@@ -1005,7 +1036,7 @@ int str_length(const char *str);
 		... - Parameters for the formatting.
 
 	Returns:
-		Length of written string
+		Length of written string, even if it has been truncated
 
 	Remarks:
 		- See the C manual for syntax for the printf formatting string.
@@ -1309,7 +1340,7 @@ int str_utf32_dist_buffer(const int *a, int a_len, const int *b, int b_len, int
 
 	Returns:
 		A pointer into haystack where the needle was found.
-		Returns NULL of needle could not be found.
+		Returns NULL if needle could not be found.
 
 	Remarks:
 		- Only guaranteed to work with a-z/A-Z.
@@ -1328,13 +1359,30 @@ const char *str_find_nocase(const char *haystack, const char *needle);
 
 	Returns:
 		A pointer into haystack where the needle was found.
-		Returns NULL of needle could not be found.
+		Returns NULL if needle could not be found.
 
 	Remarks:
 		- The strings are treated as zero-terminated strings.
 */
 const char *str_find(const char *haystack, const char *needle);
 
+/*
+	Function: str_rchr
+		Finds the last occurance of a character
+
+	Parameters:
+		haystack - String to search in
+		needle - Character to search for
+
+	Returns:
+		A pointer into haystack where the needle was found.
+		Returns NULL if needle could not be found.
+
+	Remarks:
+		- The strings are treated as zero-terminated strings.
+*/
+const char *str_rchr(const char *haystack, char needle);
+
 /*
 	Function: str_hex
 		Takes a datablock and generates a hex string of it, with spaces
@@ -1609,9 +1657,11 @@ void net_stats(NETSTATS *stats);
 
 int str_toint(const char *str);
 int str_toint_base(const char *str, int base);
+unsigned long str_toulong_base(const char *str, int base);
 float str_tofloat(const char *str);
 int str_isspace(char c);
 char str_uppercase(char c);
+int str_isallnum(const char *str);
 unsigned str_quickhash(const char *str);
 
 struct SKELETON;
@@ -1686,7 +1736,7 @@ int str_utf8_comp_nocase_num(const char *a, const char *b, int num);
 
 	Returns:
 		A pointer into haystack where the needle was found.
-		Returns NULL of needle could not be found.
+		Returns NULL if needle could not be found.
 
 	Remarks:
 		- The strings are treated as zero-terminated strings.
@@ -1831,6 +1881,21 @@ int str_utf16le_encode(char *ptr, int chr);
 */
 int str_utf8_check(const char *str);
 
+/*
+	Function: str_next_token
+		Writes the next token after str into buf, returns the rest of the string.
+	Parameters:
+		str - Pointer to string.
+		delim - Delimiter for tokenization.
+		buffer - Buffer to store token in.
+		buffer_size - Size of the buffer.
+	Returns:
+		Pointer to rest of the string.
+	Remarks:
+		- The token is always null-terminated.
+*/
+const char *str_next_token(const char *str, const char *delim, char *buffer, int buffer_size);
+
 /*
 	Function: str_in_list
 		Checks if needle is in list delimited by delim
