@@ -13,7 +13,8 @@ enum
 	RECORDER_MANUAL=0,
 	RECORDER_AUTO=1,
 	RECORDER_RACE=2,
-	RECORDER_MAX=3,
+	RECORDER_REPLAYS=3,
+	RECORDER_MAX=4,
 };
 
 typedef bool (*CLIENTFUNC_FILTER)(const void *pData, int DataSize, void *pUser);
@@ -38,6 +39,8 @@ protected:
 	float m_RenderFrameTime;
 
 	int m_GameTickSpeed;
+
+	float m_FrameTimeAvg;
 public:
 	char m_aNews[3000];
 	int64 m_ReconnectTime;
@@ -84,6 +87,7 @@ public:
 	// other time access
 	inline float RenderFrameTime() const { return m_RenderFrameTime; }
 	inline float LocalTime() const { return m_LocalTime; }
+	inline float FrameTimeAvg() const { return m_FrameTimeAvg; }
 
 	// actions
 	virtual void Connect(const char *pAddress, const char *pPassword = NULL) = 0;
@@ -100,7 +104,7 @@ public:
 	virtual const char *DemoPlayer_Play(const char *pFilename, int StorageType) = 0;
 	virtual void DemoRecorder_Start(const char *pFilename, bool WithTimestamp, int Recorder) = 0;
 	virtual void DemoRecorder_HandleAutoStart() = 0;
-	virtual void DemoRecorder_Stop(int Recorder) = 0;
+	virtual void DemoRecorder_Stop(int Recorder, bool RemoveFile = false) = 0;
 	virtual class IDemoRecorder *DemoRecorder(int Recorder) = 0;
 	virtual void AutoScreenshot_Start() = 0;
 	virtual void AutoStatScreenshot_Start() = 0;
@@ -112,6 +116,7 @@ public:
 	virtual void ToggleFullscreen() = 0;
 	virtual void ToggleWindowBordered() = 0;
 	virtual void ToggleWindowVSync() = 0;
+	virtual void LoadFont() = 0;
 
 	// networking
 	virtual void EnterGame() = 0;
@@ -123,7 +128,7 @@ public:
 
 	// input
 	virtual int *GetInput(int Tick) = 0;
-	virtual bool InputExists(int Tick) = 0;
+	virtual int *GetDirectInput(int Tick) = 0;
 
 	// remote console
 	virtual void RconAuth(const char *pUsername, const char *pPassword) = 0;
@@ -179,6 +184,8 @@ public:
 	virtual const char *GetCurrentMapPath() = 0;
 	virtual unsigned GetMapCrc() = 0;
 
+	virtual int GetCurrentRaceTime() = 0;
+
 	virtual void RaceRecord_Start(const char *pFilename) = 0;
 	virtual void RaceRecord_Stop() = 0;
 	virtual bool RaceRecord_IsRecording() = 0;
@@ -193,6 +200,9 @@ public:
 	virtual void GenerateTimeoutSeed() = 0;
 
 	virtual IFriends* Foes() = 0;
+
+	virtual void GetSmoothTick(int *pSmoothTick, float *pSmoothIntraTick, float MixAmount) = 0;
+
 };
 
 class IGameClient : public IInterface
@@ -204,7 +214,6 @@ public:
 
 	virtual void OnRconType(bool UsernameReq) = 0;
 	virtual void OnRconLine(const char *pLine) = 0;
-	virtual void OnTimeScore(int AllowTimeScore, bool Dummy) = 0;
 	virtual void OnInit() = 0;
 	virtual void OnNewSnapshot() = 0;
 	virtual void OnEnterGame() = 0;
@@ -220,12 +229,14 @@ public:
 	virtual int OnSnapInput(int *pData, bool Dummy, bool Force) = 0;
 	virtual void OnDummySwap() = 0;
 	virtual void SendDummyInfo(bool Start) = 0;
+	virtual int GetLastRaceTick() = 0;
 
 	virtual const char *GetItemName(int Type) = 0;
 	virtual const char *Version() = 0;
 	virtual const char *NetVersion() = 0;
 
 	virtual void OnDummyDisconnect() = 0;
+	virtual void Echo(const char *pString) = 0;
 };
 
 extern IGameClient *CreateGameClient();
