@@ -55,7 +55,6 @@ public:
 	void Update(CGraph *pGraph, int64 Target, int TimeLeft, int AdjustDirection);
 };
 
-
 class CClient : public IClient, public CDemoPlayer::IListener
 {
 	// needed interfaces
@@ -90,10 +89,11 @@ class CClient : public IClient, public CDemoPlayer::IListener
 
 	char m_aServerAddressStr[256];
 
-	unsigned m_SnapshotParts;
+	unsigned m_SnapshotParts[2];
 	int64 m_LocalStartTime;
 
 	int m_DebugFont;
+	int m_DebugSoundIndex = 0;
 
 	int64 m_LastRenderTime;
 	float m_RenderFrameTimeLow;
@@ -123,8 +123,8 @@ class CClient : public IClient, public CDemoPlayer::IListener
 	// pinging
 	int64 m_PingStartTime;
 
-	char m_aCurrentMap[256];
-	char m_aCurrentMapPath[CEditor::MAX_PATH_LENGTH];
+	char m_aCurrentMap[MAX_PATH_LENGTH];
+	char m_aCurrentMapPath[MAX_PATH_LENGTH];
 
 	char m_aTimeoutCodes[2][32];
 	bool m_aTimeoutCodeSent[2];
@@ -132,6 +132,7 @@ class CClient : public IClient, public CDemoPlayer::IListener
 
 	//
 	char m_aCmdConnect[256];
+	char m_aCmdPlayDemo[MAX_PATH_LENGTH];
 
 	// map download
 	std::shared_ptr<CGetFile> m_pMapdownloadTask;
@@ -187,6 +188,8 @@ class CClient : public IClient, public CDemoPlayer::IListener
 
 	class CSnapshotDelta m_SnapshotDelta;
 
+	std::list<std::shared_ptr<CDemoEdit>> m_EditJobs;
+
 	//
 	class CServerInfo m_CurrentServerInfo;
 	int64 m_CurrentServerInfoRequestTime; // >= 0 should request, == -1 got info
@@ -208,7 +211,6 @@ class CClient : public IClient, public CDemoPlayer::IListener
 	volatile int m_GfxState;
 	static void GraphicsThreadProxy(void *pThis) { ((CClient*)pThis)->GraphicsThread(); }
 	void GraphicsThread();
-	vec3 GetColorV3(int v);
 
 #if defined(CONF_FAMILY_UNIX)
 	CFifo m_Fifo;
@@ -252,7 +254,7 @@ public:
 
 	// TODO: OPT: do this a lot smarter!
 	virtual int *GetInput(int Tick);
-	virtual bool InputExists(int Tick);
+	virtual int *GetDirectInput(int Tick);
 
 	const char *LatestVersion();
 
@@ -360,17 +362,20 @@ public:
 	static void ConchainWindowVSync(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData);
 	static void ConchainTimeoutSeed(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData);
 	static void ConchainPassword(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData);
-	
+	static void ConchainReplays(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData);
+
 	static void Con_DemoSlice(IConsole::IResult *pResult, void *pUserData);
 	static void Con_DemoSliceBegin(IConsole::IResult *pResult, void *pUserData);
 	static void Con_DemoSliceEnd(IConsole::IResult *pResult, void *pUserData);
+	static void Con_SaveReplay(IConsole::IResult *pResult, void *pUserData);
 
 	void RegisterCommands();
 
 	const char *DemoPlayer_Play(const char *pFilename, int StorageType);
 	void DemoRecorder_Start(const char *pFilename, bool WithTimestamp, int Recorder);
 	void DemoRecorder_HandleAutoStart();
-	void DemoRecorder_Stop(int Recorder);
+	void DemoRecorder_StartReplayRecorder();
+	void DemoRecorder_Stop(int Recorder, bool RemoveFile = false);
 	void DemoRecorder_AddDemoMarker(int Recorder);
 	class IDemoRecorder *DemoRecorder(int Recorder);
 
@@ -385,18 +390,22 @@ public:
 	void ServerBrowserUpdate();
 
 	void HandleConnectLink(const char *pLink);
+	void HandleDemoPath(const char *pPath);
 
 	// gfx
 	void SwitchWindowScreen(int Index);
 	void ToggleFullscreen();
 	void ToggleWindowBordered();
 	void ToggleWindowVSync();
+	void LoadFont();
 
 	// DDRace
 
 	void GenerateTimeoutSeed();
 	void GenerateTimeoutCodes();
 
+	virtual int GetCurrentRaceTime();
+
 	const char *GetCurrentMap();
 	const char *GetCurrentMapPath();
 	unsigned GetMapCrc();
@@ -408,9 +417,13 @@ public:
 	virtual void DemoSliceBegin();
 	virtual void DemoSliceEnd();
 	virtual void DemoSlice(const char *pDstPath, CLIENTFUNC_FILTER pfnFilter, void *pUser);
+	virtual void SaveReplay(const int Length);
 
 	bool EditorHasUnsavedData() { return m_pEditor->HasUnsavedData(); }
 
 	virtual IFriends* Foes() {return &m_Foes; }
+
+	void GetSmoothTick(int *pSmoothTick, float *pSmoothIntraTick, float MixAmount);
 };
+
 #endif
