@@ -2,10 +2,10 @@
 /* If you are missing that file, acquire a complete release at teeworlds.com.                */
 #include <algorithm> // sort  TODO: remove this
 
+#include <base/hash_ctxt.h>
 #include <base/math.h>
 #include <base/system.h>
 
-#include <engine/external/md5/md5.h>
 #include <engine/shared/config.h>
 #include <engine/shared/memheap.h>
 #include <engine/shared/network.h>
@@ -92,15 +92,12 @@ const CServerInfo *CServerBrowser::SortedGet(int Index) const
 
 int CServerBrowser::GenerateToken(const NETADDR &Addr) const
 {
-	md5_state_t Md5;
-	md5_byte_t aDigest[16];
-
-	md5_init(&Md5);
-	md5_append(&Md5, m_aTokenSeed, sizeof(m_aTokenSeed));
-	md5_append(&Md5, (unsigned char *)&Addr, sizeof(Addr));
-	md5_finish(&Md5, aDigest);
-
-	return (aDigest[0] << 16) | (aDigest[1] << 8) | aDigest[2];
+	SHA256_CTX Sha256;
+	sha256_init(&Sha256);
+	sha256_update(&Sha256, m_aTokenSeed, sizeof(m_aTokenSeed));
+	sha256_update(&Sha256, (unsigned char *)&Addr, sizeof(Addr));
+	SHA256_DIGEST Digest = sha256_finish(&Sha256);
+	return (Digest.data[0] << 16) | (Digest.data[1] << 8) | Digest.data[2];
 }
 
 int CServerBrowser::GetBasicToken(int Token)
@@ -217,7 +214,7 @@ void CServerBrowser::Filter()
 			Filtered = 1;
 		else if(!g_Config.m_BrFilterGametypeStrict && g_Config.m_BrFilterGametype[0] && !str_find_nocase(m_ppServerlist[i]->m_Info.m_aGameType, g_Config.m_BrFilterGametype))
 			Filtered = 1;
-		else if(g_Config.m_BrFilterUnfinishedMap && m_ppServerlist[i]->m_Info.m_HasRank != 0)
+		else if(g_Config.m_BrFilterUnfinishedMap && m_ppServerlist[i]->m_Info.m_HasRank == 1)
 			Filtered = 1;
 		else
 		{
@@ -286,7 +283,7 @@ void CServerBrowser::Filter()
 				{
 					MatchFound = 1;
 				}
-				
+
 				// match against gametype
 				if(str_find_nocase(m_ppServerlist[i]->m_Info.m_aGameType, g_Config.m_BrExcludeString))
 				{
@@ -306,7 +303,7 @@ void CServerBrowser::Filter()
 			{
 				m_ppServerlist[i]->m_Info.m_aClients[p].m_FriendState = m_pFriends->GetFriendState(m_ppServerlist[i]->m_Info.m_aClients[p].m_aName,
 					m_ppServerlist[i]->m_Info.m_aClients[p].m_aClan);
-				m_ppServerlist[i]->m_Info.m_FriendState = max(m_ppServerlist[i]->m_Info.m_FriendState, m_ppServerlist[i]->m_Info.m_aClients[p].m_FriendState);
+				m_ppServerlist[i]->m_Info.m_FriendState = maximum(m_ppServerlist[i]->m_Info.m_FriendState, m_ppServerlist[i]->m_Info.m_aClients[p].m_FriendState);
 			}
 
 			if(!g_Config.m_BrFilterFriends || m_ppServerlist[i]->m_Info.m_FriendState != IFriends::FRIEND_NO)
@@ -436,9 +433,11 @@ void CServerBrowser::QueueRequest(CServerEntry *pEntry)
 
 void CServerBrowser::SetInfo(CServerEntry *pEntry, const CServerInfo &Info)
 {
-	int Fav = pEntry->m_Info.m_Favorite;
+	bool Fav = pEntry->m_Info.m_Favorite;
+	bool Off = pEntry->m_Info.m_Official;
 	pEntry->m_Info = Info;
 	pEntry->m_Info.m_Favorite = Fav;
+	pEntry->m_Info.m_Official = Off;
 	pEntry->m_Info.m_NetAddr = pEntry->m_Addr;
 
 	// all these are just for nice compatibility
@@ -481,7 +480,27 @@ CServerBrowser::CServerEntry *CServerBrowser::Add(const NETADDR &Addr)
 	for(i = 0; i < m_NumFavoriteServers; i++)
 	{
 		if(net_addr_comp(&Addr, &m_aFavoriteServers[i]) == 0)
-			pEntry->m_Info.m_Favorite = 1;
+		{
+			pEntry->m_Info.m_Favorite = true;
+			break;
+		}
+	}
+
+	// check if it's an official server
+	for(int Network = 0; Network < NUM_NETWORKS; Network++)
+	{
+		for(int i = 0; i < m_aNetworks[Network].m_NumCountries; i++)
+		{
+			CNetworkCountry *pCntr = &m_aNetworks[Network].m_aCountries[i];
+			for(int j = 0; j < pCntr->m_NumServers; j++)
+			{
+				if(net_addr_comp(&Addr, &pCntr->m_aServers[j]) == 0)
+				{
+					pEntry->m_Info.m_Official = true;
+					break;
+				}
+			}
+		}
 	}
 
 	// add to the hash list
@@ -542,6 +561,17 @@ void CServerBrowser::Set(const NETADDR &Addr, int Type, int Token, const CServer
 			QueueRequest(pEntry);
 		}
 	}
+	else if(Type == IServerBrowser::SET_KOG_ADD)
+	{
+		if(m_ServerlistType != IServerBrowser::TYPE_KOG)
+			return;
+
+		if(!Find(Addr))
+		{
+			pEntry = Add(Addr);
+			QueueRequest(pEntry);
+		}
+	}
 	else if(Type == IServerBrowser::SET_TOKEN)
 	{
 		int BasicToken = Token;
@@ -588,10 +618,10 @@ void CServerBrowser::Set(const NETADDR &Addr, int Type, int Token, const CServer
 			}
 			SetInfo(pEntry, *pInfo);
 			if (m_ServerlistType == IServerBrowser::TYPE_LAN)
-				pEntry->m_Info.m_Latency = min(static_cast<int>((time_get()-m_BroadcastTime)*1000/time_freq()), 999);
+				pEntry->m_Info.m_Latency = minimum(static_cast<int>((time_get()-m_BroadcastTime)*1000/time_freq()), 999);
 			else if (pEntry->m_RequestTime > 0)
 			{
-				pEntry->m_Info.m_Latency = min(static_cast<int>((time_get()-pEntry->m_RequestTime)*1000/time_freq()), 999);
+				pEntry->m_Info.m_Latency = minimum(static_cast<int>((time_get()-pEntry->m_RequestTime)*1000/time_freq()), 999);
 				pEntry->m_RequestTime = -1; // Request has been answered
 			}
 			RemoveRequest(pEntry);
@@ -659,12 +689,12 @@ void CServerBrowser::Refresh(int Type)
 	else if(Type == IServerBrowser::TYPE_DDNET)
 	{
 		// remove unknown elements of exclude list
-		DDNetCountryFilterClean();
-		DDNetTypeFilterClean();
+		CountryFilterClean(NETWORK_DDNET);
+		TypeFilterClean(NETWORK_DDNET);
 
-		for(int i = 0; i < m_NumDDNetCountries; i++)
+		for(int i = 0; i < m_aNetworks[NETWORK_DDNET].m_NumCountries; i++)
 		{
-			CDDNetCountry *pCntr = &m_aDDNetCountries[i];
+			CNetworkCountry *pCntr = &m_aNetworks[NETWORK_DDNET].m_aCountries[i];
 
 			// check for filter
 			if(DDNetFiltered(g_Config.m_BrFilterExcludeCountries, pCntr->m_aName))
@@ -677,6 +707,27 @@ void CServerBrowser::Refresh(int Type)
 			}
 		}
 	}
+	else if(Type == IServerBrowser::TYPE_KOG)
+	{
+		// remove unknown elements of exclude list
+		CountryFilterClean(NETWORK_KOG);
+		TypeFilterClean(NETWORK_KOG);
+
+		for(int i = 0; i < m_aNetworks[NETWORK_KOG].m_NumCountries; i++)
+		{
+			CNetworkCountry *pCntr = &m_aNetworks[NETWORK_KOG].m_aCountries[i];
+
+			// check for filter
+			if(DDNetFiltered(g_Config.m_BrFilterExcludeCountriesKoG, pCntr->m_aName))
+				continue;
+
+			for(int g = 0; g < pCntr->m_NumServers; g++)
+			{
+				if(!DDNetFiltered(g_Config.m_BrFilterExcludeTypesKoG, pCntr->m_aTypes[g]))
+					Set(pCntr->m_aServers[g], IServerBrowser::SET_KOG_ADD, -1, 0);
+			}
+		}
+	}
 }
 
 void CServerBrowser::RequestImpl(const NETADDR &Addr, CServerEntry *pEntry) const
@@ -952,7 +1003,7 @@ void CServerBrowser::AddFavorite(const NETADDR &Addr)
 	m_aFavoriteServers[m_NumFavoriteServers++] = Addr;
 	pEntry = Find(Addr);
 	if(pEntry)
-		pEntry->m_Info.m_Favorite = 1;
+		pEntry->m_Info.m_Favorite = true;
 
 	if(g_Config.m_Debug)
 	{
@@ -978,7 +1029,7 @@ void CServerBrowser::RemoveFavorite(const NETADDR &Addr)
 
 			pEntry = Find(Addr);
 			if(pEntry)
-				pEntry->m_Info.m_Favorite = 0;
+				pEntry->m_Info.m_Favorite = false;
 
 			return;
 		}
@@ -990,82 +1041,106 @@ void CServerBrowser::LoadDDNetServers()
 	if (!m_pDDNetInfo)
 		return;
 
-	// parse JSON
-	const json_value *pServers = json_object_get(m_pDDNetInfo, "servers");
-
-	if (!pServers || pServers->type != json_array)
-		return;
-
 	// reset servers / countries
-	m_NumDDNetCountries = 0;
-	m_NumDDNetTypes = 0;
-
-	for (int i = 0; i < json_array_length(pServers) && m_NumDDNetCountries < MAX_DDNET_COUNTRIES; i++)
+	for (int Network = 0; Network < NUM_NETWORKS; Network++)
 	{
-		// pSrv - { name, flagId, servers }
-		const json_value *pSrv = json_array_get(pServers, i);
-		const json_value *pTypes = json_object_get(pSrv, "servers");
-		const json_value *pName = json_object_get(pSrv, "name");
-		const json_value *pFlagID = json_object_get(pSrv, "flagId");
+		CNetwork *pNet = &m_aNetworks[Network];
 
-		if (pSrv->type != json_object || pTypes->type != json_object || pName->type != json_string || pFlagID->type != json_integer)
+		// parse JSON
+		const json_value *pServers = json_object_get(m_pDDNetInfo, Network == NETWORK_DDNET ? "servers" : "servers-kog");
+
+		if (!pServers || pServers->type != json_array)
+			return;
+
+		pNet->m_NumCountries = 0;
+		pNet->m_NumTypes = 0;
+
+		for (int i = 0; i < json_array_length(pServers) && pNet->m_NumCountries < MAX_COUNTRIES; i++)
 		{
-			dbg_msg("client_srvbrowse", "invalid attributes");
-			continue;
-		}
+			// pSrv - { name, flagId, servers }
+			const json_value *pSrv = json_array_get(pServers, i);
+			const json_value *pTypes = json_object_get(pSrv, "servers");
+			const json_value *pName = json_object_get(pSrv, "name");
+			const json_value *pFlagID = json_object_get(pSrv, "flagId");
 
-		// build structure
-		CDDNetCountry *pCntr = &m_aDDNetCountries[m_NumDDNetCountries];
-
-		pCntr->Reset();
-
-		str_copy(pCntr->m_aName, json_string_get(pName), sizeof(pCntr->m_aName));
-		pCntr->m_FlagID = json_int_get(pFlagID);
-
-		// add country
-		for (unsigned int t = 0; t < pTypes->u.object.length; t++)
-		{
-			const char *pType = pTypes->u.object.values[t].name;
-			const json_value *pAddrs = pTypes->u.object.values[t].value;
-
-			if (pAddrs->type != json_array)
+			if (pSrv->type != json_object || pTypes->type != json_object || pName->type != json_string || pFlagID->type != json_integer)
 			{
 				dbg_msg("client_srvbrowse", "invalid attributes");
 				continue;
 			}
 
-			// add type
-			if(json_array_length(pAddrs) > 0 && m_NumDDNetTypes < MAX_DDNET_TYPES)
-			{
-				int Pos;
-				for(Pos = 0; Pos < m_NumDDNetTypes; Pos++)
-				{
-					if(!str_comp(m_aDDNetTypes[Pos], pType))
-						break;
-				}
-				if(Pos == m_NumDDNetTypes)
-				{
-					str_copy(m_aDDNetTypes[m_NumDDNetTypes], pType, sizeof(m_aDDNetTypes[m_NumDDNetTypes]));
-					m_NumDDNetTypes++;
-				}
-			}
+			// build structure
+			CNetworkCountry *pCntr = &pNet->m_aCountries[pNet->m_NumCountries];
 
-			// add addresses
-			for (int g = 0; g < json_array_length(pAddrs); g++, pCntr->m_NumServers++)
+			pCntr->Reset();
+
+			str_copy(pCntr->m_aName, json_string_get(pName), sizeof(pCntr->m_aName));
+			pCntr->m_FlagID = json_int_get(pFlagID);
+
+			// add country
+			for (unsigned int t = 0; t < pTypes->u.object.length; t++)
 			{
-				const json_value *pAddr = json_array_get(pAddrs, g);
-				if (pAddr->type != json_string)
+				const char *pType = pTypes->u.object.values[t].name;
+				const json_value *pAddrs = pTypes->u.object.values[t].value;
+
+				if (pAddrs->type != json_array)
 				{
 					dbg_msg("client_srvbrowse", "invalid attributes");
 					continue;
 				}
-				const char *pStr = json_string_get(pAddr);
-				net_addr_from_str(&pCntr->m_aServers[pCntr->m_NumServers], pStr);
-				str_copy(pCntr->m_aTypes[pCntr->m_NumServers], pType, sizeof(pCntr->m_aTypes[pCntr->m_NumServers]));
+
+				// add type
+				if(json_array_length(pAddrs) > 0 && pNet->m_NumTypes < MAX_TYPES)
+				{
+					int Pos;
+					for(Pos = 0; Pos < pNet->m_NumTypes; Pos++)
+					{
+						if(!str_comp(pNet->m_aTypes[Pos], pType))
+							break;
+					}
+					if(Pos == pNet->m_NumTypes)
+					{
+						str_copy(pNet->m_aTypes[pNet->m_NumTypes], pType, sizeof(pNet->m_aTypes[pNet->m_NumTypes]));
+						pNet->m_NumTypes++;
+					}
+				}
+
+				// add addresses
+				for (int g = 0; g < json_array_length(pAddrs); g++, pCntr->m_NumServers++)
+				{
+					const json_value *pAddr = json_array_get(pAddrs, g);
+					if (pAddr->type != json_string)
+					{
+						dbg_msg("client_srvbrowse", "invalid attributes");
+						continue;
+					}
+					const char *pStr = json_string_get(pAddr);
+					net_addr_from_str(&pCntr->m_aServers[pCntr->m_NumServers], pStr);
+					str_copy(pCntr->m_aTypes[pCntr->m_NumServers], pType, sizeof(pCntr->m_aTypes[pCntr->m_NumServers]));
+				}
+			}
+
+			pNet->m_NumCountries++;
+		}
+	}
+}
+
+void CServerBrowser::RecheckOfficial()
+{
+	for(int Network = 0; Network < NUM_NETWORKS; Network++)
+	{
+		for(int i = 0; i < m_aNetworks[Network].m_NumCountries; i++)
+		{
+			CNetworkCountry *pCntr = &m_aNetworks[Network].m_aCountries[i];
+			for(int j = 0; j < pCntr->m_NumServers; j++)
+			{
+				CServerEntry *pEntry = Find(pCntr->m_aServers[j]);
+				if(pEntry)
+				{
+					pEntry->m_Info.m_Official = true;
+				}
 			}
 		}
-
-		m_NumDDNetCountries++;
 	}
 }
 
@@ -1144,9 +1219,14 @@ const json_value *CServerBrowser::LoadDDNetInfo()
 	LoadDDNetServers();
 
 	if(m_NumServers == 0)
+	{
 		Refresh(m_ServerlistType);
+	}
 	else
+	{
+		RecheckOfficial();
 		LoadDDNetRanks();
+	}
 
 	return m_pDDNetInfo;
 }
@@ -1204,55 +1284,60 @@ void CServerBrowser::DDNetFilterRem(char *pFilter, const char *pName)
 
 	// rewrite exclude/filter list
 	char aBuf[128];
-	char *p;
 
 	str_copy(aBuf, pFilter, sizeof(aBuf));
 	pFilter[0] = '\0';
 
-	p = strtok(aBuf, ",");
-
-	while(p)
+	char aToken[128];
+	for(const char *tok = aBuf; (tok = str_next_token(tok, ",", aToken, sizeof(aToken)));)
 	{
-		if(str_comp_nocase(pName, p) != 0)
+		if(str_comp_nocase(pName, aToken) != 0)
 		{
 			char aBuf2[128];
-			str_format(aBuf2, sizeof(aBuf2), ",%s", p);
+			str_format(aBuf2, sizeof(aBuf2), ",%s", aToken);
 			str_append(pFilter, aBuf2, 128);
 		}
-
-		p = strtok(NULL, ",");
 	}
 }
 
 bool CServerBrowser::DDNetFiltered(char *pFilter, const char *pName)
 {
-	char aBuf[128];
-	char *p;
-
-	str_copy(aBuf, pFilter, sizeof(aBuf));
-
-	p = strtok(aBuf, ",");
-
-	while(p)
-	{
-		if(str_comp_nocase(pName, p) == 0)
-			return true; // country excluded
-
-		p = strtok(NULL, ",");
-	}
-
-	return false; // country not excluded
+	return str_in_list(pFilter, ",", pName); // country not excluded
 }
 
-void CServerBrowser::DDNetCountryFilterClean()
+void CServerBrowser::CountryFilterClean(int Network)
 {
+	char *pExcludeCountries = Network == NETWORK_KOG ? g_Config.m_BrFilterExcludeCountriesKoG : g_Config.m_BrFilterExcludeCountries;
 	char aNewList[128];
 	aNewList[0] = '\0';
 
-	for(int i = 0; i < m_NumDDNetCountries; i++)
+	for(int Network = 0; Network < NUM_NETWORKS; Network++)
 	{
-		const char *pName = m_aDDNetCountries[i].m_aName;
-		if(DDNetFiltered(g_Config.m_BrFilterExcludeCountries, pName))
+		for(int i = 0; i < m_aNetworks[Network].m_NumCountries; i++)
+		{
+			const char *pName = m_aNetworks[Network].m_aCountries[i].m_aName;
+			if(DDNetFiltered(pExcludeCountries, pName))
+			{
+				char aBuf[128];
+				str_format(aBuf, sizeof(aBuf), ",%s", pName);
+				str_append(aNewList, aBuf, sizeof(aNewList));
+			}
+		}
+	}
+
+	str_copy(pExcludeCountries, aNewList, sizeof(g_Config.m_BrFilterExcludeCountries));
+}
+
+void CServerBrowser::TypeFilterClean(int Network)
+{
+	char *pExcludeTypes = Network == NETWORK_KOG ? g_Config.m_BrFilterExcludeTypesKoG : g_Config.m_BrFilterExcludeTypes;
+	char aNewList[128];
+	aNewList[0] = '\0';
+
+	for(int i = 0; i < m_aNetworks[Network].m_NumTypes; i++)
+	{
+		const char *pName = m_aNetworks[Network].m_aTypes[i];
+		if(DDNetFiltered(pExcludeTypes, pName))
 		{
 			char aBuf[128];
 			str_format(aBuf, sizeof(aBuf), ",%s", pName);
@@ -1260,24 +1345,5 @@ void CServerBrowser::DDNetCountryFilterClean()
 		}
 	}
 
-	str_copy(g_Config.m_BrFilterExcludeCountries, aNewList, sizeof(g_Config.m_BrFilterExcludeCountries));
-}
-
-void CServerBrowser::DDNetTypeFilterClean()
-{
-	char aNewList[128];
-	aNewList[0] = '\0';
-
-	for(int i = 0; i < m_NumDDNetTypes; i++)
-	{
-		const char *pName = m_aDDNetTypes[i];
-		if(DDNetFiltered(g_Config.m_BrFilterExcludeTypes, pName))
-		{
-			char aBuf[128];
-			str_format(aBuf, sizeof(aBuf), ",%s", pName);
-			str_append(aNewList, aBuf, sizeof(aNewList));
-		}
-	}
-
-	str_copy(g_Config.m_BrFilterExcludeTypes, aNewList, sizeof(g_Config.m_BrFilterExcludeTypes));
+	str_copy(pExcludeTypes, aNewList, sizeof(g_Config.m_BrFilterExcludeTypes));
 }
