@@ -146,6 +146,7 @@ public:
 	virtual void SetClientClan(int ClientID, char const *pClan) = 0;
 	virtual void SetClientCountry(int ClientID, int Country) = 0;
 	virtual void SetClientScore(int ClientID, int Score) = 0;
+	virtual void SetClientFlags(int ClientID, int Flags) = 0;
 
 	virtual int SnapNewID() = 0;
 	virtual void SnapFreeID(int ID) = 0;
@@ -153,13 +154,7 @@ public:
 
 	virtual void SnapSetStaticsize(int ItemType, int Size) = 0;
 
-	enum
-	{
-		AUTHED_NO=0,
-		AUTHED_HELPER,
-		AUTHED_MOD,
-		AUTHED_ADMIN,
-
+	enum {
 		RCON_CID_SERV=-1,
 		RCON_CID_VOTE=-2,
 	};
