@@ -3,6 +3,8 @@
 #include <new>
 
 #include <base/math.h>
+#include <base/vmath.h>
+#include <base/color.h>
 #include <base/system.h>
 
 #include <engine/storage.h>
@@ -35,6 +37,64 @@ float CConsole::CResult::GetFloat(unsigned Index)
 	return str_tofloat(m_apArgs[Index]);
 }
 
+ColorHSLA CConsole::CResult::GetColor(unsigned Index, bool Light)
+{
+	ColorHSLA hsl = ColorHSLA(0, 0, 0).Lighten();
+	if(Index >= m_NumArgs)
+		return hsl;
+
+	const char *pStr = m_apArgs[Index];
+	if(str_isallnum(pStr) || ((pStr[0] == '-' || pStr[0] == '+') && str_isallnum(pStr+1))) // Teeworlds Color (Packed HSL)
+	{
+		hsl = ColorHSLA(str_toint(pStr), true);
+	}
+	else if(*pStr == '$') // Hex RGB
+	{
+		ColorRGBA rgb = ColorRGBA(0, 0, 0, 1);
+		int Len = str_length(pStr);
+		if(Len == 4)
+		{
+			unsigned Num = str_toulong_base(pStr + 1, 16);
+			rgb.r = (((Num >> 8) & 0x0F) + ((Num >> 4) & 0xF0)) / 255.0f;
+			rgb.g = (((Num >> 4) & 0x0F) + ((Num >> 0) & 0xF0)) / 255.0f;
+			rgb.b = (((Num >> 0) & 0x0F) + ((Num << 4) & 0xF0)) / 255.0f;
+		}
+		else if(Len == 7)
+		{
+			unsigned Num = str_toulong_base(pStr + 1, 16);
+			rgb.r = ((Num >> 16) & 0xFF) / 255.0f;
+			rgb.g = ((Num >> 8) & 0xFF) / 255.0f;
+			rgb.b = ((Num >> 0) & 0xFF) / 255.0f;
+		}
+		else
+		{
+			return hsl;
+		}
+
+		hsl = color_cast<ColorHSLA>(rgb);
+	}
+	else if(!str_comp_nocase(pStr, "red"))
+		hsl = ColorHSLA(0.0f/6.0f, 1, .5f);
+	else if(!str_comp_nocase(pStr, "yellow"))
+		hsl = ColorHSLA(1.0f/6.0f, 1, .5f);
+	else if(!str_comp_nocase(pStr, "green"))
+		hsl = ColorHSLA(2.0f/6.0f, 1, .5f);
+	else if(!str_comp_nocase(pStr, "cyan"))
+		hsl = ColorHSLA(3.0f/6.0f, 1, .5f);
+	else if(!str_comp_nocase(pStr, "blue"))
+		hsl = ColorHSLA(4.0f/6.0f, 1, .5f);
+	else if(!str_comp_nocase(pStr, "magenta"))
+		hsl = ColorHSLA(5.0f/6.0f, 1, .5f);
+	else if(!str_comp_nocase(pStr, "white"))
+		hsl = ColorHSLA(0, 0, 1);
+	else if(!str_comp_nocase(pStr, "gray"))
+		hsl = ColorHSLA(0, 0, .5f);
+	else if(!str_comp_nocase(pStr, "black"))
+		hsl = ColorHSLA(0, 0, 0);
+
+	return Light ? hsl.Lighten() : hsl;
+}
+
 const IConsole::CCommandInfo *CConsole::CCommand::NextCommandInfo(int AccessLevel, int FlagMask) const
 {
 	const CCommand *pInfo = m_pNext;
@@ -645,6 +705,15 @@ struct CIntVariableData
 	int m_OldValue;
 };
 
+struct CColVariableData
+{
+	IConsole *m_pConsole;
+	unsigned *m_pVariable;
+	bool m_Light;
+	bool m_Alpha;
+	unsigned m_OldValue;
+};
+
 struct CStrVariableData
 {
 	IConsole *m_pConsole;
@@ -682,6 +751,42 @@ static void IntVariableCommand(IConsole::IResult *pResult, void *pUserData)
 	}
 }
 
+static void ColVariableCommand(IConsole::IResult *pResult, void *pUserData)
+{
+	CColVariableData *pData = (CColVariableData *)pUserData;
+
+	if(pResult->NumArguments())
+	{
+		int Val = pResult->GetColor(0, pData->m_Light).Pack() & (pData->m_Alpha ? 0xFFFFFFFF : 0xFFFFFF);
+
+		*(pData->m_pVariable) = Val;
+		if(pResult->m_ClientID != IConsole::CLIENT_ID_GAME)
+			pData->m_OldValue = Val;
+	}
+	else
+	{
+		char aBuf[256];
+		str_format(aBuf, sizeof(aBuf), "Value: %u", *(pData->m_pVariable));
+		pData->m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "console", aBuf);
+
+		ColorHSLA hsl(*(pData->m_pVariable), true);
+		str_format(aBuf, sizeof(aBuf), "H: %d°, S: %d%%, L: %d%%", round_truncate(hsl.h * 360), round_truncate(hsl.s * 100), round_truncate(hsl.l * 100));
+		pData->m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "console", aBuf);
+
+		ColorRGBA rgb = color_cast<ColorRGBA>(hsl);
+		str_format(aBuf, sizeof(aBuf), "R: %d, G: %d, B: %d, #%06X", round_truncate(rgb.r * 255), round_truncate(rgb.g * 255), round_truncate(rgb.b * 255), rgb.Pack() & 0xFFFFFF);
+		pData->m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "console", aBuf);
+
+		if(pData->m_Alpha)
+		{
+			str_format(aBuf, sizeof(aBuf), "A: %d%%", round_truncate(hsl.a * 100));
+			pData->m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "console", aBuf);
+		}
+	}
+}
+
+
+
 static void StrVariableCommand(IConsole::IResult *pResult, void *pUserData)
 {
 	CStrVariableData *pData = (CStrVariableData *)pUserData;
@@ -837,6 +942,13 @@ CConsole::CConsole(int FlagMask)
 		Register(#ScriptName, "?i", Flags, IntVariableCommand, &Data, Desc); \
 	}
 
+	#define MACRO_CONFIG_COL(Name,ScriptName,Def,Flags,Desc) \
+	{ \
+		static CColVariableData Data = { this, &g_Config.m_##Name, static_cast<bool>((Flags) & CFGFLAG_COLLIGHT), \
+			static_cast<bool>((Flags) & CFGFLAG_COLALPHA), Def}; \
+		Register(#ScriptName, "?i", Flags, ColVariableCommand, &Data, Desc); \
+	}
+
 	#define MACRO_CONFIG_STR(Name,ScriptName,Len,Def,Flags,Desc) \
 	{ \
 		static char OldValue[Len] = Def; \
@@ -847,6 +959,7 @@ CConsole::CConsole(int FlagMask)
 	#include "config_variables.h"
 
 	#undef MACRO_CONFIG_INT
+	#undef MACRO_CONFIG_COL
 	#undef MACRO_CONFIG_STR
 
 	// DDRace
@@ -862,7 +975,9 @@ CConsole::~CConsole()
 		CCommand *pNext = pCommand->m_pNext;
 		if(pCommand->m_pfnCallback == Con_Chain)
 			delete static_cast<CChain *>(pCommand->m_pUserData);
-		delete pCommand;
+		// Temp commands are on m_TempCommands heap, so don't delete them
+		if(!pCommand->m_Temp)
+			delete pCommand;
 		pCommand = pNext;
 	}
 }
@@ -1108,6 +1223,8 @@ void CConsole::ResetServerGameSettings()
 		} \
 	}
 
+	#define MACRO_CONFIG_COL(Name,ScriptName,Def,Save,Desc) MACRO_CONFIG_INT(Name,ScriptName,Def,0,0,Save,Desc)
+
 	#define MACRO_CONFIG_STR(Name,ScriptName,Len,Def,Flags,Desc) \
 	{ \
 		if(((Flags) & (CFGFLAG_SERVER|CFGFLAG_GAME)) == (CFGFLAG_SERVER|CFGFLAG_GAME)) \
@@ -1129,6 +1246,7 @@ void CConsole::ResetServerGameSettings()
 	#include "config_variables.h"
 
 	#undef MACRO_CONFIG_INT
+	#undef MACRO_CONFIG_COL
 	#undef MACRO_CONFIG_STR
 }
 
