@@ -40,13 +40,17 @@ int CDemoRecorder::Start(class IStorage *pStorage, class IConsole *pConsole, con
 
 	m_MapSize = MapSize;
 	m_pMapData = pMapData;
+	m_pConsole = pConsole;
 
 	IOHANDLE DemoFile = pStorage->OpenFile(pFilename, IOFLAG_WRITE, IStorage::TYPE_SAVE);
 	if(!DemoFile)
 	{
-		char aBuf[256];
-		str_format(aBuf, sizeof(aBuf), "Unable to open '%s' for recording", pFilename);
-		m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "demo_recorder", aBuf);
+		if(m_pConsole)
+		{
+			char aBuf[256];
+			str_format(aBuf, sizeof(aBuf), "Unable to open '%s' for recording", pFilename);
+			m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "demo_recorder", aBuf);
+		}
 		return -1;
 	}
 
@@ -57,8 +61,6 @@ int CDemoRecorder::Start(class IStorage *pStorage, class IConsole *pConsole, con
 		return -1;
 	}
 
-	m_pConsole = pConsole;
-
 	bool CloseMapFile = false;
 
 	if(MapFile)
@@ -68,13 +70,13 @@ int CDemoRecorder::Start(class IStorage *pStorage, class IConsole *pConsole, con
 	{
 		// open mapfile
 		char aMapFilename[128];
-		// try the normal maps folder
-		str_format(aMapFilename, sizeof(aMapFilename), "maps/%s.map", pMap);
+		// try the downloaded maps
+		str_format(aMapFilename, sizeof(aMapFilename), "downloadedmaps/%s_%08x.map", pMap, Crc);
 		MapFile = pStorage->OpenFile(aMapFilename, IOFLAG_READ, IStorage::TYPE_ALL);
 		if(!MapFile)
 		{
-			// try the downloaded maps
-			str_format(aMapFilename, sizeof(aMapFilename), "downloadedmaps/%s_%08x.map", pMap, Crc);
+			// try the normal maps folder
+			str_format(aMapFilename, sizeof(aMapFilename), "maps/%s.map", pMap);
 			MapFile = pStorage->OpenFile(aMapFilename, IOFLAG_READ, IStorage::TYPE_ALL);
 		}
 		if(!MapFile)
@@ -87,9 +89,12 @@ int CDemoRecorder::Start(class IStorage *pStorage, class IConsole *pConsole, con
 		}
 		if(!MapFile)
 		{
-			char aBuf[256];
-			str_format(aBuf, sizeof(aBuf), "Unable to open mapfile '%s'", pMap);
-			m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "demo_recorder", aBuf);
+			if(m_pConsole)
+			{
+				char aBuf[256];
+				str_format(aBuf, sizeof(aBuf), "Unable to open mapfile '%s'", pMap);
+				m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "demo_recorder", aBuf);
+			}
 			return -1;
 		}
 
@@ -145,10 +150,14 @@ int CDemoRecorder::Start(class IStorage *pStorage, class IConsole *pConsole, con
 	m_FirstTick = -1;
 	m_NumTimelineMarkers = 0;
 
-	char aBuf[256];
-	str_format(aBuf, sizeof(aBuf), "Recording to '%s'", pFilename);
-	m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "demo_recorder", aBuf);
+	if(m_pConsole)
+	{
+		char aBuf[256];
+		str_format(aBuf, sizeof(aBuf), "Recording to '%s'", pFilename);
+		m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "demo_recorder", aBuf);
+	}
 	m_File = DemoFile;
+	str_copy(m_aCurrentFilename, pFilename, sizeof(m_aCurrentFilename));
 
 	return 0;
 }
@@ -343,7 +352,8 @@ int CDemoRecorder::Stop()
 
 	io_close(m_File);
 	m_File = 0;
-	m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "demo_recorder", "Stopped recording");
+	if(m_pConsole)
+		m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "demo_recorder", "Stopped recording");
 
 	return 0;
 }
@@ -363,7 +373,8 @@ void CDemoRecorder::AddDemoMarker()
 
 	m_aTimelineMarkers[m_NumTimelineMarkers++] = m_LastTickMarker;
 
-	m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "demo_recorder", "Added timeline marker");
+	if(m_pConsole)
+		m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "demo_recorder", "Added timeline marker");
 }
 
 
@@ -524,10 +535,12 @@ void CDemoPlayer::DoTick()
 		if(ReadChunkHeader(&ChunkType, &ChunkSize, &ChunkTick))
 		{
 			// stop on error or eof
-			m_pConsole->Print(IConsole::OUTPUT_LEVEL_ADDINFO, "demo_player", "end of file");
+			if(m_pConsole)
+				m_pConsole->Print(IConsole::OUTPUT_LEVEL_ADDINFO, "demo_player", "end of file");
 			if(m_Info.m_PreviousTick == -1)
 			{
-				m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "demo_player", "empty demo");
+				if(m_pConsole)
+					m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "demo_player", "empty demo");
 				Stop();
 			}
 			else
@@ -541,7 +554,8 @@ void CDemoPlayer::DoTick()
 			if(io_read(m_File, aCompresseddata, ChunkSize) != (unsigned)ChunkSize)
 			{
 				// stop on error or eof
-				m_pConsole->Print(IConsole::OUTPUT_LEVEL_ADDINFO, "demo_player", "error reading chunk");
+				if(m_pConsole)
+					m_pConsole->Print(IConsole::OUTPUT_LEVEL_ADDINFO, "demo_player", "error reading chunk");
 				Stop();
 				break;
 			}
@@ -550,7 +564,8 @@ void CDemoPlayer::DoTick()
 			if(DataSize < 0)
 			{
 				// stop on error or eof
-				m_pConsole->Print(IConsole::OUTPUT_LEVEL_ADDINFO, "demo_player", "error during network decompression");
+				if(m_pConsole)
+					m_pConsole->Print(IConsole::OUTPUT_LEVEL_ADDINFO, "demo_player", "error during network decompression");
 				Stop();
 				break;
 			}
@@ -559,7 +574,8 @@ void CDemoPlayer::DoTick()
 
 			if(DataSize < 0)
 			{
-				m_pConsole->Print(IConsole::OUTPUT_LEVEL_ADDINFO, "demo_player", "error during intpack decompression");
+				if(m_pConsole)
+					m_pConsole->Print(IConsole::OUTPUT_LEVEL_ADDINFO, "demo_player", "error during intpack decompression");
 				Stop();
 				break;
 			}
@@ -584,9 +600,12 @@ void CDemoPlayer::DoTick()
 			}
 			else
 			{
-				char aBuf[256];
-				str_format(aBuf, sizeof(aBuf), "error during unpacking of delta, err=%d", DataSize);
-				m_pConsole->Print(IConsole::OUTPUT_LEVEL_ADDINFO, "demo_player", aBuf);
+				if(m_pConsole)
+				{
+					char aBuf[256];
+					str_format(aBuf, sizeof(aBuf), "error during unpacking of delta, err=%d", DataSize);
+					m_pConsole->Print(IConsole::OUTPUT_LEVEL_ADDINFO, "demo_player", aBuf);
+				}
 			}
 		}
 		else if(ChunkType == CHUNKTYPE_SNAPSHOT)
@@ -644,9 +663,12 @@ int CDemoPlayer::Load(class IStorage *pStorage, class IConsole *pConsole, const
 	m_File = pStorage->OpenFile(pFilename, IOFLAG_READ, StorageType);
 	if(!m_File)
 	{
-		char aBuf[256];
-		str_format(aBuf, sizeof(aBuf), "could not open '%s'", pFilename);
-		m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "demo_player", aBuf);
+		if(m_pConsole)
+		{
+			char aBuf[256];
+			str_format(aBuf, sizeof(aBuf), "could not open '%s'", pFilename);
+			m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "demo_player", aBuf);
+		}
 		return -1;
 	}
 
@@ -669,9 +691,12 @@ int CDemoPlayer::Load(class IStorage *pStorage, class IConsole *pConsole, const
 	io_read(m_File, &m_Info.m_Header, sizeof(m_Info.m_Header));
 	if(mem_comp(m_Info.m_Header.m_aMarker, gs_aHeaderMarker, sizeof(gs_aHeaderMarker)) != 0)
 	{
-		char aBuf[256];
-		str_format(aBuf, sizeof(aBuf), "'%s' is not a demo file", pFilename);
-		m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "demo_player", aBuf);
+		if(m_pConsole)
+		{
+			char aBuf[256];
+			str_format(aBuf, sizeof(aBuf), "'%s' is not a demo file", pFilename);
+			m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "demo_player", aBuf);
+		}
 		io_close(m_File);
 		m_File = 0;
 		return -1;
@@ -679,9 +704,12 @@ int CDemoPlayer::Load(class IStorage *pStorage, class IConsole *pConsole, const
 
 	if(m_Info.m_Header.m_Version < gs_OldVersion)
 	{
-		char aBuf[256];
-		str_format(aBuf, sizeof(aBuf), "demo version %d is not supported", m_Info.m_Header.m_Version);
-		m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "demo_player", aBuf);
+		if(m_pConsole)
+		{
+			char aBuf[256];
+			str_format(aBuf, sizeof(aBuf), "demo version %d is not supported", m_Info.m_Header.m_Version);
+			m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "demo_player", aBuf);
+		}
 		io_close(m_File);
 		m_File = 0;
 		return -1;
@@ -738,7 +766,7 @@ int CDemoPlayer::Load(class IStorage *pStorage, class IConsole *pConsole, const
 		// get timeline markers
 		int Num = ((m_Info.m_TimelineMarkers.m_aNumTimelineMarkers[0]<<24)&0xFF000000) | ((m_Info.m_TimelineMarkers.m_aNumTimelineMarkers[1]<<16)&0xFF0000) |
 					((m_Info.m_TimelineMarkers.m_aNumTimelineMarkers[2]<<8)&0xFF00) | (m_Info.m_TimelineMarkers.m_aNumTimelineMarkers[3]&0xFF);
-		m_Info.m_Info.m_NumTimelineMarkers = min(Num, (int)MAX_TIMELINE_MARKERS);
+		m_Info.m_Info.m_NumTimelineMarkers = minimum(Num, (int)MAX_TIMELINE_MARKERS);
 		for(int i = 0; i < m_Info.m_Info.m_NumTimelineMarkers; i++)
 		{
 			char *pTimelineMarker = m_Info.m_TimelineMarkers.m_aTimelineMarkers[i];
@@ -778,33 +806,40 @@ int CDemoPlayer::Play()
 	return 0;
 }
 
-int CDemoPlayer::SetPos(float Percent)
+int CDemoPlayer::SeekPercent(float Percent)
+{
+	int WantedTick = m_Info.m_Info.m_FirstTick + ((m_Info.m_Info.m_LastTick - m_Info.m_Info.m_FirstTick) * Percent);
+	return SetPos(WantedTick);
+}
+
+int CDemoPlayer::SeekTime(float Seconds)
+{
+	int WantedTick = m_Info.m_Info.m_CurrentTick + (Seconds * SERVER_TICK_SPEED);
+	return SetPos(WantedTick);
+}
+
+int CDemoPlayer::SetPos(int WantedTick)
 {
-	int Keyframe;
-	int WantedTick;
 	if(!m_File)
 		return -1;
 
 	// -5 because we have to have a current tick and previous tick when we do the playback
-	WantedTick = m_Info.m_Info.m_FirstTick + (int)((m_Info.m_Info.m_LastTick-m_Info.m_Info.m_FirstTick)*Percent) - 5;
-
-	Keyframe = (int)(m_Info.m_SeekablePoints*Percent);
-
-	if(Keyframe < 0 || Keyframe >= m_Info.m_SeekablePoints)
-		return -1;
+	WantedTick = clamp(WantedTick, m_Info.m_Info.m_FirstTick, m_Info.m_Info.m_LastTick) - 5;
 
 	// get correct key frame
-	if(m_pKeyFrames[Keyframe].m_Tick < WantedTick)
-		while(Keyframe < m_Info.m_SeekablePoints-1 && m_pKeyFrames[Keyframe].m_Tick < WantedTick)
-			Keyframe++;
+	int KeyFrame = 0;
+	while(KeyFrame < m_Info.m_SeekablePoints - 1 && m_pKeyFrames[KeyFrame].m_Tick < WantedTick)
+	{
+		KeyFrame++;
+	}
+	while(KeyFrame > 0 && m_pKeyFrames[KeyFrame].m_Tick > WantedTick)
+	{
+		KeyFrame--;
+	}
 
-	while(Keyframe && m_pKeyFrames[Keyframe].m_Tick > WantedTick)
-		Keyframe--;
+	// seek to the correct key frame
+	io_seek(m_File, m_pKeyFrames[KeyFrame].m_Filepos, IOSEEK_START);
 
-	// seek to the correct keyframe
-	io_seek(m_File, m_pKeyFrames[Keyframe].m_Filepos, IOSEEK_START);
-
-	//m_Info.start_tick = -1;
 	m_Info.m_NextTick = -1;
 	m_Info.m_Info.m_CurrentTick = -1;
 	m_Info.m_PreviousTick = -1;
@@ -820,7 +855,7 @@ int CDemoPlayer::SetPos(float Percent)
 
 void CDemoPlayer::SetSpeed(float Speed)
 {
-	m_Info.m_Info.m_Speed = clamp(Speed, 0.f, 64.f);
+	m_Info.m_Info.m_Speed = clamp(Speed, 0.f, 256.f);
 }
 
 void CDemoPlayer::SetSpeedIndex(int Offset)
@@ -873,10 +908,13 @@ int CDemoPlayer::Update(bool RealTime)
 		if(m_Info.m_Info.m_CurrentTick == m_Info.m_PreviousTick ||
 			m_Info.m_Info.m_CurrentTick == m_Info.m_NextTick)
 		{
-			char aBuf[256];
-			str_format(aBuf, sizeof(aBuf), "tick error prev=%d cur=%d next=%d",
-				m_Info.m_PreviousTick, m_Info.m_Info.m_CurrentTick, m_Info.m_NextTick);
-			m_pConsole->Print(IConsole::OUTPUT_LEVEL_ADDINFO, "demo_player", aBuf);
+			if(m_pConsole)
+			{
+				char aBuf[256];
+				str_format(aBuf, sizeof(aBuf), "tick error prev=%d cur=%d next=%d",
+					m_Info.m_PreviousTick, m_Info.m_Info.m_CurrentTick, m_Info.m_NextTick);
+				m_pConsole->Print(IConsole::OUTPUT_LEVEL_ADDINFO, "demo_player", aBuf);
+			}
 		}
 	}
 
@@ -888,7 +926,8 @@ int CDemoPlayer::Stop()
 	if(!m_File)
 		return -1;
 
-	m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "demo_player", "Stopped playback");
+	if(m_pConsole)
+		m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "demo_player", "Stopped playback");
 	io_close(m_File);
 	m_File = 0;
 	free(m_pKeyFrames);
@@ -910,7 +949,7 @@ void CDemoPlayer::GetDemoName(char *pBuffer, int BufferSize) const
 			pEnd = pFileName;
 	}
 
-	int Length = pEnd > pExtractedName ? min(BufferSize, (int)(pEnd-pExtractedName+1)) : BufferSize;
+	int Length = pEnd > pExtractedName ? minimum(BufferSize, (int)(pEnd-pExtractedName+1)) : BufferSize;
 	str_copy(pBuffer, pExtractedName, Length);
 }
 
@@ -984,6 +1023,7 @@ void CDemoEditor::Slice(const char *pDemo, const char *pDst, int StartTick, int
 			break;
 	}
 
+	m_pDemoPlayer->Stop();
 	m_pDemoRecorder->Stop();
 }
 
