@@ -14,6 +14,7 @@ class CDemoRecorder : public IDemoRecorder
 {
 	class IConsole *m_pConsole;
 	IOHANDLE m_File;
+	char m_aCurrentFilename[256];
 	int m_LastTickMarker;
 	int m_LastKeyFrame;
 	int m_FirstTick;
@@ -42,6 +43,7 @@ public:
 	void RecordMessage(const void *pData, int Size);
 
 	bool IsRecording() const { return m_File != 0; }
+	char *GetCurrentFilename() { return m_aCurrentFilename; }
 
 	int Length() const { return (m_LastTickMarker - m_FirstTick)/SERVER_TICK_SPEED; }
 };
@@ -132,7 +134,9 @@ public:
 	int Stop();
 	void SetSpeed(float Speed);
 	void SetSpeedIndex(int Offset);
-	int SetPos(float Percent);
+	int SeekPercent(float Percent);
+	int SeekTime(float Seconds);
+	int SetPos(int WantedTick);
 	const CInfo *BaseInfo() const { return &m_Info.m_Info; }
 	void GetDemoName(char *pBuffer, int BufferSize) const;
 	bool GetDemoInfo(class IStorage *pStorage, const char *pFilename, int StorageType, CDemoHeader *pDemoHeader, CTimelineMarkers *pTimelineMarkers) const;
