@@ -3,6 +3,7 @@
 
 #include <base/tl/string.h>
 
+#include <engine/editor.h>
 #include <engine/engine.h>
 #include <engine/graphics.h>
 #include <engine/textrender.h>
@@ -32,9 +33,19 @@ CChat::CChat()
 		// reset the container indices, so the text containers can be deleted on reset
 		m_aLines[i].m_TextContainerIndex = -1;
 	}
+
+	#define CHAT_COMMAND(name, params, flags, callback, userdata, help) RegisterCommand(name, params, flags, help);
+	#include <game/server/ddracechat.h>
+	m_Commands.sort_range();
+
 	OnReset();
 }
 
+void CChat::RegisterCommand(const char *pName, const char *pParams, int flags, const char *pHelp)
+{
+	m_Commands.add_unsorted(CCommand{pName, pParams});
+}
+
 void CChat::OnWindowResize()
 {
 	for(int i = 0; i < MAX_LINES; i++)
@@ -127,7 +138,12 @@ void CChat::ConShowChat(IConsole::IResult *pResult, void *pUserData)
 
 void CChat::ConEcho(IConsole::IResult *pResult, void *pUserData)
 {
-	((CChat *)pUserData)->AddLine(-2, 0, pResult->GetString(0));
+	((CChat *)pUserData)->Echo(pResult->GetString(0));
+}
+
+void CChat::Echo(const char *pString)
+{
+	AddLine(-2, 0, pString);
 }
 
 void CChat::OnConsoleInit()
@@ -157,14 +173,14 @@ bool CChat::OnInput(IInput::CEvent Event)
 			{
 				if(Text[i] == '\n')
 				{
-					int max = min(i - Begin + 1, (int)sizeof(Line));
+					int max = minimum(i - Begin + 1, (int)sizeof(Line));
 					str_copy(Line, Text + Begin, max);
 					Begin = i+1;
 					SayChat(Line);
 					while(Text[i] == '\n') i++;
 				}
 			}
-			int max = min(i - Begin + 1, (int)sizeof(Line));
+			int max = minimum(i - Begin + 1, (int)sizeof(Line));
 			str_copy(Line, Text + Begin, max);
 			Begin = i+1;
 			m_Input.Add(Line);
@@ -228,7 +244,7 @@ bool CChat::OnInput(IInput::CEvent Event)
 
 					if(FoundAt != m_Input.GetLength())
 						str_append(aText, m_Input.GetString() + FoundAt, sizeof(aText));
-					
+
 					m_Input.Set(aText);
 					FoundAt = OldOffset;
 				}
@@ -291,85 +307,153 @@ bool CChat::OnInput(IInput::CEvent Event)
 			for(m_PlaceholderLength = 0; *pCursor && *pCursor != ' '; ++pCursor)
 				++m_PlaceholderLength;
 
-			str_copy(m_aCompletionBuffer, m_Input.GetString()+m_PlaceholderOffset, min(static_cast<int>(sizeof(m_aCompletionBuffer)), m_PlaceholderLength+1));
+			str_copy(m_aCompletionBuffer, m_Input.GetString()+m_PlaceholderOffset, minimum(static_cast<int>(sizeof(m_aCompletionBuffer)), m_PlaceholderLength+1));
 		}
 
-		// find next possible name
-		const char *pCompletionString = 0;
+		if(m_aCompletionBuffer[0] == '/')
+		{
+			CCommand *pCompletionCommand = 0;
+
+			const size_t NumCommands = m_Commands.size();
+
+			if(m_ReverseTAB)
+				m_CompletionChosen = (m_CompletionChosen-1 + 2*NumCommands)%(2*NumCommands);
+			else
+				m_CompletionChosen = (m_CompletionChosen+1)%(2*NumCommands);
+
+			const char *pCommandStart = m_aCompletionBuffer+1;
+			for(size_t i = 0; i < 2*NumCommands; ++i)
+			{
+				int SearchType;
+				int Index;
+
+				if(m_ReverseTAB)
+				{
+					SearchType = ((m_CompletionChosen-i +2*NumCommands)%(2*NumCommands))/NumCommands;
+					Index = (m_CompletionChosen-i + NumCommands )%NumCommands;
+				}
+				else
+				{
+					SearchType = ((m_CompletionChosen+i)%(2*NumCommands))/NumCommands;
+					Index = (m_CompletionChosen+i)%NumCommands;
+				}
+
+				auto &Command = m_Commands[Index];
+
+				if(str_comp_nocase_num(Command.pName, pCommandStart, str_length(pCommandStart)) == 0)
+				{
+					pCompletionCommand = &Command;
+					m_CompletionChosen = Index+SearchType*NumCommands;
+					break;
+				}
+			}
+
+			// insert the command
+			if(pCompletionCommand)
+			{
+				char aBuf[256];
+				// add part before the name
+				str_copy(aBuf, m_Input.GetString(), minimum(static_cast<int>(sizeof(aBuf)), m_PlaceholderOffset+1));
+
+				// add the command
+				str_append(aBuf, "/", sizeof(aBuf));
+				str_append(aBuf, pCompletionCommand->pName, sizeof(aBuf));
+
+				// add separator
+				const char *pSeparator = pCompletionCommand->pParams[0] == '\0' ? "" : " ";
+				str_append(aBuf, pSeparator, sizeof(aBuf));
+				if(*pSeparator)
+					str_append(aBuf, pSeparator, sizeof(aBuf));
+
+				// add part after the name
+				str_append(aBuf, m_Input.GetString()+m_PlaceholderOffset+m_PlaceholderLength, sizeof(aBuf));
+
+				m_PlaceholderLength = str_length(pSeparator)+str_length(pCompletionCommand->pName)+1;
+				m_OldChatStringLength = m_Input.GetLength();
+				m_Input.Set(aBuf); // TODO: Use Add instead
+				m_Input.SetCursorOffset(m_PlaceholderOffset+m_PlaceholderLength);
+				m_InputUpdate = true;
+			}
+		}
+		else
+		{
+			// find next possible name
+			const char *pCompletionString = 0;
 
 			if(m_ReverseTAB)
 				m_CompletionChosen = (m_CompletionChosen-1 + 2*MAX_CLIENTS)%(2*MAX_CLIENTS);
 			else
 				m_CompletionChosen = (m_CompletionChosen+1)%(2*MAX_CLIENTS);
 
-		for(int i = 0; i < 2*MAX_CLIENTS; ++i)
-		{
-			int SearchType;
-			int Index;
-
-			if(m_ReverseTAB)
+			for(int i = 0; i < 2*MAX_CLIENTS; ++i)
 			{
-				SearchType = ((m_CompletionChosen-i +2*MAX_CLIENTS)%(2*MAX_CLIENTS))/MAX_CLIENTS;
-				Index = (m_CompletionChosen-i + MAX_CLIENTS )%MAX_CLIENTS;
-			}
-			else
-			{
-				SearchType = ((m_CompletionChosen+i)%(2*MAX_CLIENTS))/MAX_CLIENTS;
-				Index = (m_CompletionChosen+i)%MAX_CLIENTS;
-			}
+				int SearchType;
+				int Index;
+
+				if(m_ReverseTAB)
+				{
+					SearchType = ((m_CompletionChosen-i +2*MAX_CLIENTS)%(2*MAX_CLIENTS))/MAX_CLIENTS;
+					Index = (m_CompletionChosen-i + MAX_CLIENTS )%MAX_CLIENTS;
+				}
+				else
+				{
+					SearchType = ((m_CompletionChosen+i)%(2*MAX_CLIENTS))/MAX_CLIENTS;
+					Index = (m_CompletionChosen+i)%MAX_CLIENTS;
+				}
 
 
-			if(!m_pClient->m_Snap.m_paInfoByName[Index])
-				continue;
+				if(!m_pClient->m_Snap.m_paInfoByName[Index])
+					continue;
 
-			int Index2 = m_pClient->m_Snap.m_paInfoByName[Index]->m_ClientID;
+				int Index2 = m_pClient->m_Snap.m_paInfoByName[Index]->m_ClientID;
 
-			bool Found = false;
-			if(SearchType == 1)
-			{
-				if(str_utf8_comp_nocase_num(m_pClient->m_aClients[Index2].m_aName, m_aCompletionBuffer, str_length(m_aCompletionBuffer)) &&
-					str_utf8_find_nocase(m_pClient->m_aClients[Index2].m_aName, m_aCompletionBuffer))
+				bool Found = false;
+				if(SearchType == 1)
+				{
+					if(str_utf8_comp_nocase_num(m_pClient->m_aClients[Index2].m_aName, m_aCompletionBuffer, str_length(m_aCompletionBuffer)) &&
+						str_utf8_find_nocase(m_pClient->m_aClients[Index2].m_aName, m_aCompletionBuffer))
+						Found = true;
+				}
+				else if(!str_utf8_comp_nocase_num(m_pClient->m_aClients[Index2].m_aName, m_aCompletionBuffer, str_length(m_aCompletionBuffer)))
 					Found = true;
-			}
-			else if(!str_utf8_comp_nocase_num(m_pClient->m_aClients[Index2].m_aName, m_aCompletionBuffer, str_length(m_aCompletionBuffer)))
-				Found = true;
 
-			if(Found)
+				if(Found)
+				{
+					pCompletionString = m_pClient->m_aClients[Index2].m_aName;
+					m_CompletionChosen = Index+SearchType*MAX_CLIENTS;
+					break;
+				}
+			}
+
+			// insert the name
+			if(pCompletionString)
 			{
-				pCompletionString = m_pClient->m_aClients[Index2].m_aName;
-				m_CompletionChosen = Index+SearchType*MAX_CLIENTS;
-				break;
+				char aBuf[256];
+				// add part before the name
+				str_copy(aBuf, m_Input.GetString(), minimum(static_cast<int>(sizeof(aBuf)), m_PlaceholderOffset+1));
+
+				// add the name
+				str_append(aBuf, pCompletionString, sizeof(aBuf));
+
+				// add separator
+				const char *pSeparator = "";
+				if(*(m_Input.GetString()+m_PlaceholderOffset+m_PlaceholderLength) != ' ')
+					pSeparator = m_PlaceholderOffset == 0 ? ": " : " ";
+				else if(m_PlaceholderOffset == 0)
+					pSeparator = ":";
+				if(*pSeparator)
+					str_append(aBuf, pSeparator, sizeof(aBuf));
+
+				// add part after the name
+				str_append(aBuf, m_Input.GetString()+m_PlaceholderOffset+m_PlaceholderLength, sizeof(aBuf));
+
+				m_PlaceholderLength = str_length(pSeparator)+str_length(pCompletionString);
+				m_OldChatStringLength = m_Input.GetLength();
+				m_Input.Set(aBuf); // TODO: Use Add instead
+				m_Input.SetCursorOffset(m_PlaceholderOffset+m_PlaceholderLength);
+				m_InputUpdate = true;
 			}
 		}
-
-		// insert the name
-		if(pCompletionString)
-		{
-			char aBuf[256];
-			// add part before the name
-			str_copy(aBuf, m_Input.GetString(), min(static_cast<int>(sizeof(aBuf)), m_PlaceholderOffset+1));
-
-			// add the name
-			str_append(aBuf, pCompletionString, sizeof(aBuf));
-
-			// add separator
-			const char *pSeparator = "";
-			if(*(m_Input.GetString()+m_PlaceholderOffset+m_PlaceholderLength) != ' ')
-				pSeparator = m_PlaceholderOffset == 0 ? ": " : " ";
-			else if(m_PlaceholderOffset == 0)
-				pSeparator = ":";
-			if(*pSeparator)
-				str_append(aBuf, pSeparator, sizeof(aBuf));
-
-			// add part after the name
-			str_append(aBuf, m_Input.GetString()+m_PlaceholderOffset+m_PlaceholderLength, sizeof(aBuf));
-
-			m_PlaceholderLength = str_length(pSeparator)+str_length(pCompletionString);
-			m_OldChatStringLength = m_Input.GetLength();
-			m_Input.Set(aBuf); // TODO: Use Add instead
-			m_Input.SetCursorOffset(m_PlaceholderOffset+m_PlaceholderLength);
-			m_InputUpdate = true;
-		}
 	}
 	else
 	{
@@ -435,7 +519,6 @@ void CChat::EnableMode(int Team)
 		Input()->SetIMEState(true);
 		Input()->Clear();
 		m_CompletionChosen = -1;
-		UI()->AndroidShowTextInput("", Team ? Localize("Team chat") : Localize("Chat"));
 	}
 }
 
@@ -633,6 +716,11 @@ void CChat::AddLine(int ClientID, int Team, const char *pLine)
 				m_pClient->m_pSounds->Play(CSounds::CHN_GUI, SOUND_CHAT_HIGHLIGHT, 0);
 				m_aLastSoundPlayed[CHAT_HIGHLIGHT] = Now;
 			}
+
+			if(g_Config.m_ClEditor)
+			{
+				GameClient()->Editor()->UpdateMentions();
+			}
 		}
 	}
 	else if(Team != 2)
@@ -653,15 +741,7 @@ void CChat::OnPrepareLines()
 {
 	float x = 5.0f;
 	float y = 300.0f - 28.0f;
-#if defined(__ANDROID__)
-	x += 120.0f;
-#endif
-
-#if defined(__ANDROID__)
-	float FontSize = 10.0f;
-#else
 	float FontSize = 6.0f;
-#endif
 
 	bool ForceRecreate = m_pClient->m_pScoreboard->Active() != m_PrevScoreBoardShowed;
 	ForceRecreate |= m_Show != m_PrevShowChat;
@@ -690,7 +770,7 @@ void CChat::OnPrepareLines()
 		m_aLines[r].m_TextContainerIndex = -1;
 
 		char aName[64] = "";
-		if(g_Config.m_ClShowIDs && m_aLines[r].m_ClientID != -1 && m_aLines[r].m_aName[0] != '\0')
+		if(g_Config.m_ClShowIDs && m_aLines[r].m_ClientID >= 0 && m_aLines[r].m_aName[0] != '\0')
 		{
 			if(m_aLines[r].m_ClientID >= 10)
 				str_format(aName, sizeof(aName), "%d: ", m_aLines[r].m_ClientID);
@@ -729,26 +809,26 @@ void CChat::OnPrepareLines()
 
 		if(g_Config.m_ClMessageFriend)
 		{
-			vec3 rgb = HslToRgb(vec3(g_Config.m_ClMessageFriendHue / 255.0f, g_Config.m_ClMessageFriendSat / 255.0f, g_Config.m_ClMessageFriendLht / 255.0f));
-			TextRender()->TextColor(rgb.r, rgb.g, rgb.b, m_aLines[r].m_Friend ? 1.f : 0.f); //Less ugly hack to align messages
+			ColorRGBA rgb = color_cast<ColorRGBA>(ColorHSLA(g_Config.m_ClMessageFriendColor));
+			TextRender()->TextColor(rgb.SetAlpha(m_aLines[r].m_Friend ? 1.f : 0.f)); //Less ugly hack to align messages
 			m_aLines[r].m_TextContainerIndex = TextRender()->CreateTextContainer(&Cursor, "♥ ");
 		}
 
 		// render name
 		if(m_aLines[r].m_ClientID == -1) // system
 		{
-			vec3 rgb = HslToRgb(vec3(g_Config.m_ClMessageSystemHue / 255.0f, g_Config.m_ClMessageSystemSat / 255.0f, g_Config.m_ClMessageSystemLht / 255.0f));
-			TextRender()->TextColor(rgb.r, rgb.g, rgb.b, 1.f);
+			ColorRGBA rgb = color_cast<ColorRGBA>(ColorHSLA(g_Config.m_ClMessageSystemColor));
+			TextRender()->TextColor(rgb);
 		}
 		else if(m_aLines[r].m_ClientID == -2) // client
 		{
-			vec3 rgb = HslToRgb(vec3(g_Config.m_ClMessageClientHue / 255.0f, g_Config.m_ClMessageClientSat / 255.0f, g_Config.m_ClMessageClientLht / 255.0f));
-			TextRender()->TextColor(rgb.r, rgb.g, rgb.b, 1.f);
+			ColorRGBA rgb = color_cast<ColorRGBA>(ColorHSLA(g_Config.m_ClMessageClientColor));
+			TextRender()->TextColor(rgb);
 		}
 		else if(m_aLines[r].m_Team)
 		{
-			vec3 rgb = CalculateNameColor(vec3(g_Config.m_ClMessageTeamHue / 255.0f, g_Config.m_ClMessageTeamSat / 255.0f, g_Config.m_ClMessageTeamLht / 255.0f));
-			TextRender()->TextColor(rgb.r, rgb.g, rgb.b, 1.f); // team message
+			ColorRGBA rgb = CalculateNameColor(ColorHSLA(g_Config.m_ClMessageTeamColor));
+			TextRender()->TextColor(rgb); // team message
 		}
 		else if(m_aLines[r].m_NameColor == TEAM_RED)
 			TextRender()->TextColor(1.0f, 0.5f, 0.5f, 1.f); // red
@@ -758,8 +838,8 @@ void CChat::OnPrepareLines()
 			TextRender()->TextColor(0.75f, 0.5f, 0.75f, 1.f); // spectator
 		else if(m_aLines[r].m_ClientID >= 0 && g_Config.m_ClChatTeamColors && m_pClient->m_Teams.Team(m_aLines[r].m_ClientID))
 		{
-			vec3 rgb = HslToRgb(vec3(m_pClient->m_Teams.Team(m_aLines[r].m_ClientID) / 64.0f, 1.0f, 0.75f));
-			TextRender()->TextColor(rgb.r, rgb.g, rgb.b, 1.f);
+			ColorRGBA rgb = color_cast<ColorRGBA>(ColorHSLA(m_pClient->m_Teams.Team(m_aLines[r].m_ClientID) / 64.0f, 1.0f, 0.75f));
+			TextRender()->TextColor(rgb);
 		}
 		else
 			TextRender()->TextColor(0.8f, 0.8f, 0.8f, 1.f);
@@ -772,28 +852,28 @@ void CChat::OnPrepareLines()
 		// render line
 		if(m_aLines[r].m_ClientID == -1) // system
 		{
-			vec3 rgb = HslToRgb(vec3(g_Config.m_ClMessageSystemHue / 255.0f, g_Config.m_ClMessageSystemSat / 255.0f, g_Config.m_ClMessageSystemLht / 255.0f));
-			TextRender()->TextColor(rgb.r, rgb.g, rgb.b, 1.f);
+			ColorRGBA rgb = color_cast<ColorRGBA>(ColorHSLA(g_Config.m_ClMessageSystemColor));
+			TextRender()->TextColor(rgb);
 		}
 		else if(m_aLines[r].m_ClientID == -2) // client
 		{
-			vec3 rgb = HslToRgb(vec3(g_Config.m_ClMessageClientHue / 255.0f, g_Config.m_ClMessageClientSat / 255.0f, g_Config.m_ClMessageClientLht / 255.0f));
-			TextRender()->TextColor(rgb.r, rgb.g, rgb.b, 1.f);
+			ColorRGBA rgb = color_cast<ColorRGBA>(ColorHSLA(g_Config.m_ClMessageClientColor));
+			TextRender()->TextColor(rgb);
 		}
 		else if(m_aLines[r].m_Highlighted) // highlighted
 		{
-			vec3 rgb = HslToRgb(vec3(g_Config.m_ClMessageHighlightHue / 255.0f, g_Config.m_ClMessageHighlightSat / 255.0f, g_Config.m_ClMessageHighlightLht / 255.0f));
-			TextRender()->TextColor(rgb.r, rgb.g, rgb.b, 1.f);
+			ColorRGBA rgb = color_cast<ColorRGBA>(ColorHSLA(g_Config.m_ClMessageHighlightColor));
+			TextRender()->TextColor(rgb);
 		}
 		else if(m_aLines[r].m_Team) // team message
 		{
-			vec3 rgb = HslToRgb(vec3(g_Config.m_ClMessageTeamHue / 255.0f, g_Config.m_ClMessageTeamSat / 255.0f, g_Config.m_ClMessageTeamLht / 255.0f));
-			TextRender()->TextColor(rgb.r, rgb.g, rgb.b, 1.f);
+			ColorRGBA rgb = color_cast<ColorRGBA>(ColorHSLA(g_Config.m_ClMessageTeamColor));
+			TextRender()->TextColor(rgb);
 		}
 		else // regular message
 		{
-			vec3 rgb = HslToRgb(vec3(g_Config.m_ClMessageHue / 255.0f, g_Config.m_ClMessageSat / 255.0f, g_Config.m_ClMessageLht / 255.0f));
-			TextRender()->TextColor(rgb.r, rgb.g, rgb.b, 1.f);
+			ColorRGBA rgb = color_cast<ColorRGBA>(ColorHSLA(g_Config.m_ClMessageColor));
+			TextRender()->TextColor(rgb);
 		}
 
 		if(m_aLines[r].m_TextContainerIndex == -1)
@@ -859,7 +939,7 @@ void CChat::OnRender()
 		if(m_InputUpdate)
 		{
 			if(m_ChatStringOffset > 0 && m_Input.GetLength(Editing) < m_OldChatStringLength)
-				m_ChatStringOffset = max(0, m_ChatStringOffset-(m_OldChatStringLength-m_Input.GetLength(Editing)));
+				m_ChatStringOffset = maximum(0, m_ChatStringOffset-(m_OldChatStringLength-m_Input.GetLength(Editing)));
 
 			if(m_ChatStringOffset > m_Input.GetCursorOffset(Editing))
 				m_ChatStringOffset -= m_ChatStringOffset-m_Input.GetCursorOffset(Editing);
@@ -893,9 +973,6 @@ void CChat::OnRender()
 		return;
 
 	y -= 8.0f;
-#if defined(__ANDROID__)
-	x += 120.0f;
-#endif
 
 	OnPrepareLines();
 
@@ -923,37 +1000,6 @@ void CChat::OnRender()
 			TextRender()->RenderTextContainer(m_aLines[r].m_TextContainerIndex, &Text, &TextOutline, 0, y - m_aLines[r].m_TextYOffset);
 		}
 	}
-
-#if defined(__ANDROID__)
-	static int deferEvent = 0;
-	if( UI()->AndroidTextInputShown() )
-	{
-		if(m_Mode == MODE_NONE)
-		{
-			deferEvent++;
-			if( deferEvent > 2 )
-				EnableMode(0);
-		}
-		else
-			deferEvent = 0;
-	}
-	else
-	{
-		if(m_Mode != MODE_NONE)
-		{
-			deferEvent++;
-			if( deferEvent > 2 )
-			{
-				IInput::CEvent Event;
-				Event.m_Flags = IInput::FLAG_PRESS;
-				Event.m_Key = KEY_RETURN;
-				OnInput(Event);
-			}
-		}
-		else
-			deferEvent = 0;
-	}
-#endif
 }
 
 void CChat::Say(int Team, const char *pLine)
