@@ -109,6 +109,53 @@ void CGameConsole::CInstance::OnInput(IInput::CEvent Event)
 {
 	bool Handled = false;
 
+	if(m_pGameConsole->Input()->KeyIsPressed(KEY_LCTRL)) // jump to spaces and special ASCII characters
+	{
+		int SearchDirection = 0;
+		if(m_pGameConsole->Input()->KeyPress(KEY_LEFT))
+			SearchDirection = -1;
+		else if(m_pGameConsole->Input()->KeyPress(KEY_RIGHT) || m_pGameConsole->Input()->KeyPress(KEY_DELETE))
+			SearchDirection = 1;
+
+		if(SearchDirection != 0)
+		{
+			int OldOffset = m_Input.GetCursorOffset();
+
+			int FoundAt = SearchDirection > 0 ? m_Input.GetLength() - 1 : 0;
+			for(int i = m_Input.GetCursorOffset() + SearchDirection; SearchDirection > 0 ? i < m_Input.GetLength() - 1 : i > 0; i += SearchDirection)
+			{
+				int Next = i + SearchDirection;
+				if(	(m_Input.GetString()[Next] == ' ') ||
+					(m_Input.GetString()[Next] >= 32 && m_Input.GetString()[Next] <= 47) ||
+					(m_Input.GetString()[Next] >= 58 && m_Input.GetString()[Next] <= 64) ||
+					(m_Input.GetString()[Next] >= 91 && m_Input.GetString()[Next] <= 96))
+				{
+					FoundAt = i;
+					if(SearchDirection < 0)
+						FoundAt++;
+					break;
+				}
+			}
+
+			if(m_pGameConsole->Input()->KeyPress(KEY_DELETE))
+			{
+				if(m_Input.GetCursorOffset() != m_Input.GetLength())
+				{
+					char aText[512];
+					aText[0] = '\0';
+
+					str_copy(aText, m_Input.GetString(), m_Input.GetCursorOffset() + 1);
+
+					if(FoundAt != m_Input.GetLength())
+						str_append(aText, m_Input.GetString() + FoundAt, sizeof(aText));
+
+					m_Input.Set(aText);
+					FoundAt = OldOffset;
+				}
+			}
+			m_Input.SetCursorOffset(FoundAt);
+		}
+	}
 	if(m_pGameConsole->Input()->KeyIsPressed(KEY_LCTRL) && m_pGameConsole->Input()->KeyPress(KEY_V))
 	{
 		const char *Text = m_pGameConsole->Input()->GetClipboardText();
@@ -125,23 +172,34 @@ void CGameConsole::CInstance::OnInput(IInput::CEvent Event)
 						Begin++;
 						continue;
 					}
-					int max = min(i - Begin + 1, (int)sizeof(Line));
+					int max = minimum(i - Begin + 1, (int)sizeof(Line));
 					str_copy(Line, Text + Begin, max);
 					Begin = i+1;
 					ExecuteLine(Line);
 				}
 			}
-			int max = min(i - Begin + 1, (int)sizeof(Line));
+			int max = minimum(i - Begin + 1, (int)sizeof(Line));
 			str_copy(Line, Text + Begin, max);
 			Begin = i+1;
 			m_Input.Add(Line);
 		}
 	}
-
-	if(m_pGameConsole->Input()->KeyIsPressed(KEY_LCTRL) && m_pGameConsole->Input()->KeyPress(KEY_C))
+	else if(m_pGameConsole->Input()->KeyIsPressed(KEY_LCTRL) && m_pGameConsole->Input()->KeyPress(KEY_C))
 	{
 		m_pGameConsole->Input()->SetClipboardText(m_Input.GetString());
 	}
+	else if(m_pGameConsole->Input()->KeyIsPressed(KEY_LCTRL) && m_pGameConsole->Input()->KeyPress(KEY_A))
+	{
+		m_Input.SetCursorOffset(0);
+	}
+	else if(m_pGameConsole->Input()->KeyIsPressed(KEY_LCTRL) && m_pGameConsole->Input()->KeyPress(KEY_E))
+	{
+		m_Input.SetCursorOffset(m_Input.GetLength());
+	}
+	else if(m_pGameConsole->Input()->KeyIsPressed(KEY_LCTRL) && m_pGameConsole->Input()->KeyPress(KEY_U))
+	{
+		m_Input.DeleteUntilCursor();
+	}
 
 	if(Event.m_Flags&IInput::FLAG_PRESS)
 	{
@@ -220,11 +278,13 @@ void CGameConsole::CInstance::OnInput(IInput::CEvent Event)
 			if(m_BacklogActPage < 0)
 				m_BacklogActPage = 0;
 		}
-		else if(Event.m_Key == KEY_HOME)
+		// in order not to conflict with CLineInput's handling of Home/End only
+		// react to it when the input is empty
+		else if(Event.m_Key == KEY_HOME && m_Input.GetString()[0] == '\0')
 		{
 			m_BacklogActPage = INT_MAX;
 		}
-		else if(Event.m_Key == KEY_END)
+		else if(Event.m_Key == KEY_END && m_Input.GetString()[0] == '\0')
 		{
 			m_BacklogActPage = 0;
 		}
@@ -386,7 +446,7 @@ void CGameConsole::OnRender()
 	float ConsoleMaxHeight = Screen.h*3/5.0f;
 	float ConsoleHeight;
 
-	float Progress = (TimeNow()-(m_StateChangeEnd-m_StateChangeDuration))/float(m_StateChangeDuration);
+	float Progress = (TimeNow()-(m_StateChangeEnd-m_StateChangeDuration))/m_StateChangeDuration;
 
 	if (Progress >= 1.0f)
 	{
@@ -579,7 +639,7 @@ void CGameConsole::OnRender()
 			}
 		}
 
-		vec3 rgb = HslToRgb(vec3(g_Config.m_ClMessageHighlightHue / 255.0f, g_Config.m_ClMessageHighlightSat / 255.0f, g_Config.m_ClMessageHighlightLht / 255.0f));
+		ColorRGBA rgb = color_cast<ColorRGBA>(ColorHSLA(g_Config.m_ClMessageHighlightColor));
 
 		//	render log (actual page, wrap lines)
 		CInstance::CBacklogEntry *pEntry = pConsole->m_Backlog.Last();
@@ -591,7 +651,7 @@ void CGameConsole::OnRender()
 			while(pEntry)
 			{
 				if(pEntry->m_Highlighted)
-					TextRender()->TextColor(rgb.r, rgb.g, rgb.b, 1);
+					TextRender()->TextColor(rgb);
 				else
 					TextRender()->TextColor(1,1,1,1);
 
