@@ -54,12 +54,8 @@ bool CEmoticon::OnMouseMove(float x, float y)
 	if(!m_Active)
 		return false;
 
-#if defined(__ANDROID__) // No relative mouse on Android
-	m_SelectorMouse = vec2(x,y);
-#else
 	UI()->ConvertMouseMove(&x, &y);
 	m_SelectorMouse += vec2(x,y);
-#endif
 	return true;
 }
 
@@ -141,7 +137,7 @@ void CEmoticon::OnRender()
 
 	CServerInfo pServerInfo;
 	Client()->GetServerInfo(&pServerInfo);
-	if((IsDDRace(&pServerInfo) || IsDDNet(&pServerInfo) || IsPlus(&pServerInfo)) && g_Config.m_ClEyeWheel)
+	if((IsDDRace(&pServerInfo) || IsDDNet(&pServerInfo) || IsBlockWorlds(&pServerInfo) || IsPlus(&pServerInfo)) && g_Config.m_ClEyeWheel)
 	{
 		Graphics()->TextureSet(-1);
 		Graphics()->QuadsBegin();
