@@ -1,7 +1,6 @@
 /* (c) Rajh, Redix and Sushi. */
 
 #include <engine/ghost.h>
-#include <engine/graphics.h>
 #include <engine/serverbrowser.h>
 #include <engine/storage.h>
 #include <engine/shared/config.h>
@@ -264,7 +263,7 @@ void CGhost::OnNewSnapshot()
 {
 	CServerInfo ServerInfo;
 	Client()->GetServerInfo(&ServerInfo);
-	if(!IsRace(&ServerInfo) || !g_Config.m_ClRaceGhost || Client()->State() != IClient::STATE_ONLINE)
+	if(!IsRace(&ServerInfo) || Client()->State() != IClient::STATE_ONLINE)
 		return;
 	if(!m_pClient->m_Snap.m_pGameInfoObj || m_pClient->m_Snap.m_SpecInfo.m_Active || !m_pClient->m_Snap.m_pLocalCharacter || !m_pClient->m_Snap.m_pLocalPrevCharacter)
 		return;
@@ -272,14 +271,18 @@ void CGhost::OnNewSnapshot()
 	bool RaceFlag = m_pClient->m_Snap.m_pGameInfoObj->m_GameStateFlags&GAMESTATEFLAG_RACETIME;
 	bool ServerControl = RaceFlag && g_Config.m_ClRaceGhostServerControl;
 
-	if(!ServerControl)
-		CheckStartLocal(false);
-	else
-		CheckStart();
+	if(g_Config.m_ClRaceGhost)
+	{
+		if(!ServerControl)
+			CheckStartLocal(false);
+		else
+			CheckStart();
 
-	if(m_Recording)
-		AddInfos(m_pClient->m_Snap.m_pLocalCharacter);
+		if(m_Recording)
+			AddInfos(m_pClient->m_Snap.m_pLocalCharacter);
+	}
 
+	// Record m_LastRaceTick for g_Config.m_ClConfirmDisconnect/QuitTime anyway
 	int RaceTick = -m_pClient->m_Snap.m_pGameInfoObj->m_WarmupTimer;
 	m_LastRaceTick = RaceFlag ? RaceTick : -1;
 }
@@ -327,7 +330,7 @@ void CGhost::OnRender()
 			continue;
 
 		int CurPos = pGhost->m_PlaybackPos;
-		int PrevPos = max(0, CurPos - 1);
+		int PrevPos = maximum(0, CurPos - 1);
 		if(pGhost->m_Path.Get(PrevPos)->m_Tick > GhostTick)
 			continue;
 
@@ -342,8 +345,8 @@ void CGhost::OnRender()
 
 		Player.m_AttackTick += Client()->GameTick() - GhostTick;
 
-		m_pClient->m_pPlayers->RenderHook(&Prev, &Player, &pGhost->m_RenderInfo , -2, vec2(), vec2(), IntraTick);
-		m_pClient->m_pPlayers->RenderPlayer(&Prev, &Player, &pGhost->m_RenderInfo, -2, vec2(), IntraTick);
+		m_pClient->m_pPlayers->RenderHook(&Prev, &Player, &pGhost->m_RenderInfo , -2, IntraTick);
+		m_pClient->m_pPlayers->RenderPlayer(&Prev, &Player, &pGhost->m_RenderInfo, -2, IntraTick);
 	}
 }
 
@@ -358,18 +361,16 @@ void CGhost::InitRenderInfos(CGhostItem *pGhost)
 	if(pGhost->m_Skin.m_UseCustomColor)
 	{
 		pRenderInfo->m_Texture = m_pClient->m_pSkins->Get(SkinId)->m_ColorTexture;
-		pRenderInfo->m_ColorBody = m_pClient->m_pSkins->GetColorV4(pGhost->m_Skin.m_ColorBody);
-		pRenderInfo->m_ColorFeet = m_pClient->m_pSkins->GetColorV4(pGhost->m_Skin.m_ColorFeet);
+		pRenderInfo->m_ColorBody = color_cast<ColorRGBA>(ColorHSLA(pGhost->m_Skin.m_ColorBody));
+		pRenderInfo->m_ColorFeet = color_cast<ColorRGBA>(ColorHSLA(pGhost->m_Skin.m_ColorFeet));
 	}
 	else
 	{
 		pRenderInfo->m_Texture = m_pClient->m_pSkins->Get(SkinId)->m_OrgTexture;
-		pRenderInfo->m_ColorBody = vec4(1, 1, 1, 1);
-		pRenderInfo->m_ColorFeet = vec4(1, 1, 1, 1);
+		pRenderInfo->m_ColorBody = ColorRGBA(1, 1, 1);
+		pRenderInfo->m_ColorFeet = ColorRGBA(1, 1, 1);
 	}
 
-	pRenderInfo->m_ColorBody.a = 0.5f;
-	pRenderInfo->m_ColorFeet.a = 0.5f;
 	pRenderInfo->m_Size = 64;
 }
 
@@ -623,3 +624,8 @@ void CGhost::OnMapLoad()
 	m_pClient->m_pMenus->GhostlistPopulate();
 	m_AllowRestart = false;
 }
+
+int CGhost::GetLastRaceTick()
+{
+	return m_LastRaceTick;
+}
