@@ -17,7 +17,7 @@ class CItems : public CComponent
 	void RenderProjectile(const CNetObj_Projectile *pCurrent, int ItemID);
 	void RenderPickup(const CNetObj_Pickup *pPrev, const CNetObj_Pickup *pCurrent);
 	void RenderFlag(const CNetObj_Flag *pPrev, const CNetObj_Flag *pCurrent, const CNetObj_GameData *pPrevGameData, const CNetObj_GameData *pCurGameData);
-	void RenderLaser(const struct CNetObj_Laser *pCurrent);
+	void RenderLaser(const struct CNetObj_Laser *pCurrent, bool IsPredicted = false);
 
 	int m_ItemsQuadContainerIndex;
 public:
@@ -26,6 +26,8 @@ public:
 	virtual void OnInit();
 
 	void AddExtraProjectile(CNetObj_Projectile *pProj);
+
+	void ReconstructSmokeTrail(const CNetObj_Projectile *pCurrent, int ItemID, int DestroyTick, int LifeSpan);
 };
 
 #endif
