@@ -13,6 +13,7 @@
 
 #include <game/client/render.h>
 #include <game/client/gameclient.h>
+#include <game/client/components/console.h>
 #include <game/localization.h>
 
 #include <game/client/ui.h>
@@ -24,14 +25,14 @@
 
 int CMenus::DoButton_DemoPlayer(const void *pID, const char *pText, int Checked, const CUIRect *pRect)
 {
-	RenderTools()->DrawUIRect(pRect, vec4(1,1,1, Checked ? 0.10f : 0.5f)*ButtonColorMul(pID), CUI::CORNER_ALL, 5.0f);
+	RenderTools()->DrawUIRect(pRect, ColorRGBA(1,1,1, (Checked ? 0.10f : 0.5f)*ButtonColorMul(pID)), CUI::CORNER_ALL, 5.0f);
 	UI()->DoLabel(pRect, pText, 14.0f, 0);
 	return UI()->DoButtonLogic(pID, pText, Checked, pRect);
 }
 
 int CMenus::DoButton_Sprite(const void *pID, int ImageID, int SpriteID, int Checked, const CUIRect *pRect, int Corners)
 {
-	RenderTools()->DrawUIRect(pRect, Checked ? vec4(1.0f, 1.0f, 1.0f, 0.10f) : vec4(1.0f, 1.0f, 1.0f, 0.5f)*ButtonColorMul(pID), Corners, 5.0f);
+	RenderTools()->DrawUIRect(pRect, ColorRGBA(1.0f, 1.0f, 1.0f, (Checked ? 0.10f : 0.5f)*ButtonColorMul(pID)), Corners, 5.0f);
 	Graphics()->TextureSet(g_pData->m_aImages[ImageID].m_Id);
 	Graphics()->QuadsBegin();
 	if(!Checked)
@@ -80,14 +81,10 @@ void CMenus::RenderDemoPlayer(CUIRect MainView)
 		CUIRect Box, Part, Part2;
 		Box = Screen;
 		Box.VMargin(150.0f/UI()->Scale(), &Box);
-#if defined(__ANDROID__)
-		Box.HMargin(100.0f/UI()->Scale(), &Box);
-#else
 		Box.HMargin(150.0f/UI()->Scale(), &Box);
-#endif
 
 		// render the box
-		RenderTools()->DrawUIRect(&Box, vec4(0,0,0,0.5f), CUI::CORNER_ALL, 15.0f);
+		RenderTools()->DrawUIRect(&Box, ColorRGBA(0,0,0,0.5f), CUI::CORNER_ALL, 15.0f);
 
 		Box.HSplitTop(20.f/UI()->Scale(), &Part, &Box);
 		Box.HSplitTop(24.f/UI()->Scale(), &Part, &Box);
@@ -101,11 +98,7 @@ void CMenus::RenderDemoPlayer(CUIRect MainView)
 		CUIRect Label, TextBox, Ok, Abort;
 
 		Box.HSplitBottom(20.f, &Box, &Part);
-#if defined(__ANDROID__)
-		Box.HSplitBottom(60.f, &Box, &Part);
-#else
 		Box.HSplitBottom(24.f, &Box, &Part);
-#endif
 		Part.VMargin(80.0f, &Part);
 
 		Part.VSplitMid(&Abort, &Ok);
@@ -149,13 +142,8 @@ void CMenus::RenderDemoPlayer(CUIRect MainView)
 
 		Box.HSplitBottom(60.f, &Box, &Part);
 		Box.HSplitBottom(60.f, &Box, &Part2);
-#if defined(__ANDROID__)
-		Box.HSplitBottom(60.f, &Box, &Part2);
-		Box.HSplitBottom(60.f, &Box, &Part);
-#else
 		Box.HSplitBottom(24.f, &Box, &Part2);
 		Box.HSplitBottom(24.f, &Box, &Part);
-#endif
 
 		Part2.VSplitLeft(60.0f, 0, &Label);
 		if(DoButton_CheckBox(&s_RemoveChat, Localize("Remove chat"), s_RemoveChat, &Label))
@@ -175,16 +163,72 @@ void CMenus::RenderDemoPlayer(CUIRect MainView)
 		}
 	}
 
-	// handle mousewheel independent of active menu
-	if(Input()->KeyPress(KEY_MOUSE_WHEEL_UP))
+	// handle keyboard shortcuts independent of active menu
+	if(m_pClient->m_pGameConsole->IsClosed() && m_DemoPlayerState == DEMOPLAYER_NONE)
 	{
-		DemoPlayer()->SetSpeedIndex(+1);
-		LastSpeedChange = time_get();
-	}
-	else if(Input()->KeyPress(KEY_MOUSE_WHEEL_DOWN))
-	{
-		DemoPlayer()->SetSpeedIndex(-1);
-		LastSpeedChange = time_get();
+		// increase/decrease speed
+		if(Input()->KeyPress(KEY_MOUSE_WHEEL_UP) || Input()->KeyPress(KEY_UP))
+		{
+			DemoPlayer()->SetSpeedIndex(+1);
+			LastSpeedChange = time_get();
+		}
+		else if(Input()->KeyPress(KEY_MOUSE_WHEEL_DOWN) || Input()->KeyPress(KEY_DOWN))
+		{
+			DemoPlayer()->SetSpeedIndex(-1);
+			LastSpeedChange = time_get();
+		}
+
+		// pause/unpause
+		if(Input()->KeyPress(KEY_SPACE) || Input()->KeyPress(KEY_RETURN) || Input()->KeyPress(KEY_K))
+		{
+			if(pInfo->m_Paused)
+			{
+				DemoPlayer()->Unpause();
+			}
+			else
+			{
+				DemoPlayer()->Pause();
+			}
+		}
+
+		// seek backward/forward 10/5 seconds
+		if(Input()->KeyPress(KEY_J))
+		{
+			DemoPlayer()->SeekTime(-10.0f);
+		}
+		else if(Input()->KeyPress(KEY_L))
+		{
+			DemoPlayer()->SeekTime(10.0f);
+		}
+		else if(Input()->KeyPress(KEY_LEFT))
+		{
+			DemoPlayer()->SeekTime(-5.0f);
+		}
+		else if(Input()->KeyPress(KEY_RIGHT))
+		{
+			DemoPlayer()->SeekTime(5.0f);
+		}
+
+		// seek to 0-90%
+		const int SeekPercentKeys[] = {KEY_0, KEY_1, KEY_2, KEY_3, KEY_4, KEY_5, KEY_6, KEY_7, KEY_8, KEY_9};
+		for(unsigned i = 0; i < sizeof(SeekPercentKeys) / sizeof(SeekPercentKeys[0]); i++)
+		{
+			if(Input()->KeyPress(SeekPercentKeys[i]))
+			{
+				DemoPlayer()->SeekPercent(i * 0.1f);
+				break;
+			}
+		}
+
+		// seek to the beginning/end
+		if(Input()->KeyPress(KEY_HOME))
+		{
+			DemoPlayer()->SeekPercent(0.0f);
+		}
+		else if(Input()->KeyPress(KEY_END))
+		{
+			DemoPlayer()->SeekPercent(1.0f);
+		}
 	}
 
 	TotalHeight = SeekBarHeight+ButtonbarHeight+NameBarHeight+Margins*3;
@@ -227,13 +271,13 @@ void CMenus::RenderDemoPlayer(CUIRect MainView)
 		char aBuffer[128];
 
 		// draw seek bar
-		RenderTools()->DrawUIRect(&SeekBar, vec4(0,0,0,0.5f), CUI::CORNER_ALL, 5.0f);
+		RenderTools()->DrawUIRect(&SeekBar, ColorRGBA(0,0,0,0.5f), CUI::CORNER_ALL, 5.0f);
 
 		// draw filled bar
 		float Amount = CurrentTick/(float)TotalTicks;
 		CUIRect FilledBar = SeekBar;
 		FilledBar.w = 10.0f + (FilledBar.w-10.0f)*Amount;
-		RenderTools()->DrawUIRect(&FilledBar, vec4(1,1,1,0.5f), CUI::CORNER_ALL, 5.0f);
+		RenderTools()->DrawUIRect(&FilledBar, ColorRGBA(1,1,1,0.5f), CUI::CORNER_ALL, 5.0f);
 
 		// draw markers
 		for(int i = 0; i < pInfo->m_NumTimelineMarkers; i++)
@@ -288,7 +332,7 @@ void CMenus::RenderDemoPlayer(CUIRect MainView)
 			else
 			{
 				static float PrevAmount = 0.0f;
-				float Amount = (UI()->MouseX()-SeekBar.x)/(float)SeekBar.w;
+				float Amount = (UI()->MouseX()-SeekBar.x)/SeekBar.w;
 
 				if(Input()->KeyIsPressed(KEY_LSHIFT) || Input()->KeyIsPressed(KEY_RSHIFT))
 				{
@@ -299,7 +343,7 @@ void CMenus::RenderDemoPlayer(CUIRect MainView)
 						//PrevAmount = Amount;
 						m_pClient->OnReset();
 						m_pClient->m_SuppressEvents = true;
-						DemoPlayer()->SetPos(Amount);
+						DemoPlayer()->SeekPercent(Amount);
 						m_pClient->m_SuppressEvents = false;
 						m_pClient->m_pMapLayersBackGround->EnvelopeUpdate();
 						m_pClient->m_pMapLayersForeGround->EnvelopeUpdate();
@@ -312,7 +356,7 @@ void CMenus::RenderDemoPlayer(CUIRect MainView)
 						PrevAmount = Amount;
 						m_pClient->OnReset();
 						m_pClient->m_SuppressEvents = true;
-						DemoPlayer()->SetPos(Amount);
+						DemoPlayer()->SeekPercent(Amount);
 						m_pClient->m_SuppressEvents = false;
 						m_pClient->m_pMapLayersBackGround->EnvelopeUpdate();
 						m_pClient->m_pMapLayersForeGround->EnvelopeUpdate();
@@ -334,7 +378,7 @@ void CMenus::RenderDemoPlayer(CUIRect MainView)
 	{
 		m_pClient->OnReset();
 		DemoPlayer()->Pause();
-		DemoPlayer()->SetPos(0);
+		DemoPlayer()->SeekPercent(0.0f);
 	}
 
 	bool IncreaseDemoSpeed = false, DecreaseDemoSpeed = false;
@@ -345,15 +389,16 @@ void CMenus::RenderDemoPlayer(CUIRect MainView)
 	// combined play and pause button
 	ButtonBar.VSplitLeft(ButtonbarHeight, &Button, &ButtonBar);
 	static int s_PlayPauseButton = 0;
-	if(!pInfo->m_Paused)
+	if(DoButton_Sprite(&s_PlayPauseButton, IMAGE_DEMOBUTTONS, pInfo->m_Paused ? SPRITE_DEMOBUTTON_PLAY : SPRITE_DEMOBUTTON_PAUSE, false, &Button, CUI::CORNER_ALL))
 	{
-		if(DoButton_Sprite(&s_PlayPauseButton, IMAGE_DEMOBUTTONS, SPRITE_DEMOBUTTON_PAUSE, false, &Button, CUI::CORNER_ALL))
-			DemoPlayer()->Pause();
-	}
-	else
-	{
-		if(DoButton_Sprite(&s_PlayPauseButton, IMAGE_DEMOBUTTONS, SPRITE_DEMOBUTTON_PLAY, false, &Button, CUI::CORNER_ALL))
+		if(pInfo->m_Paused)
+		{
 			DemoPlayer()->Unpause();
+		}
+		else
+		{
+			DemoPlayer()->Pause();
+		}
 	}
 
 	// stop button
@@ -365,7 +410,7 @@ void CMenus::RenderDemoPlayer(CUIRect MainView)
 	{
 		m_pClient->OnReset();
 		DemoPlayer()->Pause();
-		DemoPlayer()->SetPos(0);
+		DemoPlayer()->SeekPercent(0.0f);
 	}
 
 	// slowdown
@@ -466,24 +511,20 @@ void CMenus::UiDoListboxStart(const void *pID, const CUIRect *pRect, float RowHe
 
 	// draw header
 	View.HSplitTop(ms_ListheaderHeight, &Header, &View);
-	RenderTools()->DrawUIRect(&Header, vec4(1,1,1,0.25f), CUI::CORNER_T, 5.0f);
+	RenderTools()->DrawUIRect(&Header, ColorRGBA(1,1,1,0.25f), CUI::CORNER_T, 5.0f);
 	UI()->DoLabel(&Header, pTitle, Header.h*ms_FontmodHeight, 0);
 
 	// draw footers
 	View.HSplitBottom(ms_ListheaderHeight, &View, &Footer);
-	RenderTools()->DrawUIRect(&Footer, vec4(1,1,1,0.25f), CUI::CORNER_B, 5.0f);
+	RenderTools()->DrawUIRect(&Footer, ColorRGBA(1,1,1,0.25f), CUI::CORNER_B, 5.0f);
 	Footer.VSplitLeft(10.0f, 0, &Footer);
 	UI()->DoLabel(&Footer, pBottomText, Header.h*ms_FontmodHeight, 0);
 
 	// background
-	RenderTools()->DrawUIRect(&View, vec4(0,0,0,0.15f), 0, 0);
+	RenderTools()->DrawUIRect(&View, ColorRGBA(0,0,0,0.15f), 0, 0);
 
 	// prepare the scroll
-#if defined(__ANDROID__)
-	View.VSplitRight(50, &View, &Scroll);
-#else
 	View.VSplitRight(15, &View, &Scroll);
-#endif
 
 	// setup the variables
 	gs_ListBoxOriginalView = View;
@@ -618,8 +659,8 @@ CMenus::CListboxItem CMenus::UiDoListboxNextItem(const void *pId, bool Selected,
 					{
 						if(m_aInputEvents[i].m_Key == KEY_DOWN) NewIndex = gs_ListBoxNewSelected + 1;
 						else if(m_aInputEvents[i].m_Key == KEY_UP) NewIndex = gs_ListBoxNewSelected - 1;
-						else if(m_aInputEvents[i].m_Key == KEY_PAGEUP) NewIndex = max(gs_ListBoxNewSelected - 20, 0);
-						else if(m_aInputEvents[i].m_Key == KEY_PAGEDOWN) NewIndex = min(gs_ListBoxNewSelected + 20, gs_ListBoxNumItems - 1);
+						else if(m_aInputEvents[i].m_Key == KEY_PAGEUP) NewIndex = maximum(gs_ListBoxNewSelected - 20, 0);
+						else if(m_aInputEvents[i].m_Key == KEY_PAGEDOWN) NewIndex = minimum(gs_ListBoxNewSelected + 20, gs_ListBoxNumItems - 1);
 						else if(m_aInputEvents[i].m_Key == KEY_HOME) NewIndex = 0;
 						else if(m_aInputEvents[i].m_Key == KEY_END) NewIndex = gs_ListBoxNumItems - 1;
 					}
@@ -657,7 +698,7 @@ CMenus::CListboxItem CMenus::UiDoListboxNextItem(const void *pId, bool Selected,
 		//selected_index = i;
 		CUIRect r = Item.m_Rect;
 		r.Margin(1.5f, &r);
-		RenderTools()->DrawUIRect(&r, vec4(1,1,1,0.5f), CUI::CORNER_ALL, 4.0f);
+		RenderTools()->DrawUIRect(&r, ColorRGBA(1,1,1,0.5f), CUI::CORNER_ALL, 4.0f);
 	}
 
 	return Item;
@@ -690,11 +731,12 @@ int CMenus::DemolistFetchCallback(const char *pName, time_t Date, int IsDir, int
 	if(IsDir)
 	{
 		str_format(Item.m_aName, sizeof(Item.m_aName), "%s/", pName);
+		Item.m_InfosLoaded = false;
 		Item.m_Valid = false;
 	}
 	else
 	{
-		str_copy(Item.m_aName, pName, min(static_cast<int>(sizeof(Item.m_aName)), str_length(pName) - 4));
+		str_copy(Item.m_aName, pName, minimum(static_cast<int>(sizeof(Item.m_aName)), str_length(pName) - 4));
 		Item.m_InfosLoaded = false;
 		Item.m_Date = Date;
 	}
@@ -711,6 +753,10 @@ void CMenus::DemolistPopulate()
 	if(!str_comp(m_aCurrentDemoFolder, "demos"))
 		m_DemolistStorageType = IStorage::TYPE_ALL;
 	Storage()->ListDirectoryInfo(m_DemolistStorageType, m_aCurrentDemoFolder, DemolistFetchCallback, this);
+
+	if(g_Config.m_BrDemoFetchInfo)
+		FetchAllHeaders();
+
 	m_lDemos.sort_range();
 }
 
@@ -755,6 +801,15 @@ bool CMenus::FetchHeader(CDemoItem &Item)
 	return Item.m_Valid;
 }
 
+void CMenus::FetchAllHeaders()
+{
+	for(sorted_array<CDemoItem>::range r = m_lDemos.all(); !r.empty(); r.pop_front())
+	{
+		FetchHeader(r.front());
+	}
+	m_lDemos.sort_range();
+}
+
 void CMenus::RenderDemoList(CUIRect MainView)
 {
 	static int s_Inited = 0;
@@ -780,28 +835,28 @@ void CMenus::RenderDemoList(CUIRect MainView)
 	}
 
 	// render background
-	RenderTools()->DrawUIRect(&MainView, ms_ColorTabbarActive, CUI::CORNER_ALL, 10.0f);
+	RenderTools()->DrawUIRect(&MainView, ms_ColorTabbarActive, CUI::CORNER_B, 10.0f);
 	MainView.Margin(10.0f, &MainView);
 
 	CUIRect ButtonBar, RefreshRect, FetchRect, PlayRect, DeleteRect, RenameRect, LabelRect, ListBox;
 	MainView.HSplitBottom(ms_ButtonHeight+5.0f, &MainView, &ButtonBar);
 	ButtonBar.HSplitTop(5.0f, 0, &ButtonBar);
-	ButtonBar.VSplitRight(130.0f, &ButtonBar, &PlayRect);
-	ButtonBar.VSplitLeft(130.0f, &RefreshRect, &ButtonBar);
+	ButtonBar.VSplitRight(110.0f, &ButtonBar, &PlayRect);
+	ButtonBar.VSplitLeft(110.0f, &RefreshRect, &ButtonBar);
 	ButtonBar.VSplitLeft(10.0f, 0, &ButtonBar);
-	ButtonBar.VSplitLeft(130.0f, &FetchRect, &ButtonBar);
+	ButtonBar.VSplitLeft(110.0f, &FetchRect, &ButtonBar);
 	ButtonBar.VSplitLeft(10.0f, 0, &ButtonBar);
-	ButtonBar.VSplitLeft(120.0f, &DeleteRect, &ButtonBar);
+	ButtonBar.VSplitLeft(110.0f, &DeleteRect, &ButtonBar);
 	ButtonBar.VSplitLeft(10.0f, 0, &ButtonBar);
-	ButtonBar.VSplitLeft(120.0f, &RenameRect, &ButtonBar);
+	ButtonBar.VSplitLeft(110.0f, &RenameRect, &ButtonBar);
 	ButtonBar.VSplitLeft(10.0f, 0, &ButtonBar);
-	ButtonBar.VSplitLeft(120.0f, &LabelRect, &ButtonBar);
+	ButtonBar.VSplitLeft(110.0f, &LabelRect, &ButtonBar);
 	MainView.HSplitBottom(140.0f, &ListBox, &MainView);
 
 	// render demo info
 	MainView.VMargin(5.0f, &MainView);
 	MainView.HSplitBottom(5.0f, &MainView, 0);
-	RenderTools()->DrawUIRect(&MainView, vec4(0,0,0,0.15f), CUI::CORNER_B, 4.0f);
+	RenderTools()->DrawUIRect(&MainView, ColorRGBA(0,0,0,0.15f), CUI::CORNER_B, 4.0f);
 	if(!m_DemolistSelectedIsDir && m_DemolistSelectedIndex >= 0 && m_lDemos[m_DemolistSelectedIndex].m_Valid)
 	{
 		CUIRect Left, Right, Labels;
@@ -912,7 +967,7 @@ void CMenus::RenderDemoList(CUIRect MainView)
 		{COL_DATE,     SORT_DATE,     Localize("Date"),     1, 160.0f, 1, {0}, {0}},
 	};
 
-	RenderTools()->DrawUIRect(&Headers, vec4(0.0f,0,0,0.15f), 0, 0);
+	RenderTools()->DrawUIRect(&Headers, ColorRGBA(0.0f,0,0,0.15f), 0, 0);
 
 	int NumCols = sizeof(s_aCols)/sizeof(CColumn);
 
@@ -968,11 +1023,7 @@ void CMenus::RenderDemoList(CUIRect MainView)
 
 	// scrollbar
 	CUIRect Scroll;
-#if defined(__ANDROID__)
-	ListBox.VSplitRight(50, &ListBox, &Scroll);
-#else
 	ListBox.VSplitRight(15, &ListBox, &Scroll);
-#endif
 
 	int Num = (int)(ListBox.h/s_aCols[0].m_Rect.h) + 1;
 	static int s_ScrollBar = 0;
@@ -1006,8 +1057,8 @@ void CMenus::RenderDemoList(CUIRect MainView)
 			{
 				if(m_aInputEvents[i].m_Key == KEY_DOWN) NewIndex = m_DemolistSelectedIndex + 1;
 				else if(m_aInputEvents[i].m_Key == KEY_UP) NewIndex = m_DemolistSelectedIndex - 1;
-				else if(m_aInputEvents[i].m_Key == KEY_PAGEUP) NewIndex = max(m_DemolistSelectedIndex - 20, 0);
-				else if(m_aInputEvents[i].m_Key == KEY_PAGEDOWN) NewIndex = min(m_DemolistSelectedIndex + 20, m_lDemos.size() - 1);
+				else if(m_aInputEvents[i].m_Key == KEY_PAGEUP) NewIndex = maximum(m_DemolistSelectedIndex - 20, 0);
+				else if(m_aInputEvents[i].m_Key == KEY_PAGEDOWN) NewIndex = minimum(m_DemolistSelectedIndex + 20, m_lDemos.size() - 1);
 				else if(m_aInputEvents[i].m_Key == KEY_HOME) NewIndex = 0;
 				else if(m_aInputEvents[i].m_Key == KEY_END) NewIndex = m_lDemos.size() - 1;
 			}
@@ -1048,9 +1099,6 @@ void CMenus::RenderDemoList(CUIRect MainView)
 	ListBox.y -= s_ScrollValue*ScrollNum*s_aCols[0].m_Rect.h;
 
 	int NewSelected = -1;
-#if defined(__ANDROID__)
-	int DoubleClicked = 0;
-#endif
 	int ItemIndex = -1;
 
 	for(sorted_array<CDemoItem>::range r = m_lDemos.all(); !r.empty(); r.pop_front())
@@ -1072,7 +1120,7 @@ void CMenus::RenderDemoList(CUIRect MainView)
 			{
 				CUIRect r = Row;
 				r.Margin(1.5f, &r);
-				RenderTools()->DrawUIRect(&r, vec4(1,1,1,0.5f), CUI::CORNER_ALL, 4.0f);
+				RenderTools()->DrawUIRect(&r, ColorRGBA(1,1,1,0.5f), CUI::CORNER_ALL, 4.0f);
 			}
 
 			// clip the selection
@@ -1089,11 +1137,6 @@ void CMenus::RenderDemoList(CUIRect MainView)
 				NewSelected = ItemIndex;
 				str_copy(g_Config.m_UiDemoSelected, r.front().m_aName, sizeof(g_Config.m_UiDemoSelected));
 				DemolistOnUpdate(false);
-#if defined(__ANDROID__)
-				if(NewSelected == m_DoubleClickIndex)
-					DoubleClicked = 1;
-#endif
-
 				m_DoubleClickIndex = NewSelected;
 			}
 		}
@@ -1156,11 +1199,7 @@ void CMenus::RenderDemoList(CUIRect MainView)
 
 	bool Activated = false;
 
-#if defined(__ANDROID__)
-	if(m_EnterPressed || (DoubleClicked && UI()->HotItem() == m_lDemos[m_DemolistSelectedIndex].m_aName))
-#else
 	if(m_EnterPressed || (Input()->MouseDoubleClick() && UI()->HotItem() == m_lDemos[m_DemolistSelectedIndex].m_aName))
-#endif
 	{
 		UI()->SetActiveItem(0);
 		Activated = true;
@@ -1173,14 +1212,11 @@ void CMenus::RenderDemoList(CUIRect MainView)
 		DemolistOnUpdate(false);
 	}
 
-	static int s_FetchButton = 0;
-	if(DoButton_Menu(&s_FetchButton, Localize("Fetch Info"), 0, &FetchRect) || (Input()->KeyPress(KEY_F) && (Input()->KeyIsPressed(KEY_LCTRL) || Input()->KeyIsPressed(KEY_RCTRL))))
+	if(DoButton_CheckBox(&g_Config.m_BrDemoFetchInfo, Localize("Fetch Info"), g_Config.m_BrDemoFetchInfo, &FetchRect))
 	{
-		for(sorted_array<CDemoItem>::range r = m_lDemos.all(); !r.empty(); r.pop_front())
-		{
-			FetchHeader(r.front());
-		}
-		m_lDemos.sort_range();
+		g_Config.m_BrDemoFetchInfo ^= 1;
+		if(g_Config.m_BrDemoFetchInfo)
+			FetchAllHeaders();
 	}
 
 	static int s_PlayButton = 0;
