@@ -28,7 +28,7 @@
 #include "players.h"
 #include <stdio.h>
 
-void CPlayers::RenderHand(CTeeRenderInfo *pInfo, vec2 CenterPos, vec2 Dir, float AngleOffset, vec2 PostRotOffset)
+void CPlayers::RenderHand(CTeeRenderInfo *pInfo, vec2 CenterPos, vec2 Dir, float AngleOffset, vec2 PostRotOffset, float Alpha)
 {
 	vec2 HandPos = CenterPos + Dir;
 	float Angle = GetAngle(Dir);
@@ -48,7 +48,7 @@ void CPlayers::RenderHand(CTeeRenderInfo *pInfo, vec2 CenterPos, vec2 Dir, float
 
 	//Graphics()->TextureSet(data->m_aImages[IMAGE_CHAR_DEFAULT].id);
 	Graphics()->TextureSet(pInfo->m_Texture);
-	Graphics()->SetColor(pInfo->m_ColorBody.r, pInfo->m_ColorBody.g, pInfo->m_ColorBody.b, pInfo->m_ColorBody.a);
+	Graphics()->SetColor(pInfo->m_ColorBody.r, pInfo->m_ColorBody.g, pInfo->m_ColorBody.b, Alpha);
 
 	// two passes
 	for(int i = 0; i < 2; i++)
@@ -77,132 +77,18 @@ inline float AngularApproach(float Src, float Dst, float Amount)
 	return n;
 }
 
-void CPlayers::Predict(
-	const CNetObj_Character *pPrevChar,
-	const CNetObj_Character *pPlayerChar,
-	const CNetObj_PlayerInfo *pPrevInfo,
-	const CNetObj_PlayerInfo *pPlayerInfo,
-	vec2 &PrevPredPos,
-	vec2 &SmoothPos,
-	int &MoveCnt,
-	vec2 &Position
-	)
+bool CPlayers::IsOtherTeam(int ClientID)
 {
-	CNetObj_Character Prev;
-	CNetObj_Character Player;
-	Prev = *pPrevChar;
-	Player = *pPlayerChar;
+	bool Local = m_pClient->m_Snap.m_LocalClientID == ClientID;
 
-	CNetObj_PlayerInfo pInfo = *pPlayerInfo;
+	if((m_pClient->m_aClients[m_pClient->m_Snap.m_LocalClientID].m_Team == TEAM_SPECTATORS && m_pClient->m_Snap.m_SpecInfo.m_SpectatorID == SPEC_FREEVIEW) || ClientID < 0)
+		return false;
+	else if(m_pClient->m_Snap.m_SpecInfo.m_Active && m_pClient->m_Snap.m_SpecInfo.m_SpectatorID != SPEC_FREEVIEW)
+		return m_pClient->m_Teams.Team(ClientID) != m_pClient->m_Teams.Team(m_pClient->m_Snap.m_SpecInfo.m_SpectatorID);
+	else if((m_pClient->m_aClients[m_pClient->m_Snap.m_LocalClientID].m_Solo || m_pClient->m_aClients[ClientID].m_Solo) && !Local)
+		return true;
 
-
-	// set size
-
-	float IntraTick = Client()->IntraGameTick();
-
-
-	//float angle = 0;
-
-	if(pInfo.m_Local && Client()->State() != IClient::STATE_DEMOPLAYBACK)
-	{
-		// just use the direct input if it's local player we are rendering
-	}
-	else
-	{
-		/*
-		float mixspeed = Client()->FrameTime()*2.5f;
-		if(player.attacktick != prev.attacktick) // shooting boosts the mixing speed
-			mixspeed *= 15.0f;
-
-		// move the delta on a constant speed on a x^2 curve
-		float current = g_GameClient.m_aClients[info.cid].angle;
-		float target = player.angle/256.0f;
-		float delta = angular_distance(current, target);
-		float sign = delta < 0 ? -1 : 1;
-		float new_delta = delta - 2*mixspeed*sqrt(delta*sign)*sign + mixspeed*mixspeed;
-
-		// make sure that it doesn't vibrate when it's still
-		if(fabs(delta) < 2/256.0f)
-			angle = target;
-		else
-			angle = angular_approach(current, target, fabs(delta-new_delta));
-
-		g_GameClient.m_aClients[info.cid].angle = angle;*/
-	}
-
-	vec2 NonPredPos = mix(vec2(Prev.m_X, Prev.m_Y), vec2(Player.m_X, Player.m_Y), IntraTick);
-
-	// use preditect players if needed
-	if(g_Config.m_ClPredict && Client()->State() != IClient::STATE_DEMOPLAYBACK)
-	{
-		if(m_pClient->m_Snap.m_pLocalCharacter && !(m_pClient->m_Snap.m_pGameInfoObj && m_pClient->m_Snap.m_pGameInfoObj->m_GameStateFlags&GAMESTATEFLAG_GAMEOVER))
-		{
-			// apply predicted results
-			m_pClient->m_aClients[pInfo.m_ClientID].m_Predicted.Write(&Player);
-			m_pClient->m_aClients[pInfo.m_ClientID].m_PrevPredicted.Write(&Prev);
-
-			IntraTick = Client()->PredIntraGameTick();
-		}
-	}
-
-	Position = mix(vec2(Prev.m_X, Prev.m_Y), vec2(Player.m_X, Player.m_Y), IntraTick);
-
-
-	static double s_Ping = 0;
-
-	if(pInfo.m_Local) {
-		s_Ping = mix(s_Ping, (double)pInfo.m_Latency, 0.1);
-	}
-
-	if(!pInfo.m_Local)
-	{
-		/*
-		for ping = 260, usual missprediction distances:
-
-		move = 120-140
-		jump = 130
-		dj = 250
-
-		normalized:
-		move = 0.461 - 0.538
-		jump = 0.5
-		dj = .961
-
-		*/
-		//printf("%d\n", m_pClient->m_Snap.m_pLocalInfo->m_Latency);
-
-
-		if(m_pClient->m_Snap.m_pLocalInfo)
-			s_Ping = mix(s_Ping, (double)m_pClient->m_Snap.m_pLocalInfo->m_Latency, 0.1);
-
-		double d = length(PrevPredPos - Position)/s_Ping;
-
-		if((d > 0.4) && (d < 5.))
-		{
-//			if(MoveCnt == 0)
-//				printf("[\n");
-			if(MoveCnt == 0)
-				SmoothPos = NonPredPos;
-
-			MoveCnt = 10;
-//		SmoothPos = PrevPredPos;
-//		SmoothPos = mix(NonPredPos, Position, 0.6);
-		}
-
-		PrevPredPos = Position;
-
-		if(MoveCnt > 0)
-		{
-//		Position = mix(mix(NonPredPos, Position, 0.5), SmoothPos, (((float)MoveCnt))/15);
-//		Position = mix(mix(NonPredPos, Position, 0.5), SmoothPos, 0.5);
-			Position = mix(NonPredPos, Position, 0.5);
-
-			SmoothPos = Position;
-			MoveCnt--;
-//		if(MoveCnt == 0)
-//			printf("]\n\n");
-		}
-	}
+	return m_pClient->m_Teams.Team(ClientID) != m_pClient->m_Teams.Team(m_pClient->m_Snap.m_LocalClientID);
 }
 
 void CPlayers::RenderHook(
@@ -210,8 +96,6 @@ void CPlayers::RenderHook(
 	const CNetObj_Character *pPlayerChar,
 	const CTeeRenderInfo *pRenderInfo,
 	int ClientID,
-	const vec2 &parPosition,
-	const vec2 &PositionTo,
 	float Intra
 	)
 {
@@ -222,80 +106,29 @@ void CPlayers::RenderHook(
 
 	CTeeRenderInfo RenderInfo = *pRenderInfo;
 
-	bool AntiPingPlayers = m_pClient->AntiPingPlayers();
-	bool Local = m_pClient->m_Snap.m_LocalClientID == ClientID;
-
 	// don't render hooks to not active character cores
 	if(pPlayerChar->m_HookedPlayer != -1 && !m_pClient->m_Snap.m_aCharacters[pPlayerChar->m_HookedPlayer].m_Active)
 		return;
 
-	float IntraTick = Client()->IntraGameTick();
-	if(ClientID < 0)
-	{
-		IntraTick = Intra;
-		AntiPingPlayers = false;
-	}
+	float IntraTick = Intra;
+	if(ClientID >= 0)
+		IntraTick = (m_pClient->m_aClients[ClientID].m_IsPredicted) ? Client()->PredIntraGameTick() : Client()->IntraGameTick();
 
-	bool OtherTeam;
-
-	if((m_pClient->m_aClients[m_pClient->m_Snap.m_LocalClientID].m_Team == TEAM_SPECTATORS && m_pClient->m_Snap.m_SpecInfo.m_SpectatorID == SPEC_FREEVIEW) || ClientID < 0)
-		OtherTeam = false;
-	else if(m_pClient->m_Snap.m_SpecInfo.m_Active && m_pClient->m_Snap.m_SpecInfo.m_SpectatorID != SPEC_FREEVIEW)
-		OtherTeam = m_pClient->m_Teams.Team(ClientID) != m_pClient->m_Teams.Team(m_pClient->m_Snap.m_SpecInfo.m_SpectatorID);
-	else
-		OtherTeam = m_pClient->m_Teams.Team(ClientID) != m_pClient->m_Teams.Team(m_pClient->m_Snap.m_LocalClientID);
-
-	if(OtherTeam)
-	{
-		RenderInfo.m_ColorBody.a = g_Config.m_ClShowOthersAlpha / 100.0f;
-		RenderInfo.m_ColorFeet.a = g_Config.m_ClShowOthersAlpha / 100.0f;
-	}
+	bool OtherTeam = IsOtherTeam(ClientID);
 
 	RenderInfo.m_Size = 64.0f;
 
-	if(!AntiPingPlayers)
-	{
-		// use predicted players if needed
-		if(Local && g_Config.m_ClPredict && Client()->State() != IClient::STATE_DEMOPLAYBACK)
-		{
-			if(!m_pClient->m_Snap.m_pLocalCharacter || (m_pClient->m_Snap.m_pGameInfoObj && m_pClient->m_Snap.m_pGameInfoObj->m_GameStateFlags&GAMESTATEFLAG_GAMEOVER))
-			{
-			}
-			else
-			{
-				// apply predicted results
-				m_pClient->m_PredictedChar.Write(&Player);
-				m_pClient->m_PredictedPrevChar.Write(&Prev);
-				IntraTick = Client()->PredIntraGameTick();
-			}
-		}
-	}
-	else
-	{
-		if(g_Config.m_ClPredict && Client()->State() != IClient::STATE_DEMOPLAYBACK)
-		{
-			if(m_pClient->m_Snap.m_pLocalCharacter && !(m_pClient->m_Snap.m_pGameInfoObj && m_pClient->m_Snap.m_pGameInfoObj->m_GameStateFlags&GAMESTATEFLAG_GAMEOVER))
-			{
-				// apply predicted results
-				m_pClient->m_aClients[ClientID].m_Predicted.Write(&Player);
-				m_pClient->m_aClients[ClientID].m_PrevPredicted.Write(&Prev);
-
-				IntraTick = Client()->PredIntraGameTick();
-			}
-		}
-	}
-
 	vec2 Position;
-	if(!AntiPingPlayers)
-		Position = mix(vec2(Prev.m_X, Prev.m_Y), vec2(Player.m_X, Player.m_Y), IntraTick);
+	if(in_range(ClientID, MAX_CLIENTS-1))
+		Position = m_pClient->m_aClients[ClientID].m_RenderPos;
 	else
-		Position = parPosition;
+		Position = mix(vec2(Prev.m_X, Prev.m_Y), vec2(Player.m_X, Player.m_Y), IntraTick);
 
 	// draw hook
 	if(Prev.m_HookState>0 && Player.m_HookState>0)
 	{
 		Graphics()->TextureSet(g_pData->m_aImages[IMAGE_GAME].m_Id);
-		
+
 		Graphics()->SetColor(1.0f, 1.0f, 1.0f, 1.0f);
 		if(ClientID < 0)
 			Graphics()->SetColor(1.0f, 1.0f, 1.0f, 0.5f);
@@ -303,39 +136,10 @@ void CPlayers::RenderHook(
 		vec2 Pos = Position;
 		vec2 HookPos;
 
-		if(!AntiPingPlayers)
-		{
-			if(pPlayerChar->m_HookedPlayer != -1)
-			{
-				if(m_pClient->m_Snap.m_LocalClientID != -1 && pPlayerChar->m_HookedPlayer == m_pClient->m_Snap.m_LocalClientID && !m_pClient->m_Snap.m_SpecInfo.m_Active)
-				{
-					if(Client()->State() == IClient::STATE_DEMOPLAYBACK) // only use prediction if needed
-						HookPos = vec2(m_pClient->m_LocalCharacterPos.x, m_pClient->m_LocalCharacterPos.y);
-					else
-						HookPos = mix(vec2(m_pClient->m_PredictedPrevChar.m_Pos.x, m_pClient->m_PredictedPrevChar.m_Pos.y),
-							vec2(m_pClient->m_PredictedChar.m_Pos.x, m_pClient->m_PredictedChar.m_Pos.y), Client()->PredIntraGameTick());
-				}
-				else if(Local)
-				{
-					HookPos = mix(vec2(m_pClient->m_Snap.m_aCharacters[pPlayerChar->m_HookedPlayer].m_Prev.m_X,
-						m_pClient->m_Snap.m_aCharacters[pPlayerChar->m_HookedPlayer].m_Prev.m_Y),
-						vec2(m_pClient->m_Snap.m_aCharacters[pPlayerChar->m_HookedPlayer].m_Cur.m_X,
-						m_pClient->m_Snap.m_aCharacters[pPlayerChar->m_HookedPlayer].m_Cur.m_Y),
-						Client()->IntraGameTick());
-				}
-				else
-					HookPos = mix(vec2(pPrevChar->m_HookX, pPrevChar->m_HookY), vec2(pPlayerChar->m_HookX, pPlayerChar->m_HookY), Client()->IntraGameTick());
-			}
-			else
-				HookPos = mix(vec2(Prev.m_HookX, Prev.m_HookY), vec2(Player.m_HookX, Player.m_HookY), IntraTick);
-		}
+		if(in_range(pPlayerChar->m_HookedPlayer, MAX_CLIENTS-1))
+			HookPos = m_pClient->m_aClients[pPlayerChar->m_HookedPlayer].m_RenderPos;
 		else
-		{
-			if(pPrevChar->m_HookedPlayer != -1)
-				HookPos = PositionTo;
-			else
-				HookPos = mix(vec2(Prev.m_HookX, Prev.m_HookY), vec2(Player.m_HookX, Player.m_HookY), IntraTick);
-		}
+			HookPos = mix(vec2(Prev.m_HookX, Prev.m_HookY), vec2(Player.m_HookX, Player.m_HookY), IntraTick);
 
 		float d = distance(Pos, HookPos);
 		vec2 Dir = normalize(Pos-HookPos);
@@ -366,7 +170,7 @@ void CPlayers::RenderHook(
 		Graphics()->QuadsSetRotation(0);
 		Graphics()->SetColor(1.0f, 1.0f, 1.0f, 1.0f);
 
-		RenderHand(&RenderInfo, Position, normalize(HookPos-Pos), -pi/2, vec2(20, 0));
+		RenderHand(&RenderInfo, Position, normalize(HookPos-Pos), -pi/2, vec2(20, 0), OtherTeam ? g_Config.m_ClShowOthersAlpha / 100.0f : 1.0f);
 	}
 }
 
@@ -375,12 +179,8 @@ void CPlayers::RenderPlayer(
 	const CNetObj_Character *pPlayerChar,
 	const CTeeRenderInfo *pRenderInfo,
 	int ClientID,
-	const vec2 &parPosition,
 	float Intra
-/*	vec2 &PrevPos,
-	vec2 &SmoothPos,
-	int &MoveCnt
-*/	)
+	)
 {
 	CNetObj_Character Prev;
 	CNetObj_Character Player;
@@ -389,30 +189,36 @@ void CPlayers::RenderPlayer(
 
 	CTeeRenderInfo RenderInfo = *pRenderInfo;
 
-	bool AntiPingPlayers = m_pClient->AntiPingPlayers();
 	bool Local = m_pClient->m_Snap.m_LocalClientID == ClientID;
-
-	bool NewTick = m_pClient->m_NewTick;
-
-	bool OtherTeam;
-
-	if((m_pClient->m_aClients[m_pClient->m_Snap.m_LocalClientID].m_Team == TEAM_SPECTATORS && m_pClient->m_Snap.m_SpecInfo.m_SpectatorID == SPEC_FREEVIEW) || ClientID < 0)
-		OtherTeam = false;
-	else if(m_pClient->m_Snap.m_SpecInfo.m_Active && m_pClient->m_Snap.m_SpecInfo.m_SpectatorID != SPEC_FREEVIEW)
-		OtherTeam = m_pClient->m_Teams.Team(ClientID) != m_pClient->m_Teams.Team(m_pClient->m_Snap.m_SpecInfo.m_SpectatorID);
-	else
-		OtherTeam = m_pClient->m_Teams.Team(ClientID) != m_pClient->m_Teams.Team(m_pClient->m_Snap.m_LocalClientID);
+	bool OtherTeam = IsOtherTeam(ClientID);
+	float Alpha = OtherTeam ? g_Config.m_ClShowOthersAlpha / 100.0f : 1.0f;
 
 	// set size
 	RenderInfo.m_Size = 64.0f;
 
-	float IntraTick = Client()->IntraGameTick();
-	if(ClientID < 0)
+	float IntraTick = Intra;
+	if(ClientID >= 0)
+		IntraTick = m_pClient->m_aClients[ClientID].m_IsPredicted ? Client()->PredIntraGameTick() : Client()->IntraGameTick();
+
+	static float s_LastGameTickTime = Client()->GameTickTime();
+	static float s_LastPredIntraTick = Client()->PredIntraGameTick();
+	if(m_pClient->m_Snap.m_pGameInfoObj && !(m_pClient->m_Snap.m_pGameInfoObj->m_GameStateFlags&GAMESTATEFLAG_PAUSED))
 	{
-		IntraTick = Intra;
-		AntiPingPlayers = false;
+		s_LastGameTickTime = Client()->GameTickTime();
+		s_LastPredIntraTick = Client()->PredIntraGameTick();
 	}
 
+	bool PredictLocalWeapons = false;
+	float AttackTime = (Client()->PrevGameTick() - Player.m_AttackTick) / (float)SERVER_TICK_SPEED + Client()->GameTickTime();
+	float LastAttackTime = (Client()->PrevGameTick() - Player.m_AttackTick) / (float)SERVER_TICK_SPEED + s_LastGameTickTime;
+	if(Local && m_pClient->m_aClients[ClientID].m_IsPredicted && m_pClient->AntiPingWeapons() && m_pClient->AntiPingGrenade())
+	{
+		PredictLocalWeapons = true;
+		AttackTime = (Client()->PredIntraGameTick() + (Client()->PredGameTick() - 1 - Player.m_AttackTick)) / (float)SERVER_TICK_SPEED;
+		LastAttackTime = (s_LastPredIntraTick + (Client()->PredGameTick() - 1 - Player.m_AttackTick)) / (float)SERVER_TICK_SPEED;
+	}
+	float AttackTicksPassed = AttackTime*SERVER_TICK_SPEED;
+
 	float Angle;
 	if(Local && Client()->State() != IClient::STATE_DEMOPLAYBACK)
 	{
@@ -421,82 +227,33 @@ void CPlayers::RenderPlayer(
 	}
 	else
 	{
+		float AngleIntraTick = IntraTick;
+		// using unpredicted angle when rendering other players in-game
+		if(ClientID >= 0)
+			AngleIntraTick = Client()->IntraGameTick();
 		// If the player moves their weapon through top, then change
 		// the end angle by 2*Pi, so that the mix function will use the
 		// short path and not the long one.
 		if(Player.m_Angle > (256.0f * pi) && Prev.m_Angle < 0)
-		{
 			Player.m_Angle -= 256.0f * 2 * pi;
-			Angle = mix((float)Prev.m_Angle, (float)Player.m_Angle, IntraTick) / 256.0f;
-		}
 		else if(Player.m_Angle < 0 && Prev.m_Angle > (256.0f * pi))
-		{
 			Player.m_Angle += 256.0f * 2 * pi;
-			Angle = mix((float)Prev.m_Angle, (float)Player.m_Angle, IntraTick) / 256.0f;
-		}
-		else
-		{
-			// No special cases? Just use mix():
-			Angle = mix((float)Prev.m_Angle, (float)Player.m_Angle, IntraTick)/256.0f;
-		}
-	}
 
-
-	// use preditect players if needed
-	if(!AntiPingPlayers)
-	{
-		if(Local && g_Config.m_ClPredict && Client()->State() != IClient::STATE_DEMOPLAYBACK)
-		{
-			if(!m_pClient->m_Snap.m_pLocalCharacter || (m_pClient->m_Snap.m_pGameInfoObj && m_pClient->m_Snap.m_pGameInfoObj->m_GameStateFlags&GAMESTATEFLAG_GAMEOVER))
-			{
-			}
-			else
-			{
-				// apply predicted results
-				m_pClient->m_PredictedChar.Write(&Player);
-				m_pClient->m_PredictedPrevChar.Write(&Prev);
-				IntraTick = Client()->PredIntraGameTick();
-				NewTick = m_pClient->m_NewPredictedTick;
-			}
-		}
-	}
-	else
-	{
-		if(g_Config.m_ClPredict && Client()->State() != IClient::STATE_DEMOPLAYBACK)
-		{
-			if(m_pClient->m_Snap.m_pLocalCharacter && !(m_pClient->m_Snap.m_pGameInfoObj && m_pClient->m_Snap.m_pGameInfoObj->m_GameStateFlags&GAMESTATEFLAG_GAMEOVER))
-			{
-				// apply predicted results
-				m_pClient->m_aClients[ClientID].m_Predicted.Write(&Player);
-				m_pClient->m_aClients[ClientID].m_PrevPredicted.Write(&Prev);
-
-				IntraTick = Client()->PredIntraGameTick();
-				NewTick = m_pClient->m_NewPredictedTick;
-			}
-		}
+		Angle = mix((float)Prev.m_Angle, (float)Player.m_Angle, AngleIntraTick)/256.0f;
 	}
 
 	vec2 Direction = GetDirection((int)(Angle*256.0f));
 	vec2 Position;
-	if(!AntiPingPlayers)
-		Position = mix(vec2(Prev.m_X, Prev.m_Y), vec2(Player.m_X, Player.m_Y), IntraTick);
+	if(in_range(ClientID, MAX_CLIENTS-1))
+		Position = m_pClient->m_aClients[ClientID].m_RenderPos;
 	else
-		Position = parPosition;
+		Position = mix(vec2(Prev.m_X, Prev.m_Y), vec2(Player.m_X, Player.m_Y), IntraTick);
 	vec2 Vel = mix(vec2(Prev.m_VelX/256.0f, Prev.m_VelY/256.0f), vec2(Player.m_VelX/256.0f, Player.m_VelY/256.0f), IntraTick);
 
 	m_pClient->m_pFlow->Add(Position, Vel*100.0f, 10.0f);
 
 	RenderInfo.m_GotAirJump = Player.m_Jumped&2?0:1;
 
-
-	// detect events
-	if(NewTick)
-	{
-		// detect air jump
-		if(!RenderInfo.m_GotAirJump && !(Prev.m_Jumped&2))
-			m_pClient->m_pEffects->AirJump(Position);
-	}
-
 	bool Stationary = Player.m_VelX <= 1 && Player.m_VelX >= -1;
 	bool InAir = !Collision()->CheckPoint(Player.m_X, Player.m_Y+16);
 	bool WantOtherDir = (Player.m_Direction == -1 && Vel.x > 0) || (Player.m_Direction == 1 && Vel.x < 0);
@@ -513,19 +270,10 @@ void CPlayers::RenderPlayer(
 	else if(!WantOtherDir)
 		State.Add(&g_pData->m_aAnimations[ANIM_WALK], WalkTime, 1.0f);
 
-	static float s_LastGameTickTime = Client()->GameTickTime();
-	if(m_pClient->m_Snap.m_pGameInfoObj && !(m_pClient->m_Snap.m_pGameInfoObj->m_GameStateFlags&GAMESTATEFLAG_PAUSED))
-		s_LastGameTickTime = Client()->GameTickTime();
 	if(Player.m_Weapon == WEAPON_HAMMER)
-	{
-		float ct = (Client()->PrevGameTick()-Player.m_AttackTick)/(float)SERVER_TICK_SPEED + s_LastGameTickTime;
-		State.Add(&g_pData->m_aAnimations[ANIM_HAMMER_SWING], clamp(ct*5.0f,0.0f,1.0f), 1.0f);
-	}
+		State.Add(&g_pData->m_aAnimations[ANIM_HAMMER_SWING], clamp(LastAttackTime*5.0f,0.0f,1.0f), 1.0f);
 	if(Player.m_Weapon == WEAPON_NINJA)
-	{
-		float ct = (Client()->PrevGameTick()-Player.m_AttackTick)/(float)SERVER_TICK_SPEED + s_LastGameTickTime;
-		State.Add(&g_pData->m_aAnimations[ANIM_NINJA_SWING], clamp(ct*2.0f,0.0f,1.0f), 1.0f);
-	}
+		State.Add(&g_pData->m_aAnimations[ANIM_NINJA_SWING], clamp(LastAttackTime*2.0f,0.0f,1.0f), 1.0f);
 
 	// do skidding
 	if(!InAir && WantOtherDir && length(Vel*50) > 500.0f)
@@ -550,10 +298,6 @@ void CPlayers::RenderPlayer(
 		Client()->GetServerInfo(&Info);
 		if(ClientID >= 0 && (((IsDDRace(&Info) || IsDDNet(&Info)) && g_Config.m_ClShowHookCollAlways) || (Player.m_PlayerFlags&PLAYERFLAG_AIM && ((!Local && g_Config.m_ClShowHookCollOther) || (Local && g_Config.m_ClShowHookCollOwn)))))
 		{
-			float Alpha = 1.0f;
-			if(OtherTeam)
-				Alpha = g_Config.m_ClShowOthersAlpha / 100.0f;
-
 			vec2 ExDirection = Direction;
 
 			if(Local && Client()->State() != IClient::STATE_DEMOPLAYBACK)
@@ -587,12 +331,13 @@ void CPlayers::RenderPlayer(
 				int TeleNr = 0;
 				Hit = Collision()->IntersectLineTeleHook(OldPos, NewPos, &FinishPos, 0x0, &TeleNr);
 
-				if(!DoBreak && Hit) {
+				if(!DoBreak && Hit)
+				{
 					if(Hit != TILE_NOHOOK)
 						Graphics()->SetColor(130.0f/255.0f, 232.0f/255.0f, 160.0f/255.0f, Alpha);
 				}
 
-				if(m_pClient->m_Tuning[g_Config.m_ClDummy].m_PlayerHooking && m_pClient->IntersectCharacter(OldPos, FinishPos, FinishPos, ClientID) != -1)
+				if(m_pClient->IntersectCharacter(OldPos, FinishPos, FinishPos, ClientID) != -1)
 				{
 					Graphics()->SetColor(1.0f, 1.0f, 0.0f, Alpha);
 					break;
@@ -627,8 +372,7 @@ void CPlayers::RenderPlayer(
 		int iw = clamp(Player.m_Weapon, 0, NUM_WEAPONS-1);
 		int QuadOffset = iw * 2 + (Direction.x < 0 ? 1 : 0);
 
-		if(OtherTeam)
-			Graphics()->SetColor(1.0f, 1.0f, 1.0f, g_Config.m_ClShowOthersAlpha / 100.0f);
+		Graphics()->SetColor(1.0f, 1.0f, 1.0f, Alpha);
 
 		vec2 Dir = Direction;
 		float Recoil = 0.0f;
@@ -669,7 +413,7 @@ void CPlayers::RenderPlayer(
 			Graphics()->RenderQuadContainerAsSprite(m_WeaponEmoteQuadContainerIndex, QuadOffset, p.x, p.y);
 
 			// HADOKEN
-			if((Client()->GameTick()-Player.m_AttackTick) <= (SERVER_TICK_SPEED / 6) && g_pData->m_Weapons.m_aId[iw].m_NumSpriteMuzzles)
+			if(AttackTime <= 1/6.f && g_pData->m_Weapons.m_aId[iw].m_NumSpriteMuzzles)
 			{
 				int IteX = rand() % g_pData->m_Weapons.m_aId[iw].m_NumSpriteMuzzles;
 				static int s_LastIteX = IteX;
@@ -690,7 +434,11 @@ void CPlayers::RenderPlayer(
 				}
 				if(g_pData->m_Weapons.m_aId[iw].m_aSpriteMuzzles[IteX])
 				{
-					vec2 Dir = vec2(pPlayerChar->m_X,pPlayerChar->m_Y) - vec2(pPrevChar->m_X, pPrevChar->m_Y);
+					vec2 Dir;
+					if(PredictLocalWeapons)
+						Dir = vec2(pPlayerChar->m_X,pPlayerChar->m_Y) - vec2(pPrevChar->m_X, pPrevChar->m_Y);
+					else
+						Dir = vec2(m_pClient->m_Snap.m_aCharacters[ClientID].m_Cur.m_X, m_pClient->m_Snap.m_aCharacters[ClientID].m_Cur.m_Y) - vec2(m_pClient->m_Snap.m_aCharacters[ClientID].m_Prev.m_X, m_pClient->m_Snap.m_aCharacters[ClientID].m_Prev.m_Y);
 					Dir = normalize(Dir);
 					float HadOkenAngle = GetAngle(Dir);
 					Graphics()->QuadsSetRotation(HadOkenAngle);
@@ -707,11 +455,7 @@ void CPlayers::RenderPlayer(
 		{
 			// TODO: should be an animation
 			Recoil = 0;
-			static float s_LastIntraTick = IntraTick;
-			if(m_pClient->m_Snap.m_pGameInfoObj && !(m_pClient->m_Snap.m_pGameInfoObj->m_GameStateFlags&GAMESTATEFLAG_PAUSED))
-				s_LastIntraTick = IntraTick;
-
-			float a = (Client()->GameTick()-Player.m_AttackTick+s_LastIntraTick)/5.0f;
+			float a = AttackTicksPassed/5.0f;
 			if(a < 1)
 				Recoil = sinf(a*pi);
 			p = Position + Dir * g_pData->m_Weapons.m_aId[iw].m_Offsetx - Dir*Recoil*10.0f;
@@ -727,11 +471,10 @@ void CPlayers::RenderPlayer(
 			if(g_pData->m_Weapons.m_aId[iw].m_NumSpriteMuzzles)//prev.attackticks)
 			{
 				float Alpha = 0.0f;
-				int Phase1Tick = (Client()->GameTick() - Player.m_AttackTick);
-				if(Phase1Tick < (g_pData->m_Weapons.m_aId[iw].m_Muzzleduration + 3))
+				if(AttackTicksPassed < g_pData->m_Weapons.m_aId[iw].m_Muzzleduration + 3)
 				{
-					float t = ((((float)Phase1Tick) + IntraTick)/(float)g_pData->m_Weapons.m_aId[iw].m_Muzzleduration);
-					Alpha = mix(2.0f, 0.0f, min(1.0f,max(0.0f,t)));
+					float t = AttackTicksPassed/g_pData->m_Weapons.m_aId[iw].m_Muzzleduration;
+					Alpha = mix(2.0f, 0.0f, minimum(1.0f,maximum(0.0f,t)));
 				}
 
 				int IteX = rand() % g_pData->m_Weapons.m_aId[iw].m_NumSpriteMuzzles;
@@ -767,17 +510,11 @@ void CPlayers::RenderPlayer(
 		Graphics()->SetColor(1.0f, 1.0f, 1.0f, 1.0f);
 		Graphics()->QuadsSetRotation(0);
 
-		if(OtherTeam)
-		{
-			RenderInfo.m_ColorBody.a = g_Config.m_ClShowOthersAlpha / 100.0f;
-			RenderInfo.m_ColorFeet.a = g_Config.m_ClShowOthersAlpha / 100.0f;
-		}
-
 		switch (Player.m_Weapon)
 		{
-			case WEAPON_GUN: RenderHand(&RenderInfo, p, Direction, -3*pi/4, vec2(-15, 4)); break;
-			case WEAPON_SHOTGUN: RenderHand(&RenderInfo, p, Direction, -pi/2, vec2(-5, 4)); break;
-			case WEAPON_GRENADE: RenderHand(&RenderInfo, p, Direction, -pi/2, vec2(-4, 7)); break;
+			case WEAPON_GUN: RenderHand(&RenderInfo, p, Direction, -3*pi/4, vec2(-15, 4), Alpha); break;
+			case WEAPON_SHOTGUN: RenderHand(&RenderInfo, p, Direction, -pi/2, vec2(-5, 4), Alpha); break;
+			case WEAPON_GRENADE: RenderHand(&RenderInfo, p, Direction, -pi/2, vec2(-4, 7), Alpha); break;
 		}
 
 	}
@@ -785,45 +522,37 @@ void CPlayers::RenderPlayer(
 	// render the "shadow" tee
 	if(Local && (g_Config.m_Debug || g_Config.m_ClUnpredictedShadow))
 	{
-		vec2 GhostPosition = mix(vec2(pPrevChar->m_X, pPrevChar->m_Y), vec2(pPlayerChar->m_X, pPlayerChar->m_Y), Client()->IntraGameTick());
+		vec2 GhostPosition = Position;
+		if(ClientID >= 0)
+			GhostPosition = mix(
+					vec2(m_pClient->m_Snap.m_aCharacters[ClientID].m_Prev.m_X, m_pClient->m_Snap.m_aCharacters[ClientID].m_Prev.m_Y),
+					vec2(m_pClient->m_Snap.m_aCharacters[ClientID].m_Cur.m_X, m_pClient->m_Snap.m_aCharacters[ClientID].m_Cur.m_Y),
+					Client()->IntraGameTick());
+
 		CTeeRenderInfo Ghost = RenderInfo;
-		Ghost.m_ColorBody.a = 0.5f;
-		Ghost.m_ColorFeet.a = 0.5f;
-		RenderTools()->RenderTee(&State, &Ghost, Player.m_Emote, Direction, GhostPosition, true); // render ghost
+		RenderTools()->RenderTee(&State, &Ghost, Player.m_Emote, Direction, GhostPosition, 0.5f); // render ghost
 	}
 
 	RenderInfo.m_Size = 64.0f; // force some settings
 
-	if(OtherTeam)
-	{
-		RenderInfo.m_ColorBody.a = g_Config.m_ClShowOthersAlpha / 100.0f;
-		RenderInfo.m_ColorFeet.a = g_Config.m_ClShowOthersAlpha / 100.0f;
-	}
-
-	Graphics()->SetColor(1.0f, 1.0f, 1.0f, 1.0f);
+	Graphics()->SetColor(1.0f, 1.0f, 1.0f, Alpha);
 	Graphics()->QuadsSetRotation(0);
 	if(g_Config.m_ClShowDirection && ClientID >= 0 && (!Local || DemoPlayer()->IsPlaying()))
 	{
 		if(Player.m_Direction == -1)
 		{
 			Graphics()->TextureSet(g_pData->m_aImages[IMAGE_ARROW].m_Id);
-			if(OtherTeam)
-				Graphics()->SetColor(1.0f, 1.0f, 1.0f, g_Config.m_ClShowOthersAlpha / 100.0f);
 			Graphics()->QuadsSetRotation(pi);
 			Graphics()->RenderQuadContainerAsSprite(m_DirectionQuadContainerIndex, 0, Position.x - 30.f, Position.y - 70.f);
 		}
 		else if(Player.m_Direction == 1)
 		{
 			Graphics()->TextureSet(g_pData->m_aImages[IMAGE_ARROW].m_Id);
-			if(OtherTeam)
-				Graphics()->SetColor(1.0f, 1.0f, 1.0f, g_Config.m_ClShowOthersAlpha / 100.0f);
 			Graphics()->RenderQuadContainerAsSprite(m_DirectionQuadContainerIndex, 0, Position.x + 30.f, Position.y - 70.f);
 		}
 		if(Player.m_Jumped&1)
 		{
 			Graphics()->TextureSet(g_pData->m_aImages[IMAGE_ARROW].m_Id);
-			if(OtherTeam)
-				Graphics()->SetColor(1.0f, 1.0f, 1.0f, g_Config.m_ClShowOthersAlpha / 100.0f);
 			Graphics()->QuadsSetRotation(pi * 3 / 2);
 			Graphics()->RenderQuadContainerAsSprite(m_DirectionQuadContainerIndex, 0, Position.x, Position.y - 70.f);
 		}
@@ -831,15 +560,17 @@ void CPlayers::RenderPlayer(
 		Graphics()->QuadsSetRotation(0);
 	}
 
-	RenderTools()->RenderTee(&State, &RenderInfo, Player.m_Emote, Direction, Position, OtherTeam || ClientID < 0);
-	
+	if(OtherTeam || ClientID < 0)
+		RenderTools()->RenderTee(&State, &RenderInfo, Player.m_Emote, Direction, Position, g_Config.m_ClShowOthersAlpha / 100.0f);
+	else
+		RenderTools()->RenderTee(&State, &RenderInfo, Player.m_Emote, Direction, Position);
+
 	int QuadOffsetToEmoticon = NUM_WEAPONS * 2 + 2 + 2;
 	if(Player.m_PlayerFlags&PLAYERFLAG_CHATTING)
 	{
 		Graphics()->TextureSet(g_pData->m_aImages[IMAGE_EMOTICONS].m_Id);
 		int QuadOffset = QuadOffsetToEmoticon + (SPRITE_DOTDOT - SPRITE_OOP);
-		if(OtherTeam)
-			Graphics()->SetColor(1.0f, 1.0f, 1.0f, g_Config.m_ClShowOthersAlpha / 100.0f);
+		Graphics()->SetColor(1.0f, 1.0f, 1.0f, Alpha);
 		Graphics()->RenderQuadContainerAsSprite(m_WeaponEmoteQuadContainerIndex, QuadOffset, Position.x + 24.f, Position.y - 40.f);
 
 		Graphics()->SetColor(1.0f, 1.0f, 1.0f, 1.0f);
@@ -873,9 +604,7 @@ void CPlayers::RenderPlayer(
 
 		Graphics()->QuadsSetRotation(pi/6*WiggleAngle);
 
-		Graphics()->SetColor(1.0f,1.0f,1.0f,a);
-		if(OtherTeam)
-			Graphics()->SetColor(1.0f, 1.0f, 1.0f, a * (float)g_Config.m_ClShowOthersAlpha / 100.0f);
+		Graphics()->SetColor(1.0f,1.0f,1.0f,a * Alpha);
 		// client_datas::emoticon is an offset from the first emoticon
 		int QuadOffset = QuadOffsetToEmoticon + m_pClient->m_aClients[ClientID].m_Emoticon;
 		Graphics()->RenderQuadContainerAsSprite(m_WeaponEmoteQuadContainerIndex, QuadOffset, Position.x, Position.y - 23.f - 32.f * h, 1.f, (64.f*h) / 64.f);
@@ -905,63 +634,13 @@ void CPlayers::OnRender()
 				else
 				{
 					m_aRenderInfo[i].m_Texture = m_pClient->m_pSkins->Get(Skin)->m_OrgTexture;
-					m_aRenderInfo[i].m_ColorBody = vec4(1,1,1,1);
-					m_aRenderInfo[i].m_ColorFeet = vec4(1,1,1,1);
+					m_aRenderInfo[i].m_ColorBody = ColorRGBA(1,1,1);
+					m_aRenderInfo[i].m_ColorFeet = ColorRGBA(1,1,1);
 				}
 			}
 		}
 	}
 
-	static vec2 PrevPos[MAX_CLIENTS];
-	static vec2 SmoothPos[MAX_CLIENTS];
-	static int MoveCnt[MAX_CLIENTS] = {0};
-
-	static int predcnt = 0;
-
-	if(m_pClient->AntiPingPlayers())
-	{
-		for(int i = 0; i < MAX_CLIENTS; i++)
-		{
-			if(!m_pClient->m_Snap.m_aCharacters[i].m_Active)
-				continue;
-			const void *pPrevInfo = Client()->SnapFindItem(IClient::SNAP_PREV, NETOBJTYPE_PLAYERINFO, i);
-			const void *pInfo = Client()->SnapFindItem(IClient::SNAP_CURRENT, NETOBJTYPE_PLAYERINFO, i);
-
-			if(pPrevInfo && pInfo)
-			{
-				CNetObj_Character PrevChar = m_pClient->m_Snap.m_aCharacters[i].m_Prev;
-				CNetObj_Character CurChar = m_pClient->m_Snap.m_aCharacters[i].m_Cur;
-
-				Predict(
-						&PrevChar,
-						&CurChar,
-						(const CNetObj_PlayerInfo *)pPrevInfo,
-						(const CNetObj_PlayerInfo *)pInfo,
-						PrevPos[i],
-						SmoothPos[i],
-						MoveCnt[i],
-						m_CurPredictedPos[i]
-					);
-			}
-		}
-
-		if(m_pClient->AntiPingPlayers() && g_Config.m_ClPredict && Client()->State() != IClient::STATE_DEMOPLAYBACK)
-			if(m_pClient->m_Snap.m_pLocalCharacter && !(m_pClient->m_Snap.m_pGameInfoObj && m_pClient->m_Snap.m_pGameInfoObj->m_GameStateFlags&GAMESTATEFLAG_GAMEOVER))
-			{
-	//			double ping = m_pClient->m_Snap.m_pLocalInfo->m_Latency;
-	//			static double fps;
-	//			fps = mix(fps, (1. / Client()->RenderFrameTime()), 0.1);
-
-	//			int predmax = (fps * ping / 1000.);
-
-				int predmax = 19;
-	//			if( 0 <= predmax && predmax <= 100)
-						predcnt = (predcnt + 1) % predmax;
-	//			else
-	//			    predcnt = (predcnt + 1) % 2;
-			}
-	}
-
 	// render other players in two passes, first pass we render the other, second pass we render our self
 	for(int p = 0; p < 4; p++)
 	{
@@ -981,29 +660,17 @@ void CPlayers::OnRender()
 				if((p % 2) == 0 && Local) continue;
 				if((p % 2) == 1 && !Local) continue;
 
-				CNetObj_Character PrevChar = m_pClient->m_Snap.m_aCharacters[i].m_Prev;
-				CNetObj_Character CurChar = m_pClient->m_Snap.m_aCharacters[i].m_Cur;
+				CNetObj_Character PrevChar = m_pClient->m_aClients[i].m_RenderPrev;
+				CNetObj_Character CurChar = m_pClient->m_aClients[i].m_RenderCur;
 
 				if(p<2)
 				{
-					if(PrevChar.m_HookedPlayer != -1)
-						RenderHook(
-								&PrevChar,
-								&CurChar,
-								&m_aRenderInfo[i],
-								i,
-								m_CurPredictedPos[i],
-								m_CurPredictedPos[PrevChar.m_HookedPlayer]
-							);
-					else
-						RenderHook(
-								&PrevChar,
-								&CurChar,
-								&m_aRenderInfo[i],
-								i,
-								m_CurPredictedPos[i],
-								m_CurPredictedPos[i]
-							);
+					RenderHook(
+							&PrevChar,
+							&CurChar,
+							&m_aRenderInfo[i],
+							i
+						);
 				}
 				else
 				{
@@ -1011,8 +678,7 @@ void CPlayers::OnRender()
 							&PrevChar,
 							&CurChar,
 							&m_aRenderInfo[i],
-							i,
-							m_CurPredictedPos[i]
+							i
 						);
 				}
 			}
@@ -1050,7 +716,7 @@ void CPlayers::OnInit()
 		RenderTools()->SelectSprite(SPRITE_OOP + i);
 		RenderTools()->QuadContainerAddSprite(m_WeaponEmoteQuadContainerIndex, 64.f, false);
 	}
-	
+
 	for(int i = 0; i < NUM_WEAPONS; ++i)
 	{
 		m_WeaponSpriteMuzzleQuadContainerIndex[i] = Graphics()->CreateQuadContainer();
