@@ -9,40 +9,23 @@ class CPlayers : public CComponent
 	friend class CGhost;
 
 	CTeeRenderInfo m_aRenderInfo[MAX_CLIENTS];
-	void RenderHand(class CTeeRenderInfo *pInfo, vec2 CenterPos, vec2 Dir, float AngleOffset, vec2 PostRotOffset);
+	bool IsOtherTeam(int ClientID);
+	void RenderHand(class CTeeRenderInfo *pInfo, vec2 CenterPos, vec2 Dir, float AngleOffset, vec2 PostRotOffset, float Alpha = 1.0f);
 	void RenderPlayer(
 		const CNetObj_Character *pPrevChar,
 		const CNetObj_Character *pPlayerChar,
 		const CTeeRenderInfo *pRenderInfo,
 		int ClientID,
-		const vec2 &Position,
 		float Intra = 0.f
-/*		vec2 &PrevPredPos,
-		vec2 &SmoothPos,
-		int &MoveCnt
-*/
 	);
 	void RenderHook(
 		const CNetObj_Character *pPrevChar,
 		const CNetObj_Character *pPlayerChar,
 		const CTeeRenderInfo *pRenderInfo,
 		int ClientID,
-		const vec2 &Position,
-		const vec2 &PositionTo,
 		float Intra = 0.f
 	);
 
-	void Predict(
-		const CNetObj_Character *pPrevChar,
-		const CNetObj_Character *pPlayerChar,
-		const CNetObj_PlayerInfo *pPrevInfo,
-		const CNetObj_PlayerInfo *pPlayerInfo,
-		vec2 &PrevPredPos,
-		vec2 &SmoothPos,
-		int &MoveCnt,
-		vec2 &Position
-	);
-
 	int m_WeaponEmoteQuadContainerIndex;
 	int m_DirectionQuadContainerIndex;
 	int m_WeaponSpriteMuzzleQuadContainerIndex[NUM_WEAPONS];
