@@ -15,6 +15,7 @@
 #include <game/client/components/countryflags.h>
 #include <game/client/components/motd.h>
 #include <game/client/components/statboard.h>
+#include <game/client/components/skins.h>
 
 #include "scoreboard.h"
 
@@ -116,9 +117,13 @@ void CScoreboard::RenderSpectators(float x, float y, float w)
 
 	// spectator names
 	y += 30.0f;
-	char aBuffer[1024*4];
-	aBuffer[0] = 0;
 	bool Multiple = false;
+
+	CTextCursor Cursor;
+	TextRender()->SetCursor(&Cursor, x+10.0f, y, 22.0f, TEXTFLAG_RENDER);
+	Cursor.m_LineWidth = w-20.0f;
+	Cursor.m_MaxLines = 4;
+
 	for(int i = 0; i < MAX_CLIENTS; ++i)
 	{
 		const CNetObj_PlayerInfo *pInfo = m_pClient->m_Snap.m_paInfoByName[i];
@@ -126,21 +131,25 @@ void CScoreboard::RenderSpectators(float x, float y, float w)
 			continue;
 
 		if(Multiple)
-			str_append(aBuffer, ", ", sizeof(aBuffer));
+			TextRender()->TextEx(&Cursor, ", ", 2);
+
+		if(m_pClient->m_aClients[pInfo->m_ClientID].m_AuthLevel)
+		{
+			ColorRGBA Color = color_cast<ColorRGBA>(ColorHSLA(g_Config.m_ClAuthedPlayerColor));
+			TextRender()->TextColor(Color);
+		}
+
 		if(g_Config.m_ClShowIDs)
 		{
-			char aId[5];
-			str_format(aId,sizeof(aId),"%d: ",pInfo->m_ClientID);
-			str_append(aBuffer, aId, sizeof(aBuffer));
+			char aBuffer[5];
+			int size = str_format(aBuffer, sizeof(aBuffer), "%d: ", pInfo->m_ClientID);
+			TextRender()->TextEx(&Cursor, aBuffer, size);
 		}
-		str_append(aBuffer, m_pClient->m_aClients[pInfo->m_ClientID].m_aName, sizeof(aBuffer));
+		TextRender()->TextEx(&Cursor, m_pClient->m_aClients[pInfo->m_ClientID].m_aName, -1);
+		TextRender()->TextColor(1.0f, 1.0f, 1.0f, 1.0f);
+
 		Multiple = true;
 	}
-	CTextCursor Cursor;
-	TextRender()->SetCursor(&Cursor, x+10.0f, y, 22.0f, TEXTFLAG_RENDER);
-	Cursor.m_LineWidth = w-20.0f;
-	Cursor.m_MaxLines = 4;
-	TextRender()->TextEx(&Cursor, aBuffer, -1);
 }
 
 void CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const char *pTitle)
@@ -193,7 +202,7 @@ void CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const ch
 		if(m_pClient->m_Snap.m_pGameInfoObj->m_GameStateFlags&GAMESTATEFLAG_GAMEOVER)
 			pTitle = Localize("Game over");
 		else
-			pTitle = Localize("Score board");
+			pTitle = Localize("Scoreboard");
 	}
 	TextRender()->Text(0, x+20.0f, y + (50.f - TitleFontsize) / 2.f, TitleFontsize, pTitle, -1);
 
@@ -222,7 +231,7 @@ void CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const ch
 		}
 	}
 
-	if(m_IsGameTypeRace && g_Config.m_ClDDRaceScoreBoard && m_pClient->m_AllowTimeScore[g_Config.m_ClDummy])
+	if(m_pClient->TimeScore() && g_Config.m_ClDDRaceScoreBoard)
 	{
 		if (m_ServerRecord > 0)
 		{
@@ -285,9 +294,10 @@ void CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const ch
 	// render headlines
 	y += 50.0f;
 	float HeadlineFontsize = 22.0f;
-	float ScoreWidth = TextRender()->TextWidth(0, HeadlineFontsize, Localize("Score"), -1);
+	const char *pScore = (m_pClient->TimeScore() && g_Config.m_ClDDRaceScoreBoard) ? Localize("Time") : Localize("Score");
+	float ScoreWidth = TextRender()->TextWidth(0, HeadlineFontsize, pScore, -1);
 	tw = ScoreLength > ScoreWidth ? ScoreLength : ScoreWidth;
-	TextRender()->Text(0, ScoreOffset+ScoreLength-tw, y + (HeadlineFontsize * 2.f - HeadlineFontsize) / 2.f, HeadlineFontsize, Localize("Score"), -1);
+	TextRender()->Text(0, ScoreOffset+ScoreLength-tw, y + (HeadlineFontsize * 2.f - HeadlineFontsize) / 2.f, HeadlineFontsize, pScore, -1);
 
 	TextRender()->Text(0, NameOffset, y + (HeadlineFontsize * 2.f - HeadlineFontsize) / 2.f, HeadlineFontsize, Localize("Name"), -1);
 
@@ -325,7 +335,7 @@ void CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const ch
 
 		if (rendered++ < 0) continue;
 
-		int DDTeam = ((CGameClient *) m_pClient)->m_Teams.Team(pInfo->m_ClientID);
+		int DDTeam = m_pClient->m_Teams.Team(pInfo->m_ClientID);
 		int NextDDTeam = 0;
 
 		for(int j = i + 1; j < MAX_CLIENTS; j++)
@@ -335,7 +345,7 @@ void CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const ch
 			if(!pInfo2 || pInfo2->m_Team != Team)
 				continue;
 
-			NextDDTeam = ((CGameClient *) m_pClient)->m_Teams.Team(pInfo2->m_ClientID);
+			NextDDTeam = m_pClient->m_Teams.Team(pInfo2->m_ClientID);
 			break;
 		}
 
@@ -348,7 +358,7 @@ void CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const ch
 				if(!pInfo2 || pInfo2->m_Team != Team)
 					continue;
 
-				OldDDTeam = ((CGameClient *) m_pClient)->m_Teams.Team(pInfo2->m_ClientID);
+				OldDDTeam = m_pClient->m_Teams.Team(pInfo2->m_ClientID);
 				break;
 			}
 		}
@@ -357,8 +367,8 @@ void CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const ch
 		{
 			Graphics()->TextureSet(-1);
 			Graphics()->QuadsBegin();
-			vec3 rgb = HslToRgb(vec3(DDTeam / 64.0f, 1.0f, 0.5f));
-			Graphics()->SetColor(rgb.r, rgb.g, rgb.b, 0.5f);
+			ColorRGBA rgb = color_cast<ColorRGBA>(ColorHSLA(DDTeam / 64.0f, 1.0f, 0.5f, 0.5f));
+			Graphics()->SetColor(rgb);
 
 			int Corners = 0;
 
@@ -404,7 +414,7 @@ void CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const ch
 		}
 
 		// score
-		if(m_IsGameTypeRace && g_Config.m_ClDDRaceScoreBoard && m_pClient->m_AllowTimeScore[g_Config.m_ClDummy])
+		if(m_pClient->TimeScore() && g_Config.m_ClDDRaceScoreBoard)
 		{
 			if (pInfo->m_Score == -9999)
 				aBuf[0] = 0;
@@ -444,13 +454,16 @@ void CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const ch
 
 		// name
 		TextRender()->SetCursor(&Cursor, NameOffset, y + (LineHeight - FontSize) / 2.f, FontSize, TEXTFLAG_RENDER|TEXTFLAG_STOP_AT_END);
+		if(m_pClient->m_aClients[pInfo->m_ClientID].m_AuthLevel)
+		{
+			ColorRGBA Color = color_cast<ColorRGBA>(ColorHSLA(g_Config.m_ClAuthedPlayerColor));
+			TextRender()->TextColor(Color);
+		}
+
 		if(g_Config.m_ClShowIDs)
 		{
 			char aId[64] = "";
-			if (pInfo->m_ClientID >= 10)
-				str_format(aId, sizeof(aId),"%d: ", pInfo->m_ClientID);
-			else
-				str_format(aId, sizeof(aId)," %d: ", pInfo->m_ClientID);
+			str_format(aId, sizeof(aId)," %2d: ", pInfo->m_ClientID);
 			str_append(aId, m_pClient->m_aClients[pInfo->m_ClientID].m_aName,sizeof(aId));
 			Cursor.m_LineWidth = NameLength+8;
 			TextRender()->TextEx(&Cursor, aId, -1);
@@ -462,23 +475,41 @@ void CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const ch
 		}
 
 		// clan
-		tw = TextRender()->TextWidth(0, FontSize, m_pClient->m_aClients[pInfo->m_ClientID].m_aClan, -1);
-		TextRender()->SetCursor(&Cursor, ClanOffset+ClanLength/2-tw/2, y + (LineHeight - FontSize) / 2.f, FontSize, TEXTFLAG_RENDER|TEXTFLAG_STOP_AT_END);
+		if(str_comp(m_pClient->m_aClients[pInfo->m_ClientID].m_aClan,
+				m_pClient->m_aClients[GameClient()->m_LocalIDs[g_Config.m_ClDummy]].m_aClan) == 0)
+		{
+			ColorRGBA Color = color_cast<ColorRGBA>(ColorHSLA(g_Config.m_ClSameClanColor));
+			TextRender()->TextColor(Color);
+		}
+		else
+			TextRender()->TextColor(1.0f, 1.0f, 1.0f, 1.0f);
+
+		tw = TextRender()->TextWidth(nullptr, FontSize, m_pClient->m_aClients[pInfo->m_ClientID].m_aClan, -1);
+		TextRender()->SetCursor(&Cursor, ClanOffset + ClanLength / 2 - tw / 2, y + (LineHeight - FontSize) / 2.f, FontSize, TEXTFLAG_RENDER|TEXTFLAG_STOP_AT_END);
 		Cursor.m_LineWidth = ClanLength;
 		TextRender()->TextEx(&Cursor, m_pClient->m_aClients[pInfo->m_ClientID].m_aClan, -1);
 
+		TextRender()->TextColor(1.0f, 1.0f, 1.0f, 1.0f);
+
 		// country flag
-		vec4 Color(1.0f, 1.0f, 1.0f, 0.5f);
+		ColorRGBA Color(1.0f, 1.0f, 1.0f, 0.5f);
 		m_pClient->m_pCountryFlags->Render(m_pClient->m_aClients[pInfo->m_ClientID].m_Country, &Color,
 											CountryOffset, y+(Spacing+TeeSizeMod*5.0f)/2.0f, CountryLength, LineHeight-Spacing-TeeSizeMod*5.0f);
 
 		// ping
+		if(g_Config.m_ClEnablePingColor)
+		{
+			ColorRGBA rgb = color_cast<ColorRGBA>(ColorHSLA((300.0f - clamp(pInfo->m_Latency, 0, 300)) / 1000.0f, 1.0f, 0.5f));
+			TextRender()->TextColor(rgb);
+		}
 		str_format(aBuf, sizeof(aBuf), "%d", clamp(pInfo->m_Latency, 0, 1000));
-		tw = TextRender()->TextWidth(0, FontSize, aBuf, -1);
+		tw = TextRender()->TextWidth(nullptr, FontSize, aBuf, -1);
 		TextRender()->SetCursor(&Cursor, PingOffset+PingLength-tw, y + (LineHeight - FontSize) / 2.f, FontSize, TEXTFLAG_RENDER|TEXTFLAG_STOP_AT_END);
 		Cursor.m_LineWidth = PingLength;
 		TextRender()->TextEx(&Cursor, aBuf, -1);
 
+		TextRender()->TextColor(1.0f, 1.0f, 1.0f, 1.0f);
+
 		y += LineHeight+Spacing;
 		if (lower32 || upper32) {
 			if (rendered == 32) break;
@@ -494,7 +525,8 @@ void CScoreboard::RenderRecordingNotification(float x)
 {
 	if(!m_pClient->DemoRecorder(RECORDER_MANUAL)->IsRecording() &&
 	   !m_pClient->DemoRecorder(RECORDER_AUTO)->IsRecording() &&
-	   !m_pClient->DemoRecorder(RECORDER_RACE)->IsRecording())
+	   !m_pClient->DemoRecorder(RECORDER_RACE)->IsRecording() &&
+	   !m_pClient->DemoRecorder(RECORDER_REPLAYS)->IsRecording())
 	{
 		return;
 	}
@@ -522,6 +554,12 @@ void CScoreboard::RenderRecordingNotification(float x)
 		str_format(aBuf2, sizeof(aBuf2), Localize("Auto %3d:%02d  "), Seconds/60, Seconds%60);
 		str_append(aBuf, aBuf2, sizeof(aBuf));
 	}
+	if(m_pClient->DemoRecorder(RECORDER_REPLAYS)->IsRecording())
+	{
+		Seconds = m_pClient->DemoRecorder(RECORDER_REPLAYS)->Length();
+		str_format(aBuf2, sizeof(aBuf2), Localize("Replay %3d:%02d  "), Seconds / 60, Seconds % 60);
+		str_append(aBuf, aBuf2, sizeof(aBuf));
+	}
 
 	float w = TextRender()->TextWidth(0, 20.0f, aBuf, -1);
 
