@@ -3,6 +3,7 @@
 #ifndef GAME_CLIENT_COMPONENTS_SKINS_H
 #define GAME_CLIENT_COMPONENTS_SKINS_H
 #include <base/vmath.h>
+#include <base/color.h>
 #include <base/tl/sorted_array.h>
 #include <game/client/component.h>
 
@@ -16,15 +17,13 @@ public:
 		int m_OrgTexture;
 		int m_ColorTexture;
 		char m_aName[24];
-		vec3 m_BloodColor;
+		ColorRGBA m_BloodColor;
 
 		bool operator<(const CSkin &Other) { return str_comp(m_aName, Other.m_aName) < 0; }
 	};
 
 	void OnInit();
 
-	vec3 GetColorV3(int v);
-	vec4 GetColorV4(int v);
 	int Num();
 	const CSkin *Get(int Index);
 	int Find(const char *pName) const;
