@@ -24,6 +24,7 @@
 #include <engine/editor.h>
 #include <engine/graphics.h>
 #include <engine/sound.h>
+#include <engine/storage.h>
 
 #include "auto_map.h"
 
@@ -42,12 +43,6 @@ enum
 	DIALOG_FILE,
 };
 
-struct CEntity
-{
-	CPoint m_Position;
-	int m_Type;
-};
-
 class CEnvelope
 {
 public:
@@ -135,7 +130,6 @@ public:
 		str_copy(m_aName, "(invalid)", sizeof(m_aName));
 		m_Visible = true;
 		m_Readonly = false;
-		m_SaveToMap = true;
 		m_Flags = 0;
 		m_pEditor = 0;
 		m_BrushRefCount = 0;
@@ -170,7 +164,6 @@ public:
 
 	bool m_Readonly;
 	bool m_Visible;
-	bool m_SaveToMap;
 
 	int m_BrushRefCount;
 };
@@ -197,7 +190,6 @@ public:
 	char m_aName[12];
 	bool m_GameGroup;
 	bool m_Visible;
-	bool m_SaveToMap;
 	bool m_Collapse;
 
 	CLayerGroup();
@@ -535,6 +527,7 @@ public:
 	void Snap(CUIRect *pRect);
 	void Clamp(RECTi *pRect);
 
+	virtual bool IsEmpty(CLayerTiles *pLayer);
 	virtual void BrushSelecting(CUIRect Rect);
 	virtual int BrushGrab(CLayerGroup *pBrush, CUIRect Rect);
 	virtual void FillSelection(bool Empty, CLayer *pBrush, CUIRect Rect);
@@ -585,7 +578,7 @@ public:
 	~CLayerQuads();
 
 	virtual void Render(bool QuadPicker = false);
-	CQuad *NewQuad();
+	CQuad *NewQuad(int x, int y, int Width, int Height);
 
 	virtual void BrushSelecting(CUIRect Rect);
 	virtual int BrushGrab(CLayerGroup *pBrush, CUIRect Rect);
@@ -694,10 +687,6 @@ public:
 		m_MouseDeltaY = 0;
 		m_MouseDeltaWx = 0;
 		m_MouseDeltaWy = 0;
-#if defined(__ANDROID__)
-		m_OldMouseX = 0;
-		m_OldMouseY = 0;
-#endif
 
 		m_GuiActive = true;
 		m_ProofBorders = false;
@@ -745,11 +734,16 @@ public:
 		m_LargeLayerWasWarned = false;
 		m_PreventUnusedTilesWasWarned = false;
 		m_AllowPlaceUnusedTiles = 0;
+		m_BrushDrawDestructive = true;
+
+		m_Mentions = 0;
 	}
 
 	virtual void Init();
 	virtual void UpdateAndRender();
 	virtual bool HasUnsavedData() { return m_Map.m_Modified; }
+	virtual void UpdateMentions() { m_Mentions++; }
+	virtual void ResetMentions() { m_Mentions = 0; }
 
 	int64 m_LastUndoUpdateTime;
 	bool m_UndoRunning;
@@ -794,7 +788,7 @@ public:
 	bool IsQuadSelected(int Index);
 	int FindSelectedQuadIndex(int Index);
 
-	int DoProperties(CUIRect *pToolbox, CProperty *pProps, int *pIDs, int *pNewVal, vec4 Color = vec4(1,1,1,0.5f));
+	int DoProperties(CUIRect *pToolbox, CProperty *pProps, int *pIDs, int *pNewVal, ColorRGBA Color = ColorRGBA(1,1,1,0.5f));
 
 	int m_Mode;
 	int m_Dialog;
@@ -827,14 +821,15 @@ public:
 	bool m_LargeLayerWasWarned;
 	bool m_PreventUnusedTilesWasWarned;
 	int m_AllowPlaceUnusedTiles;
+	bool m_BrushDrawDestructive;
+
+	int m_Mentions;
 
 	enum
 	{
 		FILETYPE_MAP,
 		FILETYPE_IMG,
 		FILETYPE_SOUND,
-
-		MAX_PATH_LENGTH = 512
 	};
 
 	int m_FileDialogStorageType;
@@ -893,10 +888,6 @@ public:
 	float m_MouseDeltaY;
 	float m_MouseDeltaWx;
 	float m_MouseDeltaWy;
-#if defined(__ANDROID__)
-	float m_OldMouseX;
-	float m_OldMouseY;
-#endif
 
 	bool m_ShowTileInfo;
 	bool m_ShowDetail;
@@ -945,7 +936,7 @@ public:
 	void DoMapBorder();
 	int DoButton_Editor_Common(const void *pID, const char *pText, int Checked, const CUIRect *pRect, int Flags, const char *pToolTip);
 	int DoButton_Editor(const void *pID, const char *pText, int Checked, const CUIRect *pRect, int Flags, const char *pToolTip);
-	int DoButton_Env(const void *pID, const char *pText, int Checked, const CUIRect *pRect, const char *pToolTip, vec3 Color);
+	int DoButton_Env(const void *pID, const char *pText, int Checked, const CUIRect *pRect, const char *pToolTip, ColorRGBA Color);
 
 	int DoButton_Tab(const void *pID, const char *pText, int Checked, const CUIRect *pRect, int Flags, const char *pToolTip);
 	int DoButton_Ex(const void *pID, const char *pText, int Checked, const CUIRect *pRect, int Flags, const char *pToolTip, int Corners, float FontSize=10.0f);
@@ -957,7 +948,7 @@ public:
 	int DoButton_Menu(const void *pID, const char *pText, int Checked, const CUIRect *pRect, int Flags, const char *pToolTip);
 	int DoButton_MenuItem(const void *pID, const char *pText, int Checked, const CUIRect *pRect, int Flags=0, const char *pToolTip=0);
 
-	int DoButton_ColorPicker(const void *pID, const CUIRect *pRect, vec4 *pColor, const char *pToolTip=0);
+	int DoButton_ColorPicker(const void *pID, const CUIRect *pRect, ColorRGBA *pColor, const char *pToolTip=0);
 
 	int DoEditBox(void *pID, const CUIRect *pRect, char *pStr, unsigned StrSize, float FontSize, float *Offset, bool Hidden=false, int Corners=CUI::CORNER_ALL);
 
@@ -968,7 +959,7 @@ public:
 	void UiInvokePopupMenu(void *pID, int Flags, float X, float Y, float W, float H, int (*pfnFunc)(CEditor *pEditor, CUIRect Rect), void *pExtra=0);
 	void UiDoPopupMenu();
 
-	int UiDoValueSelector(void *pID, CUIRect *pRect, const char *pLabel, int Current, int Min, int Max, int Step, float Scale, const char *pToolTip, bool IsDegree=false, bool IsHex=false, int corners=CUI::CORNER_ALL, vec4* Color=0);
+	int UiDoValueSelector(void *pID, CUIRect *pRect, const char *pLabel, int Current, int Min, int Max, int Step, float Scale, const char *pToolTip, bool IsDegree=false, bool IsHex=false, int corners=CUI::CORNER_ALL, ColorRGBA* Color=0);
 
 	static int PopupGroup(CEditor *pEditor, CUIRect View);
 	static int PopupLayer(CEditor *pEditor, CUIRect View);
@@ -1005,7 +996,7 @@ public:
 	void PopupSelectSoundInvoke(int Current, float x, float y);
 	int PopupSelectSoundResult();
 
-	vec4 ButtonColorMul(const void *pID);
+	float ButtonColorMul(const void *pID);
 
 	void DoQuadEnvelopes(const array<CQuad> &m_lQuads, int TexID = -1);
 	void DoQuadEnvPoint(const CQuad *pQuad, int QIndex, int pIndex);
@@ -1017,7 +1008,7 @@ public:
 	void DoToolbar(CUIRect Toolbar);
 	void DoQuad(CQuad *pQuad, int Index);
 	float UiDoScrollbarV(const void *pID, const CUIRect *pRect, float Current);
-	vec4 GetButtonColor(const void *pID, int Checked);
+	ColorRGBA GetButtonColor(const void *pID, int Checked);
 
 	static void ReplaceImage(const char *pFilename, int StorageType, void *pUser);
 	static void ReplaceSound(const char *pFileName, int StorageType, void *pUser);
@@ -1040,11 +1031,12 @@ public:
 
 	void AddFileDialogEntry(int Index, CUIRect *pView);
 	void SortImages();
+	const char *Explain(int Tile, int Layer);
 
 	int GetLineDistance();
 	void ZoomMouseTarget(float ZoomFactor);
 
-	static vec3 ms_PickerColor;
+	static ColorHSVA ms_PickerColor;
 	static int ms_SVPicker;
 	static int ms_HuePicker;
 
@@ -1088,6 +1080,7 @@ public:
 
 	virtual void Resize(int NewW, int NewH);
 	virtual void Shift(int Direction);
+	virtual bool IsEmpty(CLayerTiles *pLayer);
 	virtual void BrushDraw(CLayer *pBrush, float wx, float wy);
 	virtual void BrushFlipX();
 	virtual void BrushFlipY();
@@ -1108,6 +1101,7 @@ public:
 
 	virtual void Resize(int NewW, int NewH);
 	virtual void Shift(int Direction);
+	virtual bool IsEmpty(CLayerTiles *pLayer);
 	virtual void BrushDraw(CLayer *pBrush, float wx, float wy);
 	virtual void BrushFlipX();
 	virtual void BrushFlipY();
@@ -1121,9 +1115,7 @@ public:
 	CLayerFront(int w, int h);
 
 	virtual void Resize(int NewW, int NewH);
-	virtual void Shift(int Direction);
 	virtual void SetTile(int x, int y, CTile tile);
-	virtual void BrushDraw(CLayer *pBrush, float wx, float wy);
 };
 
 class CLayerSwitch : public CLayerTiles
@@ -1138,7 +1130,11 @@ public:
 
 	virtual void Resize(int NewW, int NewH);
 	virtual void Shift(int Direction);
+	virtual bool IsEmpty(CLayerTiles *pLayer);
 	virtual void BrushDraw(CLayer *pBrush, float wx, float wy);
+	virtual void BrushFlipX();
+	virtual void BrushFlipY();
+	virtual void BrushRotate(float Amount);
 	virtual void FillSelection(bool Empty, CLayer *pBrush, CUIRect Rect);
 };
 
@@ -1153,6 +1149,7 @@ public:
 
 	virtual void Resize(int NewW, int NewH);
 	virtual void Shift(int Direction);
+	virtual bool IsEmpty(CLayerTiles *pLayer);
 	virtual void BrushDraw(CLayer *pBrush, float wx, float wy);
 	virtual void BrushFlipX();
 	virtual void BrushFlipY();
@@ -1167,7 +1164,7 @@ public:
 	~CLayerSounds();
 
 	virtual void Render(bool Tileset = false);
-	CSoundSource *NewSource();
+	CSoundSource *NewSource(int x, int y);
 
 	virtual void BrushSelecting(CUIRect Rect);
 	virtual int BrushGrab(CLayerGroup *pBrush, CUIRect Rect);
