@@ -357,9 +357,6 @@ int CEditorMap::Save(class IStorage *pStorage, const char *pFileName)
 	for(int g = 0; g < m_lGroups.size(); g++)
 	{
 		CLayerGroup *pGroup = m_lGroups[g];
-		if(!pGroup->m_SaveToMap)
-			continue;
-
 		CMapItemGroup GItem;
 		GItem.m_Version = CMapItemGroup::CURRENT_VERSION;
 
@@ -380,9 +377,6 @@ int CEditorMap::Save(class IStorage *pStorage, const char *pFileName)
 
 		for(int l = 0; l < pGroup->m_lLayers.size(); l++)
 		{
-			if(!pGroup->m_lLayers[l]->m_SaveToMap)
-				continue;
-
 			if(pGroup->m_lLayers[l]->m_Type == LAYERTYPE_TILES)
 			{
 				m_pEditor->Console()->Print(IConsole::OUTPUT_LEVEL_ADDINFO, "editor", "saving tiles layer");
