@@ -18,7 +18,7 @@
 CLayerTiles::CLayerTiles(int w, int h)
 {
 	m_Type = LAYERTYPE_TILES;
-	str_copy(m_aName, "Tiles", sizeof(m_aName));
+	m_aName[0] = '\0';
 	m_Width = w;
 	m_Height = h;
 	m_Image = -1;
@@ -85,7 +85,7 @@ void CLayerTiles::Render(bool Tileset)
 	if(m_Image >= 0 && m_Image < m_pEditor->m_Map.m_lImages.size())
 		m_TexID = m_pEditor->m_Map.m_lImages[m_Image]->m_TexID;
 	Graphics()->TextureSet(m_TexID);
-	vec4 Color = vec4(m_Color.r/255.0f, m_Color.g/255.0f, m_Color.b/255.0f, m_Color.a/255.0f);
+	ColorRGBA Color = ColorRGBA(m_Color.r/255.0f, m_Color.g/255.0f, m_Color.b/255.0f, m_Color.a/255.0f);
 	Graphics()->BlendNone();
 	m_pEditor->RenderTools()->RenderTilemap(m_pTiles, m_Width, m_Height, 32.0f, Color, LAYERRENDERFLAG_OPAQUE,
 												m_pEditor->EnvelopeEval, m_pEditor, m_ColorEnv, m_ColorEnvOffset);
@@ -154,6 +154,32 @@ void CLayerTiles::Clamp(RECTi *pRect)
 		pRect->w = 0;
 }
 
+bool CLayerTiles::IsEmpty(CLayerTiles *pLayer)
+{
+	for(int y = 0; y < pLayer->m_Height; y++)
+		for(int x = 0; x < pLayer->m_Width; x++)
+		{
+			int Index = pLayer->GetTile(x, y).m_Index;
+			if(Index)
+			{
+				if(pLayer->m_Game)
+				{
+					if(m_pEditor->m_AllowPlaceUnusedTiles || IsValidGameTile(Index))
+						return false;
+				}
+				else if(pLayer->m_Front)
+				{
+					if(m_pEditor->m_AllowPlaceUnusedTiles || IsValidFrontTile(Index))
+						return false;
+				}
+				else
+					return false;
+			}
+		}
+
+	return true;
+}
+
 void CLayerTiles::BrushSelecting(CUIRect Rect)
 {
 	Graphics()->TextureSet(-1);
@@ -393,6 +419,8 @@ void CLayerTiles::FillSelection(bool Empty, CLayer *pBrush, CUIRect Rect)
 
 	CLayerTiles *pLt = static_cast<CLayerTiles*>(pBrush);
 
+	bool Destructive = m_pEditor->m_BrushDrawDestructive || Empty || IsEmpty(pLt);
+
 	for(int y = 0; y < h; y++)
 	{
 		for(int x = 0; x < w; x++)
@@ -403,8 +431,24 @@ void CLayerTiles::FillSelection(bool Empty, CLayer *pBrush, CUIRect Rect)
 			if(fx < 0 || fx >= m_Width || fy < 0 || fy >= m_Height)
 				continue;
 
+			bool HasTile = GetTile(fx, fy).m_Index;
+			if(!Empty && pLt->GetTile(x, y).m_Index == TILE_THROUGH_CUT)
+			{
+				if(m_Game && m_pEditor->m_Map.m_pFrontLayer)
+				{
+					HasTile = HasTile || m_pEditor->m_Map.m_pFrontLayer->GetTile(fx, fy).m_Index;
+				}
+				else if(m_Front)
+				{
+					HasTile = HasTile || m_pEditor->m_Map.m_pGameLayer->GetTile(fx, fy).m_Index;
+				}
+			}
+
+			if(!Destructive && HasTile)
+				continue;
+
 			if(Empty)
-				m_pTiles[fy*m_Width+fx].m_Index = 1;
+				m_pTiles[fy*m_Width+fx].m_Index = 0;
 			else
 				SetTile(fx, fy, pLt->m_pTiles[(y*pLt->m_Width + x%pLt->m_Width) % (pLt->m_Width*pLt->m_Height)]);
 		}
@@ -422,16 +466,36 @@ void CLayerTiles::BrushDraw(CLayer *pBrush, float wx, float wy)
 	int sx = ConvertX(wx);
 	int sy = ConvertY(wy);
 
+	bool Destructive = m_pEditor->m_BrushDrawDestructive || IsEmpty(l);
+
 	for(int y = 0; y < l->m_Height; y++)
 		for(int x = 0; x < l->m_Width; x++)
 		{
 			int fx = x+sx;
 			int fy = y+sy;
+
 			if(fx<0 || fx >= m_Width || fy < 0 || fy >= m_Height)
 				continue;
 
-			SetTile(fx, fy, l->m_pTiles[y*l->m_Width+x]);
+			bool HasTile = GetTile(fx, fy).m_Index;
+			if(l->GetTile(x, y).m_Index == TILE_THROUGH_CUT)
+			{
+				if(m_Game && m_pEditor->m_Map.m_pFrontLayer)
+				{
+					HasTile = HasTile || m_pEditor->m_Map.m_pFrontLayer->GetTile(fx, fy).m_Index;
+				}
+				else if(m_Front)
+				{
+					HasTile = HasTile || m_pEditor->m_Map.m_pGameLayer->GetTile(fx, fy).m_Index;
+				}
+			}
+
+			if(!Destructive && HasTile)
+				continue;
+
+			SetTile(fx, fy, l->GetTile(x, y));
 		}
+
 	FlagModified(sx, sy, l->m_Width, l->m_Height);
 }
 
@@ -445,9 +509,16 @@ void CLayerTiles::BrushFlipX()
 			m_pTiles[y*m_Width+m_Width-1-x] = Tmp;
 		}
 
+	if(m_Tele || m_Switch || m_Speedup || m_Tune)
+		return;
+
+	bool Rotate = !(m_Game || m_Front) || m_pEditor->m_AllowPlaceUnusedTiles;
 	for(int y = 0; y < m_Height; y++)
 		for(int x = 0; x < m_Width; x++)
-			m_pTiles[y*m_Width+x].m_Flags ^= m_pTiles[y*m_Width+x].m_Flags&TILEFLAG_ROTATE ? TILEFLAG_HFLIP : TILEFLAG_VFLIP;
+			if(!Rotate && !IsRotatableTile(m_pTiles[y*m_Width+x].m_Index))
+				m_pTiles[y*m_Width+x].m_Flags = 0;
+			else
+				m_pTiles[y*m_Width+x].m_Flags ^= m_pTiles[y*m_Width+x].m_Flags&TILEFLAG_ROTATE ? TILEFLAG_HFLIP : TILEFLAG_VFLIP;
 }
 
 void CLayerTiles::BrushFlipY()
@@ -460,9 +531,16 @@ void CLayerTiles::BrushFlipY()
 			m_pTiles[(m_Height-1-y)*m_Width+x] = Tmp;
 		}
 
+	if(m_Tele || m_Switch || m_Speedup || m_Tune)
+		return;
+
+	bool Rotate = !(m_Game || m_Front) || m_pEditor->m_AllowPlaceUnusedTiles;
 	for(int y = 0; y < m_Height; y++)
 		for(int x = 0; x < m_Width; x++)
-			m_pTiles[y*m_Width+x].m_Flags ^= m_pTiles[y*m_Width+x].m_Flags&TILEFLAG_ROTATE ? TILEFLAG_VFLIP : TILEFLAG_HFLIP;
+			if(!Rotate && !IsRotatableTile(m_pTiles[y*m_Width+x].m_Index))
+				m_pTiles[y*m_Width+x].m_Flags = 0;
+			else
+				m_pTiles[y*m_Width+x].m_Flags ^= m_pTiles[y*m_Width+x].m_Flags&TILEFLAG_ROTATE ? TILEFLAG_VFLIP : TILEFLAG_HFLIP;
 }
 
 void CLayerTiles::BrushRotate(float Amount)
@@ -477,13 +555,19 @@ void CLayerTiles::BrushRotate(float Amount)
 		CTile *pTempData = new CTile[m_Width*m_Height];
 		mem_copy(pTempData, m_pTiles, m_Width*m_Height*sizeof(CTile));
 		CTile *pDst = m_pTiles;
+		bool Rotate = !(m_Game || m_Front) || m_pEditor->m_AllowPlaceUnusedTiles;
 		for(int x = 0; x < m_Width; ++x)
 			for(int y = m_Height-1; y >= 0; --y, ++pDst)
 			{
 				*pDst = pTempData[y*m_Width+x];
-				if(pDst->m_Flags&TILEFLAG_ROTATE)
-					pDst->m_Flags ^= (TILEFLAG_HFLIP|TILEFLAG_VFLIP);
-				pDst->m_Flags ^= TILEFLAG_ROTATE;
+				if(!Rotate && !IsRotatableTile(pDst->m_Index))
+					pDst->m_Flags = 0;
+				else
+				{
+					if(pDst->m_Flags&TILEFLAG_ROTATE)
+						pDst->m_Flags ^= (TILEFLAG_HFLIP|TILEFLAG_VFLIP);
+					pDst->m_Flags ^= TILEFLAG_ROTATE;
+				}
 			}
 
 		int Temp = m_Width;
@@ -505,8 +589,8 @@ void CLayerTiles::Resize(int NewW, int NewH)
 	mem_zero(pNewData, NewW*NewH*sizeof(CTile));
 
 	// copy old data
-	for(int y = 0; y < min(NewH, m_Height); y++)
-		mem_copy(&pNewData[y*NewW], &m_pTiles[y*m_Width], min(m_Width, NewW)*sizeof(CTile));
+	for(int y = 0; y < minimum(NewH, m_Height); y++)
+		mem_copy(&pNewData[y*NewW], &m_pTiles[y*m_Width], minimum(m_Width, NewW)*sizeof(CTile));
 
 	// replace old
 	delete [] m_pTiles;
@@ -532,7 +616,7 @@ void CLayerTiles::Resize(int NewW, int NewH)
 
 	// resize tune layer if available
 	if(m_Game && m_pEditor->m_Map.m_pTuneLayer && (m_pEditor->m_Map.m_pTuneLayer->m_Width != NewW || m_pEditor->m_Map.m_pTuneLayer->m_Height != NewH))
-			m_pEditor->m_Map.m_pTuneLayer->Resize(NewW, NewH);
+		m_pEditor->m_Map.m_pTuneLayer->Resize(NewW, NewH);
 }
 
 void CLayerTiles::Shift(int Direction)
@@ -590,10 +674,10 @@ void CLayerTiles::ShowInfo()
 	Graphics()->TextureSet(m_pEditor->Client()->GetDebugFont());
 	Graphics()->QuadsBegin();
 
-	int StartY = max(0, (int)(ScreenY0/32.0f)-1);
-	int StartX = max(0, (int)(ScreenX0/32.0f)-1);
-	int EndY = min((int)(ScreenY1/32.0f)+1, m_Height);
-	int EndX = min((int)(ScreenX1/32.0f)+1, m_Width);
+	int StartY = maximum(0, (int)(ScreenY0/32.0f)-1);
+	int StartX = maximum(0, (int)(ScreenX0/32.0f)-1);
+	int EndY = minimum((int)(ScreenY1/32.0f)+1, m_Height);
+	int EndX = minimum((int)(ScreenX1/32.0f)+1, m_Width);
 
 	for(int y = StartY; y < EndY; y++)
 		for(int x = StartX; x < EndX; x++)
@@ -622,11 +706,19 @@ int CLayerTiles::RenderProperties(CUIRect *pToolBox)
 {
 	CUIRect Button;
 
-	bool InGameGroup = !find_linear(m_pEditor->m_Map.m_pGameGroup->m_lLayers.all(), this).empty();
-	if(m_pEditor->m_Map.m_pGameLayer == this || m_pEditor->m_Map.m_pTeleLayer == this || m_pEditor->m_Map.m_pSpeedupLayer == this || m_pEditor->m_Map.m_pFrontLayer == this || m_pEditor->m_Map.m_pSwitchLayer == this || m_pEditor->m_Map.m_pTuneLayer == this)
-		InGameGroup = false;
+	bool IsGameLayer = (
+		m_pEditor->m_Map.m_pGameLayer == this
+		|| m_pEditor->m_Map.m_pTeleLayer == this
+		|| m_pEditor->m_Map.m_pSpeedupLayer == this
+		|| m_pEditor->m_Map.m_pFrontLayer == this
+		|| m_pEditor->m_Map.m_pSwitchLayer == this
+		|| m_pEditor->m_Map.m_pTuneLayer == this
+	);
 
-	if(InGameGroup)
+	CLayerGroup *pGroup = m_pEditor->m_Map.m_lGroups[m_pEditor->m_SelectedGroup];
+
+	// Game tiles can only be constructed if the layer is relative to the game layer
+	if(!IsGameLayer && !(pGroup->m_OffsetX % 32) && !(pGroup->m_OffsetY % 32) && pGroup->m_ParallaxX == 100 && pGroup->m_ParallaxY == 100)
 	{
 		pToolBox->HSplitBottom(12.0f, pToolBox, &Button);
 		static int s_ColclButton = 0;
@@ -661,39 +753,60 @@ int CLayerTiles::RenderProperties(CUIRect *pToolBox)
 		}
 		if(Result > -1)
 		{
-			if (Result != TILE_TELECHECKIN && Result != TILE_TELECHECKINEVIL)
-			{
-				CLayerTiles *gl = m_pEditor->m_Map.m_pGameLayer;
+			int OffsetX = -pGroup->m_OffsetX / 32;
+			int OffsetY = -pGroup->m_OffsetY / 32;
 
-				int w = min(gl->m_Width, m_Width);
-				int h = min(gl->m_Height, m_Height);
-				for(int y = 0; y < h; y++)
-					for(int x = 0; x < w; x++)
-						if(GetTile(x, y).m_Index) {
-							CTile result_tile = {(unsigned char)Result};
-							gl->SetTile(x, y, result_tile);
+			if(Result != TILE_TELECHECKIN && Result != TILE_TELECHECKINEVIL)
+			{
+				CLayerTiles *pGLayer = m_pEditor->m_Map.m_pGameLayer;
+
+				if(pGLayer->m_Width < m_Width + OffsetX || pGLayer->m_Height < m_Height + OffsetY)
+				{
+					int NewW = pGLayer->m_Width < m_Width + OffsetX ? m_Width + OffsetX : pGLayer->m_Width;
+					int NewH = pGLayer->m_Height < m_Height + OffsetY ? m_Height + OffsetY : pGLayer->m_Height;
+					pGLayer->Resize(NewW, NewH);
+				}
+
+				for(int y = OffsetY < 0 ? -OffsetY : 0; y < m_Height; y++)
+					for(int x = OffsetX < 0 ? -OffsetX : 0; x < m_Width; x++)
+						if(GetTile(x, y).m_Index)
+						{
+							CTile ResultTile = {(unsigned char)Result};
+							pGLayer->SetTile(x + OffsetX, y + OffsetY, ResultTile);
 						}
 			}
-			else if (m_pEditor->m_Map.m_pTeleLayer)
+			else
 			{
-				CLayerTele *gl = m_pEditor->m_Map.m_pTeleLayer;
+				if(!m_pEditor->m_Map.m_pTeleLayer)
+				{
+					CLayer *pLayer = new CLayerTele(m_Width, m_Height);
+					m_pEditor->m_Map.MakeTeleLayer(pLayer);
+					m_pEditor->m_Map.m_pGameGroup->AddLayer(pLayer);
+				}
 
-				int w = min(gl->m_Width, m_Width);
-				int h = min(gl->m_Height, m_Height);
-				for(int y = 0; y < h; y++)
-					for(int x = 0; x < w; x++)
-						if(m_pTiles[y*m_Width+x].m_Index)
+				CLayerTele *pTLayer = m_pEditor->m_Map.m_pTeleLayer;
+
+				if(pTLayer->m_Width < m_Width + OffsetX || pTLayer->m_Height < m_Height + OffsetY)
+				{
+					int NewW = pTLayer->m_Width < m_Width + OffsetX ? m_Width + OffsetX : pTLayer->m_Width;
+					int NewH = pTLayer->m_Height < m_Height + OffsetY ? m_Height + OffsetY : pTLayer->m_Height;
+					pTLayer->Resize(NewW, NewH);
+				}
+
+				for(int y = OffsetY < 0 ? -OffsetY : 0; y < m_Height; y++)
+					for(int x = OffsetX < 0 ? -OffsetX : 0; x < m_Width; x++)
+						if(GetTile(x, y).m_Index)
 						{
-							gl->m_pTiles[y*gl->m_Width+x].m_Index = TILE_AIR+Result;
-							gl->m_pTeleTile[y*gl->m_Width+x].m_Number = 1;
-							gl->m_pTeleTile[y*gl->m_Width+x].m_Type = TILE_AIR+Result;
+							pTLayer->m_pTiles[(y+OffsetY)*pTLayer->m_Width+x+OffsetX].m_Index = TILE_AIR+Result;
+							pTLayer->m_pTeleTile[(y+OffsetY)*pTLayer->m_Width+x+OffsetX].m_Number = 1;
+							pTLayer->m_pTeleTile[(y+OffsetY)*pTLayer->m_Width+x+OffsetX].m_Type = TILE_AIR+Result;
 						}
 			}
 
 			return 1;
 		}
 	}
-	
+
 	if(m_pEditor->m_Map.m_pGameLayer != this)
 	{
 		if(m_Image >= 0 && m_Image < m_pEditor->m_Map.m_lImages.size() && m_pEditor->m_Map.m_lImages[m_Image]->m_AutoMapper.IsLoaded() &&
@@ -743,10 +856,10 @@ int CLayerTiles::RenderProperties(CUIRect *pToolBox)
 	Color |= m_Color.a;
 
 	CProperty aProps[] = {
-		{"Width", m_Width, PROPTYPE_INT_SCROLL, 1, 1000000000},
-		{"Height", m_Height, PROPTYPE_INT_SCROLL, 1, 1000000000},
+		{"Width", m_Width, PROPTYPE_INT_SCROLL, 1, 100000},
+		{"Height", m_Height, PROPTYPE_INT_SCROLL, 1, 100000},
 		{"Shift", 0, PROPTYPE_SHIFT, 0, 0},
-		{"Shift by", m_pEditor->m_ShiftBy, PROPTYPE_INT_SCROLL, 1, 1000000000},
+		{"Shift by", m_pEditor->m_ShiftBy, PROPTYPE_INT_SCROLL, 1, 100000},
 		{"Image", m_Image, PROPTYPE_IMAGE, 0, 0},
 		{"Color", Color, PROPTYPE_COLOR, 0, 0},
 		{"Color Env", m_ColorEnv+1, PROPTYPE_INT_STEP, 0, m_pEditor->m_Map.m_lEnvelopes.size()+1},
@@ -756,7 +869,7 @@ int CLayerTiles::RenderProperties(CUIRect *pToolBox)
 		{0},
 	};
 
-	if(m_pEditor->m_Map.m_pGameLayer == this || m_pEditor->m_Map.m_pTeleLayer == this || m_pEditor->m_Map.m_pSpeedupLayer == this || m_pEditor->m_Map.m_pFrontLayer == this || m_pEditor->m_Map.m_pSwitchLayer == this || m_pEditor->m_Map.m_pTuneLayer == this) // remove the image and color properties if this is the game/tele/speedup/front/switch layer
+	if(IsGameLayer) // remove the image and color properties if this is a game layer
 	{
 		aProps[PROP_IMAGE].m_pName = 0;
 		aProps[PROP_COLOR].m_pName = 0;
@@ -888,8 +1001,8 @@ void CLayerTele::Resize(int NewW, int NewH)
 	mem_zero(pNewTeleData, NewW*NewH*sizeof(CTeleTile));
 
 	// copy old data
-	for(int y = 0; y < min(NewH, m_Height); y++)
-		mem_copy(&pNewTeleData[y*NewW], &m_pTeleTile[y*m_Width], min(m_Width, NewW)*sizeof(CTeleTile));
+	for(int y = 0; y < minimum(NewH, m_Height); y++)
+		mem_copy(&pNewTeleData[y*NewW], &m_pTeleTile[y*m_Width], minimum(m_Width, NewW)*sizeof(CTeleTile));
 
 	// replace old
 	delete [] m_pTeleTile;
@@ -952,6 +1065,16 @@ void CLayerTele::Shift(int Direction)
 	}
 }
 
+bool CLayerTele::IsEmpty(CLayerTiles *pLayer)
+{
+	for(int y = 0; y < pLayer->m_Height; y++)
+		for(int x = 0; x < pLayer->m_Width; x++)
+			if(IsValidTeleTile(pLayer->GetTile(x, y).m_Index))
+				return false;
+
+	return true;
+}
+
 void CLayerTele::BrushDraw(CLayer *pBrush, float wx, float wy)
 {
 	if(m_Readonly)
@@ -963,15 +1086,21 @@ void CLayerTele::BrushDraw(CLayer *pBrush, float wx, float wy)
 	if(str_comp(l->m_aFileName, m_pEditor->m_aFileName))
 		m_pEditor->m_TeleNumber = l->m_TeleNum;
 
+	bool Destructive = m_pEditor->m_BrushDrawDestructive || IsEmpty(l);
+
 	for(int y = 0; y < l->m_Height; y++)
 		for(int x = 0; x < l->m_Width; x++)
 		{
 			int fx = x+sx;
 			int fy = y+sy;
+
 			if(fx<0 || fx >= m_Width || fy < 0 || fy >= m_Height)
 				continue;
 
-			if(l->m_pTiles[y*l->m_Width+x].m_Index == TILE_TELEIN || l->m_pTiles[y*l->m_Width+x].m_Index == TILE_TELEINEVIL || l->m_pTiles[y*l->m_Width+x].m_Index == TILE_TELECHECKINEVIL || l->m_pTiles[y*l->m_Width+x].m_Index == TILE_TELEOUT || l->m_pTiles[y*l->m_Width+x].m_Index == TILE_TELECHECK || l->m_pTiles[y*l->m_Width+x].m_Index == TILE_TELECHECKOUT || l->m_pTiles[y*l->m_Width+x].m_Index == TILE_TELECHECKIN || l->m_pTiles[y*l->m_Width+x].m_Index == TILE_TELEINWEAPON || l->m_pTiles[y*l->m_Width+x].m_Index == TILE_TELEINHOOK)
+			if(!Destructive && GetTile(fx, fy).m_Index)
+				continue;
+
+			if(IsValidTeleTile(l->m_pTiles[y*l->m_Width+x].m_Index))
 			{
 				if(m_pEditor->m_TeleNumber != l->m_TeleNum)
 				{
@@ -1083,6 +1212,8 @@ void CLayerTele::FillSelection(bool Empty, CLayer *pBrush, CUIRect Rect)
 
 	CLayerTele *pLt = static_cast<CLayerTele*>(pBrush);
 
+	bool Destructive = m_pEditor->m_BrushDrawDestructive || Empty || IsEmpty(pLt);
+
 	for(int y = 0; y < h; y++)
 	{
 		for(int x = 0; x < w; x++)
@@ -1093,7 +1224,10 @@ void CLayerTele::FillSelection(bool Empty, CLayer *pBrush, CUIRect Rect)
 			if(fx < 0 || fx >= m_Width || fy < 0 || fy >= m_Height)
 				continue;
 
-			if(Empty || !(pLt->m_pTiles[(y*pLt->m_Width + x%pLt->m_Width) % (pLt->m_Width*pLt->m_Height)]).m_Index) // air chosen: reset
+			if(!Destructive && GetTile(fx, fy).m_Index)
+				continue;
+
+			if(Empty || !IsValidTeleTile((pLt->m_pTiles[(y*pLt->m_Width + x%pLt->m_Width) % (pLt->m_Width*pLt->m_Height)]).m_Index))
 			{
 				m_pTiles[fy*m_Width+fx].m_Index = 0;
 				m_pTeleTile[fy*m_Width+fx].m_Type = 0;
@@ -1139,8 +1273,8 @@ void CLayerSpeedup::Resize(int NewW, int NewH)
 	mem_zero(pNewSpeedupData, NewW*NewH*sizeof(CSpeedupTile));
 
 	// copy old data
-	for(int y = 0; y < min(NewH, m_Height); y++)
-		mem_copy(&pNewSpeedupData[y*NewW], &m_pSpeedupTile[y*m_Width], min(m_Width, NewW)*sizeof(CSpeedupTile));
+	for(int y = 0; y < minimum(NewH, m_Height); y++)
+		mem_copy(&pNewSpeedupData[y*NewW], &m_pSpeedupTile[y*m_Width], minimum(m_Width, NewW)*sizeof(CSpeedupTile));
 
 	// replace old
 	delete [] m_pSpeedupTile;
@@ -1203,6 +1337,16 @@ void CLayerSpeedup::Shift(int Direction)
 	}
 }
 
+bool CLayerSpeedup::IsEmpty(CLayerTiles *pLayer)
+{
+	for(int y = 0; y < pLayer->m_Height; y++)
+		for(int x = 0; x < pLayer->m_Width; x++)
+			if(IsValidSpeedupTile(pLayer->GetTile(x, y).m_Index))
+				return false;
+
+	return true;
+}
+
 void CLayerSpeedup::BrushDraw(CLayer *pBrush, float wx, float wy)
 {
 	if(m_Readonly)
@@ -1218,15 +1362,21 @@ void CLayerSpeedup::BrushDraw(CLayer *pBrush, float wx, float wy)
 		m_pEditor->m_SpeedupMaxSpeed = l->m_SpeedupMaxSpeed;
 	}
 
+	bool Destructive = m_pEditor->m_BrushDrawDestructive || IsEmpty(l);
+
 	for(int y = 0; y < l->m_Height; y++)
 		for(int x = 0; x < l->m_Width; x++)
 		{
 			int fx = x+sx;
 			int fy = y+sy;
+
 			if(fx<0 || fx >= m_Width || fy < 0 || fy >= m_Height)
 				continue;
 
-			if(l->m_pTiles[y*l->m_Width+x].m_Index == TILE_BOOST)
+			if(!Destructive && GetTile(fx, fy).m_Index)
+				continue;
+
+			if(IsValidSpeedupTile(l->m_pTiles[y*l->m_Width+x].m_Index))
 			{
 				if(m_pEditor->m_SpeedupAngle != l->m_SpeedupAngle || m_pEditor->m_SpeedupForce != l->m_SpeedupForce || m_pEditor->m_SpeedupMaxSpeed != l->m_SpeedupMaxSpeed)
 				{
@@ -1349,6 +1499,8 @@ void CLayerSpeedup::FillSelection(bool Empty, CLayer *pBrush, CUIRect Rect)
 
 	CLayerSpeedup *pLt = static_cast<CLayerSpeedup*>(pBrush);
 
+	bool Destructive = m_pEditor->m_BrushDrawDestructive || Empty || IsEmpty(pLt);
+
 	for(int y = 0; y < h; y++)
 	{
 		for(int x = 0; x < w; x++)
@@ -1359,7 +1511,10 @@ void CLayerSpeedup::FillSelection(bool Empty, CLayer *pBrush, CUIRect Rect)
 			if(fx < 0 || fx >= m_Width || fy < 0 || fy >= m_Height)
 				continue;
 
-			if(Empty || (pLt->m_pTiles[(y*pLt->m_Width + x%pLt->m_Width) % (pLt->m_Width*pLt->m_Height)]).m_Index != TILE_BOOST) // no speed up tile chosen: reset
+			if(!Destructive && GetTile(fx, fy).m_Index)
+				continue;
+
+			if(Empty || !IsValidSpeedupTile((pLt->m_pTiles[(y*pLt->m_Width + x%pLt->m_Width) % (pLt->m_Width*pLt->m_Height)]).m_Index)) // no speed up tile chosen: reset
 			{
 				m_pTiles[fy*m_Width+fx].m_Index = 0;
 				m_pSpeedupTile[fy*m_Width+fx].m_Force = 0;
@@ -1430,34 +1585,6 @@ void CLayerFront::Resize(int NewW, int NewH)
 		m_pEditor->m_Map.m_pGameLayer->Resize(NewW, NewH);
 }
 
-void CLayerFront::Shift(int Direction)
-{
-	CLayerTiles::Shift(Direction);
-}
-
-void CLayerFront::BrushDraw(CLayer *pBrush, float wx, float wy)
-{
-	if(m_Readonly)
-		return;
-
-	//
-	CLayerTiles *l = (CLayerTiles *)pBrush;
-	int sx = ConvertX(wx);
-	int sy = ConvertY(wy);
-
-	for(int y = 0; y < l->m_Height; y++)
-		for(int x = 0; x < l->m_Width; x++)
-		{
-			int fx = x+sx;
-			int fy = y+sy;
-			if(fx<0 || fx >= m_Width || fy < 0 || fy >= m_Height)
-				continue;
-
-			SetTile(fx, fy, l->m_pTiles[y*l->m_Width+x]);
-		}
-	FlagModified(sx, sy, l->m_Width, l->m_Height);
-}
-
 CLayerSwitch::CLayerSwitch(int w, int h)
 : CLayerTiles(w, h)
 {
@@ -1481,8 +1608,8 @@ void CLayerSwitch::Resize(int NewW, int NewH)
 	mem_zero(pNewSwitchData, NewW*NewH*sizeof(CSwitchTile));
 
 	// copy old data
-	for(int y = 0; y < min(NewH, m_Height); y++)
-		mem_copy(&pNewSwitchData[y*NewW], &m_pSwitchTile[y*m_Width], min(m_Width, NewW)*sizeof(CSwitchTile));
+	for(int y = 0; y < minimum(NewH, m_Height); y++)
+		mem_copy(&pNewSwitchData[y*NewW], &m_pSwitchTile[y*m_Width], minimum(m_Width, NewW)*sizeof(CSwitchTile));
 
 	// replace old
 	delete [] m_pSwitchTile;
@@ -1545,6 +1672,16 @@ void CLayerSwitch::Shift(int Direction)
 	}
 }
 
+bool CLayerSwitch::IsEmpty(CLayerTiles *pLayer)
+{
+	for(int y = 0; y < pLayer->m_Height; y++)
+		for(int x = 0; x < pLayer->m_Width; x++)
+			if(IsValidSwitchTile(pLayer->GetTile(x, y).m_Index))
+				return false;
+
+	return true;
+}
+
 void CLayerSwitch::BrushDraw(CLayer *pBrush, float wx, float wy)
 {
 	if(m_Readonly)
@@ -1559,29 +1696,21 @@ void CLayerSwitch::BrushDraw(CLayer *pBrush, float wx, float wy)
 		m_pEditor->m_SwitchDelay = l->m_SwitchDelay;
 	}
 
+	bool Destructive = m_pEditor->m_BrushDrawDestructive || IsEmpty(l);
+
 	for(int y = 0; y < l->m_Height; y++)
 		for(int x = 0; x < l->m_Width; x++)
 		{
 			int fx = x+sx;
 			int fy = y+sy;
+
 			if(fx<0 || fx >= m_Width || fy < 0 || fy >= m_Height)
 				continue;
 
-			if((l->m_pTiles[y*l->m_Width+x].m_Index >= (ENTITY_ARMOR_1 + ENTITY_OFFSET) && l->m_pTiles[y*l->m_Width+x].m_Index <= (ENTITY_DOOR + ENTITY_OFFSET))
-				|| l->m_pTiles[y*l->m_Width+x].m_Index == TILE_HIT_START
-				|| l->m_pTiles[y*l->m_Width+x].m_Index == TILE_HIT_END
-				|| l->m_pTiles[y*l->m_Width+x].m_Index == TILE_SWITCHOPEN
-				|| l->m_pTiles[y*l->m_Width+x].m_Index == TILE_SWITCHCLOSE
-				|| l->m_pTiles[y*l->m_Width+x].m_Index == TILE_SWITCHTIMEDOPEN
-				|| l->m_pTiles[y*l->m_Width+x].m_Index == TILE_SWITCHTIMEDCLOSE
-				|| l->m_pTiles[y*l->m_Width+x].m_Index == TILE_FREEZE
-				|| l->m_pTiles[y*l->m_Width+x].m_Index == TILE_DFREEZE
-				|| l->m_pTiles[y*l->m_Width+x].m_Index == TILE_DUNFREEZE
-				|| l->m_pTiles[y*l->m_Width+x].m_Index == TILE_JUMP
-				|| l->m_pTiles[y*l->m_Width+x].m_Index == TILE_PENALTY
-				|| l->m_pTiles[y*l->m_Width+x].m_Index == TILE_BONUS
-				|| l->m_pTiles[y*l->m_Width+x].m_Index == TILE_ALLOW_TELE_GUN
-				|| l->m_pTiles[y*l->m_Width+x].m_Index == TILE_ALLOW_BLUE_TELE_GUN)
+			if(!Destructive && GetTile(fx, fy).m_Index)
+				continue;
+
+			if(IsValidSwitchTile(l->m_pTiles[y*l->m_Width+x].m_Index))
 			{
 				if(m_pEditor->m_SwitchNum != l->m_SwitchNumber || m_pEditor->m_SwitchDelay != l->m_SwitchDelay)
 				{
@@ -1625,6 +1754,68 @@ void CLayerSwitch::BrushDraw(CLayer *pBrush, float wx, float wy)
 	FlagModified(sx, sy, l->m_Width, l->m_Height);
 }
 
+void CLayerSwitch::BrushFlipX()
+{
+	CLayerTiles::BrushFlipX();
+
+	for(int y = 0; y < m_Height; y++)
+		for(int x = 0; x < m_Width/2; x++)
+		{
+			CSwitchTile Tmp = m_pSwitchTile[y*m_Width+x];
+			m_pSwitchTile[y*m_Width+x] = m_pSwitchTile[y*m_Width+m_Width-1-x];
+			m_pSwitchTile[y*m_Width+m_Width-1-x] = Tmp;
+		}
+}
+
+void CLayerSwitch::BrushFlipY()
+{
+	CLayerTiles::BrushFlipY();
+
+	for(int y = 0; y < m_Height/2; y++)
+		for(int x = 0; x < m_Width; x++)
+		{
+			CSwitchTile Tmp = m_pSwitchTile[y*m_Width+x];
+			m_pSwitchTile[y*m_Width+x] = m_pSwitchTile[(m_Height-1-y)*m_Width+x];
+			m_pSwitchTile[(m_Height-1-y)*m_Width+x] = Tmp;
+		}
+}
+
+void CLayerSwitch::BrushRotate(float Amount)
+{
+	int Rotation = (round_to_int(360.0f*Amount/(pi*2))/90)%4;	// 0=0°, 1=90°, 2=180°, 3=270°
+	if(Rotation < 0)
+		Rotation +=4;
+
+	if(Rotation == 1 || Rotation == 3)
+	{
+		// 90° rotation
+		CSwitchTile *pTempData1 = new CSwitchTile[m_Width*m_Height];
+		CTile *pTempData2 = new CTile[m_Width*m_Height];
+		mem_copy(pTempData1, m_pSwitchTile, m_Width*m_Height*sizeof(CSwitchTile));
+		mem_copy(pTempData2, m_pTiles, m_Width*m_Height*sizeof(CTile));
+		CSwitchTile *pDst1 = m_pSwitchTile;
+		CTile *pDst2 = m_pTiles;
+		for(int x = 0; x < m_Width; ++x)
+			for(int y = m_Height-1; y >= 0; --y, ++pDst1, ++pDst2)
+			{
+				*pDst1 = pTempData1[y*m_Width+x];
+				*pDst2 = pTempData2[y*m_Width+x];
+			}
+
+		int Temp = m_Width;
+		m_Width = m_Height;
+		m_Height = Temp;
+		delete[] pTempData1;
+		delete[] pTempData2;
+	}
+
+	if(Rotation == 2 || Rotation == 3)
+	{
+		BrushFlipX();
+		BrushFlipY();
+	}
+}
+
 void CLayerSwitch::FillSelection(bool Empty, CLayer *pBrush, CUIRect Rect)
 {
 	if(m_Readonly || (!Empty && pBrush->m_Type != LAYERTYPE_TILES))
@@ -1641,6 +1832,8 @@ void CLayerSwitch::FillSelection(bool Empty, CLayer *pBrush, CUIRect Rect)
 
 	CLayerSwitch *pLt = static_cast<CLayerSwitch*>(pBrush);
 
+	bool Destructive = m_pEditor->m_BrushDrawDestructive || Empty || IsEmpty(pLt);
+
 	for(int y = 0; y < h; y++)
 	{
 		for(int x = 0; x < w; x++)
@@ -1651,7 +1844,10 @@ void CLayerSwitch::FillSelection(bool Empty, CLayer *pBrush, CUIRect Rect)
 			if(fx < 0 || fx >= m_Width || fy < 0 || fy >= m_Height)
 				continue;
 
-			if(Empty || !(pLt->m_pTiles[(y*pLt->m_Width + x%pLt->m_Width) % (pLt->m_Width*pLt->m_Height)]).m_Index) // at least reset the tile if air is chosen
+			if(!Destructive && GetTile(fx, fy).m_Index)
+				continue;
+
+			if(Empty || !IsValidSwitchTile((pLt->m_pTiles[(y*pLt->m_Width + x%pLt->m_Width) % (pLt->m_Width*pLt->m_Height)]).m_Index))
 			{
 				m_pTiles[fy*m_Width+fx].m_Index = 0;
 				m_pSwitchTile[fy*m_Width+fx].m_Type = 0;
@@ -1705,8 +1901,8 @@ void CLayerTune::Resize(int NewW, int NewH)
 	mem_zero(pNewTuneData, NewW*NewH*sizeof(CTuneTile));
 
 	// copy old data
-	for(int y = 0; y < min(NewH, m_Height); y++)
-		mem_copy(&pNewTuneData[y*NewW], &m_pTuneTile[y*m_Width], min(m_Width, NewW)*sizeof(CTuneTile));
+	for(int y = 0; y < minimum(NewH, m_Height); y++)
+		mem_copy(&pNewTuneData[y*NewW], &m_pTuneTile[y*m_Width], minimum(m_Width, NewW)*sizeof(CTuneTile));
 
 	// replace old
 	delete [] m_pTuneTile;
@@ -1769,6 +1965,16 @@ void CLayerTune::Shift(int Direction)
 	}
 }
 
+bool CLayerTune::IsEmpty(CLayerTiles *pLayer)
+{
+	for(int y = 0; y < pLayer->m_Height; y++)
+		for(int x = 0; x < pLayer->m_Width; x++)
+			if(IsValidTuneTile(pLayer->GetTile(x, y).m_Index))
+				return false;
+
+	return true;
+}
+
 void CLayerTune::BrushDraw(CLayer *pBrush, float wx, float wy)
 {
 	if(m_Readonly)
@@ -1782,15 +1988,21 @@ void CLayerTune::BrushDraw(CLayer *pBrush, float wx, float wy)
 		m_pEditor->m_TuningNum = l->m_TuningNumber;
 	}
 
+	bool Destructive = m_pEditor->m_BrushDrawDestructive || IsEmpty(l);
+
 	for(int y = 0; y < l->m_Height; y++)
 			for(int x = 0; x < l->m_Width; x++)
 			{
 				int fx = x+sx;
 				int fy = y+sy;
+
 				if(fx<0 || fx >= m_Width || fy < 0 || fy >= m_Height)
 					continue;
 
-				if(l->m_pTiles[y*l->m_Width+x].m_Index == TILE_TUNE1)
+				if(!Destructive && GetTile(fx, fy).m_Index)
+					continue;
+
+				if(IsValidTuneTile(l->m_pTiles[y*l->m_Width+x].m_Index))
 				{
 					if(m_pEditor->m_TuningNum != l->m_TuningNumber)
 					{
@@ -1902,6 +2114,8 @@ void CLayerTune::FillSelection(bool Empty, CLayer *pBrush, CUIRect Rect)
 
 		CLayerTune *pLt = static_cast<CLayerTune*>(pBrush);
 
+		bool Destructive = m_pEditor->m_BrushDrawDestructive || Empty || IsEmpty(pLt);
+
 		for(int y = 0; y < h; y++)
 		{
 			for(int x = 0; x < w; x++)
@@ -1912,7 +2126,10 @@ void CLayerTune::FillSelection(bool Empty, CLayer *pBrush, CUIRect Rect)
 				if(fx < 0 || fx >= m_Width || fy < 0 || fy >= m_Height)
 					continue;
 
-				if(Empty || (pLt->m_pTiles[(y*pLt->m_Width + x%pLt->m_Width) % (pLt->m_Width*pLt->m_Height)]).m_Index != TILE_TUNE1) // \o/ this fixes editor bug; TODO: use IsUsedInThisLayer here
+				if(!Destructive && GetTile(fx, fy).m_Index)
+					continue;
+
+				if(Empty || !IsValidTuneTile((pLt->m_pTiles[(y*pLt->m_Width + x%pLt->m_Width) % (pLt->m_Width*pLt->m_Height)]).m_Index)) // \o/ this fixes editor bug; TODO: use IsUsedInThisLayer here
 				{
 					m_pTiles[fy*m_Width+fx].m_Index = 0;
 					m_pTuneTile[fy*m_Width+fx].m_Type = 0;
