@@ -75,9 +75,9 @@ void CEditor::UiDoPopupMenu()
 			Corners = CUI::CORNER_R|CUI::CORNER_B;
 
 		CUIRect r = s_UiPopups[i].m_Rect;
-		RenderTools()->DrawUIRect(&r, vec4(0.5f,0.5f,0.5f,0.75f), Corners, 3.0f);
+		RenderTools()->DrawUIRect(&r, ColorRGBA(0.5f,0.5f,0.5f,0.75f), Corners, 3.0f);
 		r.Margin(1.0f, &r);
-		RenderTools()->DrawUIRect(&r, vec4(0,0,0,0.75f), Corners, 3.0f);
+		RenderTools()->DrawUIRect(&r, ColorRGBA(0,0,0,0.75f), Corners, 3.0f);
 		r.Margin(4.0f, &r);
 
 		if(s_UiPopups[i].m_pfnFunc(this, r))
@@ -112,7 +112,7 @@ int CEditor::PopupGroup(CEditor *pEditor, CUIRect View)
 		if(pEditor->DoButton_Editor(&s_DeleteButton, "Delete group", 0, &Button, 0, "Delete group"))
 		{
 			pEditor->m_Map.DeleteGroup(pEditor->m_SelectedGroup);
-			pEditor->m_SelectedGroup = max(0, pEditor->m_SelectedGroup-1);
+			pEditor->m_SelectedGroup = maximum(0, pEditor->m_SelectedGroup-1);
 			return 1;
 		}
 	}
@@ -517,8 +517,8 @@ int CEditor::PopupQuad(CEditor *pEditor, CUIRect View)
 		{
 			for(int k = 1; k < 4; k++)
 			{
-				lQuads[i]->m_aPoints[k].x = 1000.0f * (int(lQuads[i]->m_aPoints[k].x) / 1000);
-				lQuads[i]->m_aPoints[k].y = 1000.0f * (int(lQuads[i]->m_aPoints[k].y) / 1000);
+				lQuads[i]->m_aPoints[k].x = 1000.0f * (lQuads[i]->m_aPoints[k].x / 1000);
+				lQuads[i]->m_aPoints[k].y = 1000.0f * (lQuads[i]->m_aPoints[k].y / 1000);
 			}
 			pEditor->m_Map.m_Modified = true;
 		}
@@ -838,6 +838,8 @@ int CEditor::PopupPoint(CEditor *pEditor, CUIRect View)
 		PROP_POS_X=0,
 		PROP_POS_Y,
 		PROP_COLOR,
+		PROP_TEX_U,
+		PROP_TEX_V,
 		NUM_PROPS,
 	};
 
@@ -849,11 +851,15 @@ int CEditor::PopupPoint(CEditor *pEditor, CUIRect View)
 
 	int x = pCurrentQuad->m_aPoints[pEditor->m_SelectedQuadPoint].x/1000;
 	int y = pCurrentQuad->m_aPoints[pEditor->m_SelectedQuadPoint].y/1000;
+	int tu = pCurrentQuad->m_aTexcoords[pEditor->m_SelectedQuadPoint].x/1000 * 1024;
+	int tv = pCurrentQuad->m_aTexcoords[pEditor->m_SelectedQuadPoint].y/1000 * 1024;
 
 	CProperty aProps[] = {
 		{"Pos X", x, PROPTYPE_INT_SCROLL, -1000000, 1000000},
 		{"Pos Y", y, PROPTYPE_INT_SCROLL, -1000000, 1000000},
 		{"Color", Color, PROPTYPE_COLOR, -1, pEditor->m_Map.m_lEnvelopes.size()},
+		{"Tex U", tu, PROPTYPE_INT_SCROLL, -1000000, 1000000},
+		{"Tex V", tv, PROPTYPE_INT_SCROLL, -1000000, 1000000},
 		{0},
 	};
 
@@ -865,6 +871,8 @@ int CEditor::PopupPoint(CEditor *pEditor, CUIRect View)
 
 	float OffsetX = NewVal*1000-x*1000;
 	float OffsetY = NewVal*1000-y*1000;
+	float OffsetU = NewVal*1000/1024-tu*1000/1024;
+	float OffsetV = NewVal*1000/1024-tv*1000/1024;
 
 	for(int i = 0; i < lQuads.size(); ++i)
 	{
@@ -893,6 +901,18 @@ int CEditor::PopupPoint(CEditor *pEditor, CUIRect View)
 				}
 			}
 		}
+		if(Prop == PROP_TEX_U)
+		{
+			for(int v = 0; v < 4; v++)
+				if(pEditor->m_SelectedPoints&(1<<v))
+					lQuads[i]->m_aTexcoords[v].x += OffsetU;
+		}
+		if(Prop == PROP_TEX_V)
+		{
+			for(int v = 0; v < 4; v++)
+				if(pEditor->m_SelectedPoints&(1<<v))
+					lQuads[i]->m_aTexcoords[v].y += OffsetV;
+		}
 	}
 
 	return 0;
@@ -1192,7 +1212,7 @@ int CEditor::PopupSelectImage(CEditor *pEditor, CUIRect View)
 			ImageView.w = ImageView.h;
 		else
 			ImageView.h = ImageView.w;
-		float Max = (float)(max(pEditor->m_Map.m_lImages[ShowImage]->m_Width, pEditor->m_Map.m_lImages[ShowImage]->m_Height));
+		float Max = (float)(maximum(pEditor->m_Map.m_lImages[ShowImage]->m_Width, pEditor->m_Map.m_lImages[ShowImage]->m_Height));
 		ImageView.w *= pEditor->m_Map.m_lImages[ShowImage]->m_Width/Max;
 		ImageView.h *= pEditor->m_Map.m_lImages[ShowImage]->m_Height/Max;
 		pEditor->Graphics()->TextureSet(pEditor->m_Map.m_lImages[ShowImage]->m_TexID);
@@ -1414,7 +1434,7 @@ int CEditor::PopupTele(CEditor *pEditor, CUIRect View)
 
 	static int s_aIds[NUM_PROPS] = {0};
 	int NewVal = 0;
-	static vec4 s_color = vec4(1,1,1,0.5f);
+	static ColorRGBA s_color = ColorRGBA(1,1,1,0.5f);
 
 	int Prop = pEditor->DoProperties(&View, aProps, s_aIds, &NewVal, s_color);
 
@@ -1429,13 +1449,13 @@ int CEditor::PopupTele(CEditor *pEditor, CUIRect View)
 			{
 				if(gl->m_pTeleTile[y*gl->m_Width+x].m_Number == NewVal)
 				{
-					s_color = vec4(1,0.5f,0.5f,0.5f);
+					s_color = ColorRGBA(1,0.5f,0.5f,0.5f);
 					goto done;
 				}
 			}
 		}
 
-		s_color = vec4(0.5f,1,0.5f,0.5f);
+		s_color = ColorRGBA(0.5f,1,0.5f,0.5f);
 
 		done:
 		pEditor->m_TeleNumber = NewVal;
@@ -1498,7 +1518,7 @@ int CEditor::PopupSwitch(CEditor *pEditor, CUIRect View)
 
 	static int s_aIds[NUM_PROPS] = {0};
 	int NewVal = 0;
-	static vec4 s_color = vec4(1,1,1,0.5f);
+	static ColorRGBA s_color = ColorRGBA(1,1,1,0.5f);
 	int Prop = pEditor->DoProperties(&View, aProps, s_aIds, &NewVal, s_color);
 
 	if(Prop == PROP_SwitchNumber)
@@ -1512,13 +1532,13 @@ int CEditor::PopupSwitch(CEditor *pEditor, CUIRect View)
 			{
 				if(gl->m_pSwitchTile[y*gl->m_Width+x].m_Number == NewVal)
 				{
-					s_color = vec4(1,0.5f,0.5f,0.5f);
+					s_color = ColorRGBA(1,0.5f,0.5f,0.5f);
 					goto done;
 				}
 			}
 		}
 
-		s_color = vec4(0.5f,1,0.5f,0.5f);
+		s_color = ColorRGBA(0.5f,1,0.5f,0.5f);
 
 		done:
 		pEditor->m_SwitchNum = NewVal;
@@ -1565,16 +1585,16 @@ int CEditor::PopupColorPicker(CEditor *pEditor, CUIRect View)
 	pEditor->Graphics()->QuadsBegin();
 
 	// base: white - hue
-	vec3 hsv = pEditor->ms_PickerColor;
+	ColorHSVA hsv = pEditor->ms_PickerColor;
 	IGraphics::CColorVertex ColorArray[4];
 
-	vec3 c = HsvToRgb(vec3(hsv.x, 0.0f, 1.0f));
+	ColorRGBA c = color_cast<ColorRGBA>(ColorHSVA(hsv.x, 0.0f, 1.0f));
 	ColorArray[0] = IGraphics::CColorVertex(0, c.r, c.g, c.b, 1.0f);
-	c = HsvToRgb(vec3(hsv.x, 1.0f, 1.0f));
+	c = color_cast<ColorRGBA>(ColorHSVA(hsv.x, 1.0f, 1.0f));
 	ColorArray[1] = IGraphics::CColorVertex(1, c.r, c.g, c.b, 1.0f);
-	c = HsvToRgb(vec3(hsv.x, 1.0f, 1.0f));
+	c = color_cast<ColorRGBA>(ColorHSVA(hsv.x, 1.0f, 1.0f));
 	ColorArray[2] = IGraphics::CColorVertex(2, c.r, c.g, c.b, 1.0f);
-	c = HsvToRgb(vec3(hsv.x, 0.0f, 1.0f));
+	c = color_cast<ColorRGBA>(ColorHSVA(hsv.x, 0.0f, 1.0f));
 	ColorArray[3] = IGraphics::CColorVertex(3, c.r, c.g, c.b, 1.0f);
 
 	pEditor->Graphics()->SetColorVertex(ColorArray, 4);
@@ -1624,12 +1644,12 @@ int CEditor::PopupColorPicker(CEditor *pEditor, CUIRect View)
 	};
 
 	pEditor->Graphics()->QuadsBegin();
-	vec4 ColorTop, ColorBottom;
+	ColorRGBA ColorTop, ColorBottom;
 	float Offset = HuePicker.h/6.0f;
 	for(int j = 0; j < 6; j++)
 	{
-		ColorTop = vec4(s_aColorIndices[j][0], s_aColorIndices[j][1], s_aColorIndices[j][2], 1.0f);
-		ColorBottom = vec4(s_aColorIndices[j+1][0], s_aColorIndices[j+1][1], s_aColorIndices[j+1][2], 1.0f);
+		ColorTop = ColorRGBA(s_aColorIndices[j][0], s_aColorIndices[j][1], s_aColorIndices[j][2], 1.0f);
+		ColorBottom = ColorRGBA(s_aColorIndices[j+1][0], s_aColorIndices[j+1][1], s_aColorIndices[j+1][2], 1.0f);
 
 		ColorArray[0] = IGraphics::CColorVertex(0, ColorTop.r, ColorTop.g, ColorTop.b, ColorTop.a);
 		ColorArray[1] = IGraphics::CColorVertex(1, ColorTop.r, ColorTop.g, ColorTop.b, ColorTop.a);
