@@ -56,13 +56,13 @@ enum
 	ENTITY_POWERUP_NINJA,
 	ENTITY_WEAPON_RIFLE,
 	//DDRace - Main Lasers
-	ENTITY_LASER_FAST_CW,
-	ENTITY_LASER_NORMAL_CW,
-	ENTITY_LASER_SLOW_CW,
-	ENTITY_LASER_STOP,
-	ENTITY_LASER_SLOW_CCW,
-	ENTITY_LASER_NORMAL_CCW,
 	ENTITY_LASER_FAST_CCW,
+	ENTITY_LASER_NORMAL_CCW,
+	ENTITY_LASER_SLOW_CCW,
+	ENTITY_LASER_STOP,
+	ENTITY_LASER_SLOW_CW,
+	ENTITY_LASER_NORMAL_CW,
+	ENTITY_LASER_FAST_CW,
 	//DDRace - Laser Modifiers
 	ENTITY_LASER_SHORT,
 	ENTITY_LASER_MEDIUM,
@@ -131,6 +131,8 @@ enum
 	TILE_REFILL_JUMPS = 32,
 	TILE_BEGIN,
 	TILE_END,
+	TILE_CHECKPOINT_FIRST = 35,
+	TILE_CHECKPOINT_LAST = 59,
 	TILE_STOP = 60,
 	TILE_STOPS,
 	TILE_STOPA,
@@ -164,6 +166,14 @@ enum
 	TILE_TELE_GRENADE_DISABLE = 113,
 	TILE_TELE_LASER_ENABLE = 128,
 	TILE_TELE_LASER_DISABLE = 129,
+	TILE_CREDITS_1 = 140,
+	TILE_CREDITS_2 = 141,
+	TILE_CREDITS_3 = 142,
+	TILE_CREDITS_4 = 143,
+	TILE_CREDITS_5 = 156,
+	TILE_CREDITS_6 = 157,
+	TILE_CREDITS_7 = 158,
+	TILE_CREDITS_8 = 159,
 	TILE_ENTITIES_OFF_1 = 190,
 	TILE_ENTITIES_OFF_2,
 	//End of higher tiles
@@ -471,6 +481,8 @@ bool IsValidFrontTile(int Index);
 bool IsValidTeleTile(int Index);
 bool IsValidSpeedupTile(int Index);
 bool IsValidSwitchTile(int Index);
+bool IsValidTuneTile(int Index);
 bool IsValidEntity(int Index);
+bool IsRotatableTile(int Index);
 
 #endif
