@@ -105,6 +105,7 @@ void CGameContext::ConSuper(IConsole::IResult *pResult, void *pUserData)
 	if (pChr && !pChr->m_Super)
 	{
 		pChr->m_Super = true;
+		pSelf->m_World.m_Core.m_apCharacters[pResult->m_ClientID]->m_Super = true;
 		pChr->UnFreeze();
 		pChr->m_TeamBeforeSuper = pChr->Team();
 		pChr->Teams()->SetCharacterTeam(pResult->m_ClientID, TEAM_SUPER);
@@ -121,6 +122,7 @@ void CGameContext::ConUnSuper(IConsole::IResult *pResult, void *pUserData)
 	if (pChr && pChr->m_Super)
 	{
 		pChr->m_Super = false;
+		pSelf->m_World.m_Core.m_apCharacters[pResult->m_ClientID]->m_Super = false;
 		pChr->Teams()->SetForceCharacterTeam(pResult->m_ClientID,
 				pChr->m_TeamBeforeSuper);
 	}
@@ -343,11 +345,8 @@ void CGameContext::ConForcePause(IConsole::IResult *pResult, void *pUserData)
 	pPlayer->ForcePause(Seconds);
 }
 
-void CGameContext::VoteMute(const NETADDR *pAddr, int Secs, const char *pDisplayName, int AuthedID)
+bool CGameContext::TryVoteMute(const NETADDR *pAddr, int Secs)
 {
-	char aBuf[128];
-	bool Found = 0;
-
 	// find a matching vote mute for this ip, update expiration time if found
 	for(int i = 0; i < m_NumVoteMutes; i++)
 	{
@@ -355,73 +354,99 @@ void CGameContext::VoteMute(const NETADDR *pAddr, int Secs, const char *pDisplay
 		{
 			m_aVoteMutes[i].m_Expire = Server()->Tick()
 				+ Secs * Server()->TickSpeed();
-			Found = 1;
-			break;
+			return true;
 		}
 	}
 
-	if(!Found) // nothing found so far, find a free slot..
+	// nothing to update create new one
+	if(m_NumVoteMutes < MAX_VOTE_MUTES)
 	{
-		if(m_NumVoteMutes < MAX_VOTE_BANS)
+		m_aVoteMutes[m_NumVoteMutes].m_Addr = *pAddr;
+		m_aVoteMutes[m_NumVoteMutes].m_Expire = Server()->Tick()
+			+ Secs * Server()->TickSpeed();
+		m_NumVoteMutes++;
+		return true;
+	}
+	// no free slot found
+	Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, "votemute", "vote mute array is full");
+	return false;
+}
+
+bool CGameContext::VoteMute(const NETADDR *pAddr, int Secs, const char *pDisplayName, int AuthedID)
+{
+	if (!TryVoteMute(pAddr, Secs))
+		return false;
+
+	if(!pDisplayName)
+		return true;
+
+	char aBuf[128];
+	str_format(aBuf, sizeof aBuf, "'%s' banned '%s' for %d seconds from voting.",
+		Server()->ClientName(AuthedID), pDisplayName, Secs);
+	Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, "votemute", aBuf);
+	return true;
+}
+
+bool CGameContext::VoteUnmute(const NETADDR *pAddr, const char *pDisplayName, int AuthedID)
+{
+	for(int i = 0; i < m_NumVoteMutes; i++)
+	{
+		if(net_addr_comp_noport(&m_aVoteMutes[i].m_Addr, pAddr) == 0)
 		{
-			m_aVoteMutes[m_NumVoteMutes].m_Addr = *pAddr;
-			m_aVoteMutes[m_NumVoteMutes].m_Expire = Server()->Tick()
-				+ Secs * Server()->TickSpeed();
-			m_NumVoteMutes++;
-			Found = 1;
+			m_NumVoteMutes--;
+			m_aVoteMutes[i] = m_aVoteMutes[m_NumVoteMutes];
+			if(pDisplayName)
+			{
+				char aBuf[128];
+				str_format(aBuf, sizeof aBuf, "'%s' unbanned '%s' from voting.",
+						   Server()->ClientName(AuthedID), pDisplayName);
+				Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, "voteunmute", aBuf);
+			}
+			return true;
 		}
 	}
-	if(Found)
+	return false;
+}
+
+bool CGameContext::TryMute(const NETADDR *pAddr, int Secs)
+{
+	// find a matching mute for this ip, update expiration time if found
+	for(int i = 0; i < m_NumMutes; i++)
 	{
-		if(pDisplayName)
+		if(net_addr_comp_noport(&m_aMutes[i].m_Addr, pAddr) == 0)
 		{
-			str_format(aBuf, sizeof aBuf, "'%s' banned '%s' for %d seconds from voting.",
-				Server()->ClientName(AuthedID), pDisplayName, Secs);
-			Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, "votemute", aBuf);
+			m_aMutes[i].m_Expire = Server()->Tick()
+							+ Secs * Server()->TickSpeed();
+			return true;
 		}
 	}
-	else // no free slot found
-		Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, "votemute", "vote mute array is full");
+
+	// nothing to update create new one
+	if(m_NumMutes < MAX_MUTES)
+	{
+		m_aMutes[m_NumMutes].m_Addr = *pAddr;
+		m_aMutes[m_NumMutes].m_Expire = Server()->Tick()
+						+ Secs * Server()->TickSpeed();
+		m_NumMutes++;
+		return true;
+	}
+	// no free slot found
+	Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, "mutes", "mute array is full");
+	return false;
 }
 
 void CGameContext::Mute(const NETADDR *pAddr, int Secs, const char *pDisplayName)
 {
+	if (!TryMute(pAddr, Secs))
+		return;
+
+	if(!pDisplayName)
+		return;
+
 	char aBuf[128];
-	int Found = 0;
-
-	// find a matching mute for this ip, update expiration time if found
-	for (int i = 0; i < m_NumMutes; i++)
-	{
-		if (net_addr_comp_noport(&m_aMutes[i].m_Addr, pAddr) == 0)
-		{
-			m_aMutes[i].m_Expire = Server()->Tick()
-							+ Secs * Server()->TickSpeed();
-			Found = 1;
-		}
-	}
-
-	if (!Found) // nothing found so far, find a free slot..
-	{
-		if (m_NumMutes < MAX_MUTES)
-		{
-			m_aMutes[m_NumMutes].m_Addr = *pAddr;
-			m_aMutes[m_NumMutes].m_Expire = Server()->Tick()
-							+ Secs * Server()->TickSpeed();
-			m_NumMutes++;
-			Found = 1;
-		}
-	}
-	if (Found)
-	{
-		if (pDisplayName)
-		{
-			str_format(aBuf, sizeof aBuf, "'%s' has been muted for %d seconds.",
-					pDisplayName, Secs);
-			SendChat(-1, CHAT_ALL, aBuf);
-		}
-	}
-	else // no free slot found
-		Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, "mutes", "mute array is full");
+	str_format(aBuf, sizeof aBuf, "'%s' has been muted for %d seconds.",
+			pDisplayName, Secs);
+	SendChat(-1, CHAT_ALL, aBuf);
 }
 
 void CGameContext::ConVoteMute(IConsole::IResult *pResult, void *pUserData)
@@ -438,8 +463,66 @@ void CGameContext::ConVoteMute(IConsole::IResult *pResult, void *pUserData)
 	NETADDR Addr;
 	pSelf->Server()->GetClientAddr(Victim, &Addr);
 
-	pSelf->VoteMute(&Addr, clamp(pResult->GetInteger(1), 1, 86400),
-		pSelf->Server()->ClientName(Victim), pResult->m_ClientID);
+	int Seconds = clamp(pResult->GetInteger(1), 1, 86400);
+	bool Found = pSelf->VoteMute(&Addr, Seconds, pSelf->Server()->ClientName(Victim), pResult->m_ClientID);
+
+	if(Found)
+	{
+		char aBuf[128];
+		str_format(aBuf, sizeof aBuf, "'%s' banned '%s' for %d seconds from voting.",
+				   pSelf->Server()->ClientName(pResult->m_ClientID), pSelf->Server()->ClientName(Victim), Seconds);
+		pSelf->SendChat(-1, 0, aBuf);
+	}
+}
+
+void CGameContext::ConVoteUnmute(IConsole::IResult *pResult, void *pUserData)
+{
+	CGameContext *pSelf = (CGameContext *)pUserData;
+	int Victim = pResult->GetVictim();
+
+	if(Victim < 0 || Victim > MAX_CLIENTS || !pSelf->m_apPlayers[Victim])
+	{
+		pSelf->Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, "voteunmute", "Client ID not found");
+		return;
+	}
+
+	NETADDR Addr;
+	pSelf->Server()->GetClientAddr(Victim, &Addr);
+
+	bool Found = pSelf->VoteUnmute(&Addr, pSelf->Server()->ClientName(Victim), pResult->m_ClientID);
+	if(Found)
+	{
+		char aBuf[128];
+		str_format(aBuf, sizeof aBuf, "'%s' unbanned '%s' from voting.",
+				   pSelf->Server()->ClientName(pResult->m_ClientID), pSelf->Server()->ClientName(Victim));
+		pSelf->SendChat(-1, 0, aBuf);
+	}
+}
+
+void CGameContext::ConVoteMutes(IConsole::IResult *pResult, void *pUserData)
+{
+	CGameContext *pSelf = (CGameContext *) pUserData;
+
+	if (pSelf->m_NumVoteMutes <= 0)
+	{
+		// Just to make sure.
+		pSelf->m_NumVoteMutes = 0;
+		pSelf->Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, "votemutes",
+								"There are no active vote mutes.");
+		return;
+	}
+
+	char aIpBuf[64];
+	char aBuf[128];
+	pSelf->Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, "votemutes",
+							"Active vote mutes:");
+	for (int i = 0; i < pSelf->m_NumVoteMutes; i++)
+	{
+		net_addr_str(&pSelf->m_aVoteMutes[i].m_Addr, aIpBuf, sizeof(aIpBuf), false);
+		str_format(aBuf, sizeof aBuf, "%d: \"%s\", %d seconds left", i,
+				aIpBuf, (pSelf->m_aVoteMutes[i].m_Expire - pSelf->Server()->Tick()) / pSelf->Server()->TickSpeed());
+		pSelf->Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, "votemutes", aBuf);
+	}
 }
 
 void CGameContext::ConMute(IConsole::IResult *pResult, void *pUserData)
@@ -523,14 +606,8 @@ void CGameContext::ConMutes(IConsole::IResult *pResult, void *pUserData)
 	for (int i = 0; i < pSelf->m_NumMutes; i++)
 	{
 		net_addr_str(&pSelf->m_aMutes[i].m_Addr, aIpBuf, sizeof(aIpBuf), false);
-		str_format(
-				aBuf,
-				sizeof aBuf,
-				"%d: \"%s\", %d seconds left",
-				i,
-				aIpBuf,
-				(pSelf->m_aMutes[i].m_Expire - pSelf->Server()->Tick())
-				/ pSelf->Server()->TickSpeed());
+		str_format(aBuf, sizeof aBuf, "%d: \"%s\", %d seconds left", i, aIpBuf,
+				(pSelf->m_aMutes[i].m_Expire - pSelf->Server()->Tick()) / pSelf->Server()->TickSpeed());
 		pSelf->Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, "mutes", aBuf);
 	}
 }
@@ -587,7 +664,15 @@ void CGameContext::ConSetDDRTeam(IConsole::IResult *pResult, void *pUserData)
 	if(pController->m_Teams.m_Core.Team(Target) && pController->m_Teams.GetDDRaceState(pSelf->m_apPlayers[Target]) == DDRACE_STARTED)
 		pSelf->m_apPlayers[Target]->KillCharacter(WEAPON_SELF);
 
-	pController->m_Teams.SetForceCharacterTeam(pResult->GetVictim(), Team);
+	pController->m_Teams.SetForceCharacterTeam(Target, Team);
+}
+
+void CGameContext::ConUninvite(IConsole::IResult *pResult, void *pUserData)
+{
+	CGameContext *pSelf = (CGameContext *)pUserData;
+	CGameControllerDDRace *pController = (CGameControllerDDRace *)pSelf->m_pController;
+
+	pController->m_Teams.SetClientInvited(pResult->GetInteger(1), pResult->GetVictim(), false);
 }
 
 void CGameContext::ConFreezeHammer(IConsole::IResult *pResult, void *pUserData)
