@@ -173,6 +173,7 @@ private:
 
 	bool m_SetSavePos;
 	vec2 m_PrevSavePos;
+	bool m_Solo;
 
 public:
 	CGameTeams* Teams();
@@ -257,6 +258,7 @@ public:
 	vec2 m_TeleGunPos;
 	bool m_TeleGunTeleport;
 	bool m_IsBlueTeleGunTeleport;
+	int m_StrongWeakID;
 
 	// Setters/Getters because i don't want to modify vanilla vars access modifiers
 	int GetLastWeapon() { return m_LastWeapon; };
