@@ -168,7 +168,7 @@ void CLaser::DoBounce()
 	}
 
 	CCharacter *pOwnerChar = GameServer()->GetPlayerChar(m_Owner);
-	if (m_Owner >= 0 && m_Energy <= 0 && m_Pos && !m_TeleportCancelled && pOwnerChar && 
+	if (m_Owner >= 0 && m_Energy <= 0 && m_Pos && !m_TeleportCancelled && pOwnerChar &&
 		pOwnerChar->IsAlive() && pOwnerChar->m_HasTeleLaser && m_Type == WEAPON_RIFLE)
 	{
 		vec2 PossiblePos;
@@ -238,6 +238,15 @@ void CLaser::Reset()
 
 void CLaser::Tick()
 {
+	if(g_Config.m_SvDestroyLasersOnDeath && m_Owner >= 0)
+	{
+		CCharacter *pOwnerChar = GameServer()->GetPlayerChar(m_Owner);
+		if(!(pOwnerChar && pOwnerChar->IsAlive()))
+		{
+			Reset();
+		}
+	}
+
 	float Delay;
 	if (m_TuneZone)
 		Delay = GameServer()->TuningList()[m_TuneZone].m_LaserBounceDelay;
