@@ -18,7 +18,6 @@ public:
 		bool Explosive,
 		float Force,
 		int SoundImpact,
-		int Weapon,
 		int Layer = 0,
 		int Number = 0
 	);
@@ -39,7 +38,6 @@ private:
 	int m_Type;
 	//int m_Damage;
 	int m_SoundImpact;
-	int m_Weapon;
 	float m_Force;
 	int m_StartTick;
 	bool m_Explosive;
