@@ -1,10 +1,16 @@
 #ifndef GAME_SERVER_SCORE_H
 #define GAME_SERVER_SCORE_H
 
+#include <memory>
+
 #include "entities/character.h"
 #include "gamecontext.h"
 
-#define NUM_CHECKPOINTS 25
+enum
+{
+	NUM_CHECKPOINTS = 25,
+	TIMESTAMP_STR_LENGTH = 20, // 2019-04-02 19:38:36
+};
 
 class CPlayerData
 {
@@ -34,6 +40,17 @@ public:
 	float m_aBestCpTime[NUM_CHECKPOINTS];
 };
 
+// Watch this: TODO(2019-05-20): Temporary fix for the random maps race
+// condition. See you in ten years.
+class CRandomMapResult
+{
+public:
+	bool m_Done;
+	char m_aMap[64];
+
+	CRandomMapResult() : m_Done(false) {}
+};
+
 class IScore
 {
 	CPlayerData m_aPlayerData[MAX_CLIENTS];
@@ -47,9 +64,9 @@ public:
 	virtual void MapVote(int ClientID, const char *pMapName) = 0;
 	virtual void CheckBirthday(int ClientID) = 0;
 	virtual void LoadScore(int ClientID) = 0;
-	virtual void SaveScore(int ClientID, float Time, float aCpTime[NUM_CHECKPOINTS], bool NotEligible) = 0;
+	virtual void SaveScore(int ClientID, float Time, const char *pTimestamp, float aCpTime[NUM_CHECKPOINTS], bool NotEligible) = 0;
 
-	virtual void SaveTeamScore(int *pClientIDs, unsigned int Size, float Time) = 0;
+	virtual void SaveTeamScore(int *pClientIDs, unsigned int Size, float Time, const char *pTimestamp) = 0;
 
 	virtual void ShowTop5(IConsole::IResult *pResult, int ClientID, void *pUserData, int Debut=1) = 0;
 	virtual void ShowRank(int ClientID, const char *pName, bool Search=false) = 0;
@@ -60,8 +77,8 @@ public:
 	virtual void ShowTopPoints(IConsole::IResult *pResult, int ClientID, void *pUserData, int Debut=1) = 0;
 	virtual void ShowPoints(int ClientID, const char *pName, bool Search=false) = 0;
 
-	virtual void RandomMap(int ClientID, int Stars) = 0;
-	virtual void RandomUnfinishedMap(int ClientID, int Stars) = 0;
+	virtual void RandomMap(std::shared_ptr<CRandomMapResult> *ppResult, int ClientID, int Stars) = 0;
+	virtual void RandomUnfinishedMap(std::shared_ptr<CRandomMapResult> *ppResult, int ClientID, int Stars) = 0;
 
 	virtual void SaveTeam(int Team, const char *pCode, int ClientID, const char *pServer) = 0;
 	virtual void LoadTeam(const char *pCode, int ClientID) = 0;
